Package: ineapir
Title: Obtaining Data Published by the National Statistics Institute
Version: 0.2.6
Authors@R: c(
    person("David", "Crespo", email = "david.crespo.roces@ine.es", role = c("aut", "cre")),
    person("National Statistics Institute, INE", role = c("cph")))
Description: Get open statistical data and metadata disseminated by the
    National Statistics Institute of Spain (INE). The functions return
    data frames with the requested information thanks to calls to
    the 'INE' API <https://www.ine.es/dyngs/DAB/index.htm?cid=1100>.
License: EUPL (== 1.2)
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: httr, jsonlite
Config/Needs/website: rmarkdown
URL: https://github.com/es-ine/ineapir,
        https://es-ine.github.io/ineapir/
BugReports: https://github.com/es-ine/ineapir/issues
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-07-08 09:49:12 UTC; U851427
Author: David Crespo [aut, cre],
  National Statistics Institute, INE [cph]
Maintainer: David Crespo <david.crespo.roces@ine.es>
Repository: CRAN
Date/Publication: 2026-07-08 10:20:02 UTC

# Generated by roxygen2: do not edit by hand

export(get_data_series)
export(get_data_series_filter)
export(get_data_table)
export(get_filter_shortcuts)
export(get_metadata_classifications)
export(get_metadata_operation_table)
export(get_metadata_operations)
export(get_metadata_periodicity)
export(get_metadata_publication_dates)
export(get_metadata_publications)
export(get_metadata_series)
export(get_metadata_series_filter)
export(get_metadata_series_operation)
export(get_metadata_series_table)
export(get_metadata_series_values)
export(get_metadata_series_varval)
export(get_metadata_table_groups)
export(get_metadata_table_values)
export(get_metadata_table_varval)
export(get_metadata_tables_operation)
export(get_metadata_values)
export(get_metadata_variables)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# ineapir <img src="man/figures/hex_logo.png" align="right" width = "120" alt = "ineapir logo"/>

<!-- badges: start -->

[![R-CMD-check](https://github.com/es-ine/ineapir/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/es-ine/ineapir/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

## Overview

ineapir provides a set of functions to obtain open data and metadata
published by the National Statistics Institute of Spain
([INE](https://www.ine.es/en/index.htm)). The data is obtained thanks to
calls to the INE API service which allows access via URL requests to the
available statistical information published by INE.

## Installation

Install the released version of **ineapir** from CRAN:

``` r
install.packages("ineapir")
```

To install the development version of **ineapir** from GitHub:

``` r
remotes::install_github("es-ine/ineapir")
```

Alternatively, you can download the source code as a zip file and then
install it as follows:

``` r
remotes::install_local(path = "path/to/file.zip")
```

## Cheatsheet

<a href="https://raw.githubusercontent.com/es-ine/ineapir/main/man/figures/ineapir.pdf"><img src="man/figures/ineapir_thumbnail.png" width="315" alt = "Cheatsheet"/></a>

## Data request examples

The data is only associated with the series object and these can be
grouped together into statistical tables. The field named ‘*Valor*’ is
the only one that contains data. The rest of the fields are necessary
for the data to be well defined.

### Obtaining data from a table

To get all the data of a table it is necessary to pass the `idTable`
argument, which is the identification code of the table, to the function
`get_data_table()`.

``` r
library(ineapir)

# We use the function get_data_table with the argument idTable
# and the argument tip = 'A' for a more friendly output
table <- get_data_table(idTable = 76125, tip = "A")

# Each row represents a series
table[1,c("COD", "Nombre")]
#>         COD                             Nombre
#> 1 IPC290751 Nacional. Índice general. Índice.

# The Data column contains a data frame for each row with the values 
# of the different periods of each series
head(table$Data[[1]])
#>                           Fecha T3_TipoDato T3_Periodo Anyo   Valor
#> 1 2026-05-01T00:00:00.000+02:00  Definitivo        M05 2026 102.951
#> 2 2026-04-01T00:00:00.000+02:00  Definitivo        M04 2026 102.883
#> 3 2026-03-01T00:00:00.000+01:00  Definitivo        M03 2026 102.440
#> 4 2026-02-01T00:00:00.000+01:00  Definitivo        M02 2026 101.261
#> 5 2026-01-01T00:00:00.000+01:00  Definitivo        M01 2026 100.836
#> 6 2025-12-01T00:00:00.000+01:00  Definitivo        M12 2025 101.289

# We can concatenate all data frames into one using unnest = TRUE
table <- get_data_table(idTable = 76125, tip = "A", unnest = TRUE)
head(table[,c("COD", "Nombre", "Fecha", "Valor")])
#>           COD                             Nombre                         Fecha
#> 1   IPC290751 Nacional. Índice general. Índice.  2026-05-01T00:00:00.000+02:00
#> 1.1 IPC290751 Nacional. Índice general. Índice.  2026-04-01T00:00:00.000+02:00
#> 1.2 IPC290751 Nacional. Índice general. Índice.  2026-03-01T00:00:00.000+01:00
#> 1.3 IPC290751 Nacional. Índice general. Índice.  2026-02-01T00:00:00.000+01:00
#> 1.4 IPC290751 Nacional. Índice general. Índice.  2026-01-01T00:00:00.000+01:00
#> 1.5 IPC290751 Nacional. Índice general. Índice.  2025-12-01T00:00:00.000+01:00
#>       Valor
#> 1   102.951
#> 1.1 102.883
#> 1.2 102.440
#> 1.3 101.261
#> 1.4 100.836
#> 1.5 101.289
```

To get the last n data from a table it is necessary to pass the `nlast`
argument as well.

``` r
# We use the function get_data_table with arguments idTable and nlast
table <- get_data_table(idTable = 76125, nlast = 2)
table[1,c("COD", "Nombre")]
#>         COD                             Nombre
#> 1 IPC290751 Nacional. Índice general. Índice.
head(table$Data[[1]])
#>          Fecha FK_TipoDato FK_Periodo Anyo   Valor Secreto
#> 1 1.777586e+12           1          5 2026 102.951   FALSE
```

### Obtaining data from a series

To get the last data of a series it is necessary to pass the `codSeries`
argument, which is the identification code of the series, to the
function `get_data_series()`.

``` r
# We use the function get_data_series with the argument codSeries
series <- get_data_series(codSeries = "IPC290750", tip = "A")
series$Data
#>                           Fecha T3_TipoDato T3_Periodo Anyo Valor
#> 1 2026-06-01T00:00:00.000+02:00      Avance        M06 2026   3.2
```

To get the last n data from a series it is necessary to pass the `nlast`
argument as well.

``` r
# We use the function get_data_series with arguments codSeries and nlast
series <- get_data_series(codSeries = "IPC290750", tip = "A", nlast = 5)
series$Data
#>                           Fecha T3_TipoDato T3_Periodo Anyo Valor
#> 1 2026-02-01T00:00:00.000+01:00  Definitivo        M02 2026   2.3
#> 2 2026-03-01T00:00:00.000+01:00  Definitivo        M03 2026   3.4
#> 3 2026-04-01T00:00:00.000+02:00  Definitivo        M04 2026   3.2
#> 4 2026-05-01T00:00:00.000+02:00  Definitivo        M05 2026   3.2
#> 5 2026-06-01T00:00:00.000+02:00      Avance        M06 2026   3.2

# Using unnest = TRUE
series <- get_data_series(codSeries = "IPC290750", tip = "A", nlast = 5,
                          unnest = TRUE)
head(series[,c("COD", "Nombre", "Fecha", "Valor")])
#>           COD                                      Nombre
#> 1   IPC290750 Nacional. Índice general. Variación anual. 
#> 1.1 IPC290750 Nacional. Índice general. Variación anual. 
#> 1.2 IPC290750 Nacional. Índice general. Variación anual. 
#> 1.3 IPC290750 Nacional. Índice general. Variación anual. 
#> 1.4 IPC290750 Nacional. Índice general. Variación anual. 
#>                             Fecha Valor
#> 1   2026-02-01T00:00:00.000+01:00   2.3
#> 1.1 2026-03-01T00:00:00.000+01:00   3.4
#> 1.2 2026-04-01T00:00:00.000+02:00   3.2
#> 1.3 2026-05-01T00:00:00.000+02:00   3.2
#> 1.4 2026-06-01T00:00:00.000+02:00   3.2
```

Additionally, it is possible to obtain data from a series between two
dates. The date must have and specific format (*yyyy/mm/dd*). If the end
date is not specified we obtain all the data from the start date.

``` r
# We use the function get_data_series with arguments codSeries, dateStart and dataEnd
series <- get_data_series(codSeries = "IPC290750", dateStart = "2023/01/01", 
                          dateEnd = "2023/04/01")
series$Data
#>          Fecha FK_TipoDato FK_Periodo Anyo Valor Secreto
#> 1 1.672528e+12           1          1 2023   5.9   FALSE
#> 2 1.675206e+12           1          2 2023   6.0   FALSE
#> 3 1.677625e+12           1          3 2023   3.3   FALSE
#> 4 1.680300e+12           1          4 2023   4.1   FALSE
```

## Metadata request examples

Structural metadata are objects that describe both time series and
statistical tables and allow their definition. All these database
objects have an associated identifier that is essential for the correct
use of the service.

### Obtaining statistical operations

The database contains information about all short-term statistical
operations, those with a periodicity for disseminating results of less
than a year, as well as some structural statistical operations. We can
get all the operations using the function `get_metadata_operations()`.

``` r
# We use the function get_metadata_operations
operations <- get_metadata_operations()
head(operations)
#>   Id Cod_IOE                                                 Nombre Codigo
#> 1  4   30147           Estadística de Efectos de Comercio Impagados     EI
#> 2  6   30211                     Índice de Coste Laboral Armonizado   ICLA
#> 3  7   30168 Estadística de Transmisión de Derechos de la Propiedad   ETDP
#> 4 10   30256                                    Indicadores Urbanos     UA
#> 5 13   30219                Estadística del Procedimiento Concursal    EPC
#> 6 14   30182                Índices de Precios del Sector Servicios    IPS
#>                                                                                                     Url
#> 1                                                                                                  <NA>
#> 2                      /dyngs/INEbase/operacion.htm?c=Estadistica_C&cid=1254736053992&idp=1254735976596
#> 3                      /dyngs/INEbase/operacion.htm?c=Estadistica_C&cid=1254736171438&idp=1254735576606
#> 4 https://www.ine.es/dyngs/INEbase/es/operacion.htm?c=Estadistica_C&cid=1254736176957&idp=1254735976608
#> 5                      /dyngs/INEbase/operacion.htm?c=Estadistica_C&cid=1254736177018&idp=1254735576606
#> 6                      /dyngs/INEbase/operacion.htm?c=Estadistica_C&cid=1254736176864&idp=1254735576778
```

An operation can be identify by a numerical code (‘*Id*’), an alphabetic
code (‘*Codigo*’) or by the code of the statistical operation in the
Inventory of Statistical Operations (IOE + ‘*Cod_IOE*’). To obtain
information about only one operation we have to pass the `operation`
argument with one of these codes.

``` r
# We use the function get_metadata_operations with argument operation
operation <- get_metadata_operations(operation = "IPC")
as.data.frame(operation)
#>   Id Cod_IOE                             Nombre Codigo
#> 1 25   30138 Índice de Precios de Consumo (IPC)    IPC
#>                                                                                Url
#> 1 /dyngs/INEbase/operacion.htm?c=Estadistica_C&cid=1254736176802&idp=1254735976607
```

### Obtaining variables

We can get all the variables of the system using the function
`get_metadata_variables()`.

``` r
# We use the function get_metadata_variables
variables <- get_metadata_variables()
head(variables)
#>    Id                           Nombre Codigo
#> 1 349                   Total Nacional    NAC
#> 2 954                            Total       
#> 3  70 Comunidades y Ciudades Autónomas   CCAA
#> 4 516                     Nacionalidad      1
#> 5 955       Cultivos, pastos y huertos       
#> 6 956              SAU y Otras tierras
```

A variable can be identify by a numerical code (‘*Id*’). In addition, if
we pass the `operation` argument we obtain the variables used in an
operation.

``` r
# We use the function get_metadata_variables with argument operation,
# e.g., operation code = 'IPC'
variables <- get_metadata_variables(operation = "IPC")
head(variables)
#>    Id                           Nombre Codigo
#> 1   3                     Tipo de dato       
#> 2  70 Comunidades y Ciudades Autónomas   CCAA
#> 3 115                       Provincias   PROV
#> 4 269           Grupos especiales 2001       
#> 5 270                    Rúbricas 2001       
#> 6 349                   Total Nacional    NAC
```

### Obtaining values

To get all the values that a variable can take it is necessary to pass
the `variable` argument, which is the identifier of the variable, to the
function `get_metadata_values()`.

``` r
# We use the function get_metadata_values with argument variable,
# e.g., id = 3 (variable 'Tipo de dato')
values <- get_metadata_values(variable = 3)
head(values)
#>   Id Fk_Variable                                                   Nombre
#> 1 70           3                                             Datos brutos
#> 2 71           3 Datos corregidos de efectos estacionales y de calendario
#> 3 72           3                                                Dato base
#> 4 73           3                                     Variación trimestral
#> 5 74           3                                          Variación anual
#> 6 75           3                                                    Euros
#>   Codigo
#> 1       
#> 2       
#> 3       
#> 4       
#> 5       
#> 6
```

A value can be identify by a numerical code (‘*Id*’). In addition, if we
pass the `operation` argument as well we obtain the values that the
variable takes in that particular operation.

``` r
# We use the function get_metadata_values with arguments operation and variable,
# e.g., operation code = 'IPC'
values <- get_metadata_values(operation = "IPC", variable = 3)
head(values)
#>   Id Fk_Variable            Nombre Codigo
#> 1 72           3         Dato base       
#> 2 74           3   Variación anual       
#> 3 83           3            Índice       
#> 4 84           3 Variación mensual       
#> 5 85           3       Media anual      M
#> 6 86           3   Variación anual
```

### Obtaining tables

We can get the tables associated with an statistical operation using the
function `get_metadata_tables_operation()`.

``` r
# We use the function get_metadata_tables with argument operation
tables <- get_metadata_tables_operation(operation = "IPC")
head(tables[,c("Id","Nombre")])
#>      Id
#> 1 24077
#> 2 24080
#> 3 35083
#> 4 53458
#> 5 76125
#> 6 76131
#>                                                                         Nombre
#> 1                          Índice general nacional. Series desde enero de 1961
#> 2              Índice nacionales de grupos ECOICOP. Series desde enero de 1993
#> 3              Índices nacionales: Componentes para el análisis de la COVID-19
#> 4                               Índices por comunidades autónomas de subgrupos
#> 5                        Índices nacionales: general y de grupos ECOICOP ver.2
#> 6 Índices nacionales a impuestos constantes: general y de grupos ECOICOP ver.2
```

A table is defined by different groups or selection combo boxes and each
of them by the values that one or several variables take. To obtain the
variables and values present in a table first we have to query the
groups that define the table using the function
`get_metadata_table_groups()`.

``` r
# We use the function get_metadata_table_groups with argument idTable
groups <- get_metadata_table_groups(idTable = 76125)
head(groups)
#>       Id               Nombre
#> 1 155577 Grupos ECOICOP ver.2
#> 2 155578         Tipo de dato
```

Once we have the identification codes of the groups, we can query the
values for an specific group using the function
`get_metadata_table_values()`.

``` r
# We use the function get_metadata_table_values with arguments idTable and idGroup
values <- get_metadata_table_values(idTable = 76125, idGroup = 155577)
head(values, 4)
#>       Id Fk_Variable                             Nombre Codigo
#> 1 304092         762                     Índice general     00
#> 2 304093         762 Alimentos y bebidas no alcohólicas     01
#> 3 418050         762       Bebidas alcohólicas y tabaco     02
#> 4 304095         762                  Vestido y calzado     03
#>   FK_JerarquiaPadres
#> 1               NULL
#> 2             304092
#> 3             304092
#> 4             304092
```

Alternatively, we can use the `get_metadata_table_varval()` function to
get the variables and values present in a table.

``` r
# Using the function get_metadata_table_varval
values <- get_metadata_table_varval(idTable = 76125)
head(values, 4)
#>       Id Fk_Variable                             Nombre Codigo
#> 1 304092         762                     Índice general     00
#> 2 304093         762 Alimentos y bebidas no alcohólicas     01
#> 3 418050         762       Bebidas alcohólicas y tabaco     02
#> 4 304095         762                  Vestido y calzado     03
```

### Obtaining series

The data is only associated with the series object. To obtain
information about a particular series it is necessary to pass the
`codSeries` argument, which is the identification code of the series, to
the function `get_metadata_series()`.

``` r
# We use the function get_metadata_series with argument codSeries
series <- get_metadata_series(codSeries = "IPC290750")
as.data.frame(series)
#>       Id       COD FK_Operacion                                      Nombre
#> 1 290750 IPC290750           25 Nacional. Índice general. Variación anual. 
#>   Decimales FK_Periodicidad FK_Publicacion FK_Clasificacion FK_Escala FK_Unidad
#> 1         1               1              8              120         1       135
```

To get the values and variables that define a series it is necessary to
pass the `codSeries` argument as well.

``` r
# We use the function get_metadata_series_values with argument codSeries
values <- get_metadata_series_values(codSeries = "IPC290750")
head(values)
#>       Id Fk_Variable          Nombre Codigo
#> 1  16473         349        Nacional     00
#> 2 304092         762  Índice general     00
#> 3     74           3 Variación anual      2
```

To get all the series that define a table it is necessary to pass the
`idTable` argument, which is the identification code of the table, to
the function `get_metadata_series_table()`.

``` r
# We use the function get_metadata_series_table with argument idTable
series <- get_metadata_series_table(idTable = 76125)
head(series[,c("COD", "Nombre")], 4)
#>         COD                                                    Nombre
#> 1 IPC290751                        Nacional. Índice general. Índice. 
#> 2 IPC290752             Nacional. Índice general. Variación mensual. 
#> 3 IPC290750               Nacional. Índice general. Variación anual. 
#> 4 IPC290753 Nacional. Índice general. Variación en lo que va de año.
```

<!-- README.md is generated from README.Rmd. Please edit that file -->

# ineapir <img src="man/figures/hex_logo.png" align="right" width = "120" alt = "ineapir logo"/>

<!-- badges: start -->

[![R-CMD-check](https://github.com/es-ine/ineapir/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/es-ine/ineapir/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

## Overview

ineapir provides a set of functions to obtain open data and metadata
published by the National Statistics Institute of Spain
([INE](https://www.ine.es/en/index.htm)). The data is obtained thanks to
calls to the INE API service which allows access via URL requests to the
available statistical information published by INE.

## Installation

Install the released version of **ineapir** from CRAN:

``` r
install.packages("ineapir")
```

To install the development version of **ineapir** from GitHub:

``` r
remotes::install_github("es-ine/ineapir")
```

Alternatively, you can download the source code as a zip file and then
install it as follows:

``` r
remotes::install_local(path = "path/to/file.zip")
```

## Cheatsheet

<a href="https://raw.githubusercontent.com/es-ine/ineapir/main/man/figures/ineapir.pdf"><img src="man/figures/ineapir_thumbnail.png" width="315" alt = "Cheatsheet"/></a>

## Data request examples

The data is only associated with the series object and these can be
grouped together into statistical tables. The field named ‘*Valor*’ is
the only one that contains data. The rest of the fields are necessary
for the data to be well defined.

### Obtaining data from a table

To get all the data of a table it is necessary to pass the `idTable`
argument, which is the identification code of the table, to the function
`get_data_table()`.

``` r
library(ineapir)

# We use the function get_data_table with the argument idTable
# and the argument tip = 'A' for a more friendly output
table <- get_data_table(idTable = 50902, tip = "A")

# Each row represents a series
table[1,c("COD", "Nombre")]
#>         COD                                   Nombre
#> 1 IPC251852 Total Nacional. Índice general. Índice.

# The Data column contains a data frame for each row with the values 
# of the different periods of each series
head(table$Data[[1]])
#>                           Fecha T3_TipoDato T3_Periodo Anyo   Valor
#> 1 2025-07-01T00:00:00.000+02:00  Definitivo        M07 2025 118.777
#> 2 2025-06-01T00:00:00.000+02:00  Definitivo        M06 2025 118.867
#> 3 2025-05-01T00:00:00.000+02:00  Definitivo        M05 2025 118.077
#> 4 2025-04-01T00:00:00.000+02:00  Definitivo        M04 2025 117.997
#> 5 2025-03-01T00:00:00.000+01:00  Definitivo        M03 2025 117.260
#> 6 2025-02-01T00:00:00.000+01:00  Definitivo        M02 2025 117.191

# We can concatenate all data frames into one using unnest = TRUE
table <- get_data_table(idTable = 50902, tip = "A", unnest = TRUE)
head(table[,c("COD", "Nombre", "Fecha", "Valor")])
#>           COD                                   Nombre
#> 1   IPC251852 Total Nacional. Índice general. Índice. 
#> 1.1 IPC251852 Total Nacional. Índice general. Índice. 
#> 1.2 IPC251852 Total Nacional. Índice general. Índice. 
#> 1.3 IPC251852 Total Nacional. Índice general. Índice. 
#> 1.4 IPC251852 Total Nacional. Índice general. Índice. 
#> 1.5 IPC251852 Total Nacional. Índice general. Índice. 
#>                             Fecha   Valor
#> 1   2025-07-01T00:00:00.000+02:00 118.777
#> 1.1 2025-06-01T00:00:00.000+02:00 118.867
#> 1.2 2025-05-01T00:00:00.000+02:00 118.077
#> 1.3 2025-04-01T00:00:00.000+02:00 117.997
#> 1.4 2025-03-01T00:00:00.000+01:00 117.260
#> 1.5 2025-02-01T00:00:00.000+01:00 117.191
```

To get the last n data from a table it is necessary to pass the `nlast`
argument as well.

``` r
# We use the function get_data_table with arguments idTable and nlast
table <- get_data_table(idTable = 50902, nlast = 2)
table[1,c("COD", "Nombre")]
#>         COD                                   Nombre
#> 1 IPC251852 Total Nacional. Índice general. Índice.
head(table$Data[[1]])
#>          Fecha FK_TipoDato FK_Periodo Anyo   Valor Secreto
#> 1 1.751321e+12           1          7 2025 118.777   FALSE
```

### Obtaining data from a series

To get the last data of a series it is necessary to pass the `codSeries`
argument, which is the identification code of the series, to the
function `get_data_series()`.

``` r
# We use the function get_data_series with the argument codSeries
series <- get_data_series(codSeries = "IPC251856", tip = "A")
series$Data
#>                           Fecha T3_TipoDato T3_Periodo Anyo Valor
#> 1 2025-08-01T00:00:00.000+02:00      Avance        M08 2025   2.7
```

To get the last n data from a series it is necessary to pass the `nlast`
argument as well.

``` r
# We use the function get_data_series with arguments codSeries and nlast
series <- get_data_series(codSeries = "IPC251856", tip = "A", nlast = 5)
series$Data
#>                           Fecha T3_TipoDato T3_Periodo Anyo Valor
#> 1 2025-04-01T00:00:00.000+02:00  Definitivo        M04 2025   2.2
#> 2 2025-05-01T00:00:00.000+02:00  Definitivo        M05 2025   2.0
#> 3 2025-06-01T00:00:00.000+02:00  Definitivo        M06 2025   2.3
#> 4 2025-07-01T00:00:00.000+02:00  Definitivo        M07 2025   2.7
#> 5 2025-08-01T00:00:00.000+02:00      Avance        M08 2025   2.7

# Using unnest = TRUE
series <- get_data_series(codSeries = "IPC251856", tip = "A", nlast = 5,
                          unnest = TRUE)
head(series[,c("COD", "Nombre", "Fecha", "Valor")])
#>           COD                                            Nombre
#> 1   IPC251856 Total Nacional. Índice general. Variación anual. 
#> 1.1 IPC251856 Total Nacional. Índice general. Variación anual. 
#> 1.2 IPC251856 Total Nacional. Índice general. Variación anual. 
#> 1.3 IPC251856 Total Nacional. Índice general. Variación anual. 
#> 1.4 IPC251856 Total Nacional. Índice general. Variación anual. 
#>                             Fecha Valor
#> 1   2025-04-01T00:00:00.000+02:00   2.2
#> 1.1 2025-05-01T00:00:00.000+02:00   2.0
#> 1.2 2025-06-01T00:00:00.000+02:00   2.3
#> 1.3 2025-07-01T00:00:00.000+02:00   2.7
#> 1.4 2025-08-01T00:00:00.000+02:00   2.7
```

Additionally, it is possible to obtain data from a series between two
dates. The date must have and specific format (*yyyy/mm/dd*). If the end
date is not specified we obtain all the data from the start date.

``` r
# We use the function get_data_series with arguments codSeries, dateStart and dataEnd
series <- get_data_series(codSeries = "IPC251856", dateStart = "2023/01/01", 
                          dateEnd = "2023/04/01")
series$Data
#>          Fecha FK_TipoDato FK_Periodo Anyo Valor Secreto
#> 1 1.672528e+12           1          1 2023   5.9   FALSE
#> 2 1.675206e+12           1          2 2023   6.0   FALSE
#> 3 1.677625e+12           1          3 2023   3.3   FALSE
#> 4 1.680300e+12           1          4 2023   4.1   FALSE
```

## Metadata request examples

Structural metadata are objects that describe both time series and
statistical tables and allow their definition. All these database
objects have an associated identifier that is essential for the correct
use of the service.

### Obtaining statistical operations

The database contains information about all short-term statistical
operations, those with a periodicity for disseminating results of less
than a year, as well as some structural statistical operations. We can
get all the operations using the function `get_metadata_operations()`.

``` r
# We use the function get_metadata_operations
operations <- get_metadata_operations()
head(operations)
#>   Id Cod_IOE                                                 Nombre Codigo
#> 1  4   30147           Estadística de Efectos de Comercio Impagados     EI
#> 2  6   30211                     Índice de Coste Laboral Armonizado   ICLA
#> 3  7   30168 Estadística de Transmisión de Derechos de la Propiedad   ETDP
#> 4 10   30256                                    Indicadores Urbanos     UA
#> 5 13   30219                Estadística del Procedimiento Concursal    EPC
#> 6 14   30182                Índices de Precios del Sector Servicios    IPS
#>                                                                                                     Url
#> 1                                                                                                  <NA>
#> 2                                                                                                  <NA>
#> 3                                                                                                  <NA>
#> 4 https://www.ine.es/dyngs/INEbase/es/operacion.htm?c=Estadistica_C&cid=1254736176957&idp=1254735976608
#> 5                                                                                                  <NA>
#> 6                                                                                                  <NA>
```

An operation can be identify by a numerical code (‘*Id*’), an alphabetic
code (‘*Codigo*’) or by the code of the statistical operation in the
Inventory of Statistical Operations (IOE + ‘*Cod_IOE*’). To obtain
information about only one operation we have to pass the `operation`
argument with one of these codes.

``` r
# We use the function get_metadata_operations with argument operation
operation <- get_metadata_operations(operation = "IPC")
as.data.frame(operation)
#>   Id Cod_IOE                             Nombre Codigo
#> 1 25   30138 Índice de Precios de Consumo (IPC)    IPC
```

### Obtaining variables

We can get all the variables of the system using the function
`get_metadata_variables()`.

``` r
# We use the function get_metadata_variables
variables <- get_metadata_variables()
head(variables)
#>    Id                           Nombre Codigo
#> 1 349            Totales Territoriales    NAC
#> 2 954                            Total       
#> 3  70 Comunidades y Ciudades Autónomas   CCAA
#> 4 516                     Nacionalidad      1
#> 5 955       Cultivos, pastos y huertos       
#> 6 956              SAU y Otras tierras
```

A variable can be identify by a numerical code (‘*Id*’). In addition, if
we pass the `operation` argument we obtain the variables used in an
operation.

``` r
# We use the function get_metadata_variables with argument operation,
# e.g., operation code = 'IPC'
variables <- get_metadata_variables(operation = "IPC")
head(variables)
#>    Id                           Nombre Codigo
#> 1   3                     Tipo de dato       
#> 2  70 Comunidades y Ciudades Autónomas   CCAA
#> 3 115                       Provincias   PROV
#> 4 269           Grupos especiales 2001       
#> 5 270                    Rúbricas 2001       
#> 6 349            Totales Territoriales    NAC
```

### Obtaining values

To get all the values that a variable can take it is necessary to pass
the `variable` argument, which is the identifier of the variable, to the
function `get_metadata_values()`.

``` r
# We use the function get_metadata_values with argument variable,
# e.g., id = 3 (variable 'Tipo de dato')
values <- get_metadata_values(variable = 3)
head(values)
#>   Id Fk_Variable                                                   Nombre
#> 1 70           3                                             Datos brutos
#> 2 71           3 Datos corregidos de efectos estacionales y de calendario
#> 3 72           3                                                Dato base
#> 4 73           3                                     Variación trimestral
#> 5 74           3                                          Variación anual
#> 6 75           3                                                    Euros
#>   Codigo
#> 1       
#> 2       
#> 3       
#> 4       
#> 5       
#> 6
```

A value can be identify by a numerical code (‘*Id*’). In addition, if we
pass the `operation` argument as well we obtain the values that the
variable takes in that particular operation.

``` r
# We use the function get_metadata_values with arguments operation and variable,
# e.g., operation code = 'IPC'
values <- get_metadata_values(operation = "IPC", variable = 3)
head(values)
#>   Id Fk_Variable            Nombre Codigo
#> 1 72           3         Dato base       
#> 2 74           3   Variación anual       
#> 3 83           3            Índice       
#> 4 84           3 Variación mensual       
#> 5 85           3       Media anual      M
#> 6 86           3   Variación anual
```

### Obtaining tables

We can get the tables associated with an statistical operation using the
function `get_metadata_tables_operation()`.

``` r
# We use the function get_metadata_tables with argument operation
tables <- get_metadata_tables_operation(operation = "IPC")
head(tables[,c("Id","Nombre")])
#>      Id
#> 1 24077
#> 2 25331
#> 3 35083
#> 4 50902
#> 5 50908
#> 6 50911
#>                                                                      Nombre
#> 1                       Índice general nacional. Series desde enero de 1961
#> 2                                Ponderaciones: general y de grupos ECOICOP
#> 3           Índices nacionales: Componentes para el análisis de la COVID-19
#> 4                           Índices nacionales: general y de grupos ECOICOP
#> 5    Índices nacionales a impuestos constantes: general y de grupos ECOICOP
#> 6 Tasa de variacion del índice general nacional. Series desde enero de 1961
```

A table is defined by different groups or selection combo boxes and each
of them by the values that one or several variables take. To obtain the
variables and values present in a table first we have to query the
groups that define the table using the function
`get_metadata_table_groups()`.

``` r
# We use the function get_metadata_table_groups with argument idTable
groups <- get_metadata_table_groups(idTable = 50902)
head(groups)
#>       Id         Nombre
#> 1 110889 Grupos ECOICOP
#> 2 110890   Tipo de dato
```

Once we have the identification codes of the groups, we can query the
values for an specific group using the function
`get_metadata_table_values()`.

``` r
# We use the function get_metadata_table_values with arguments idTable and idGroup
values <- get_metadata_table_values(idTable = 50902, idGroup = 110889)
head(values, 4)
#>       Id Fk_Variable                             Nombre Codigo
#> 1 304092         762                     Índice general     00
#> 2 304093         762 Alimentos y bebidas no alcohólicas     01
#> 3 304094         762       Bebidas alcohólicas y tabaco     02
#> 4 304095         762                  Vestido y calzado     03
#>   FK_JerarquiaPadres
#> 1               NULL
#> 2             304092
#> 3             304092
#> 4             304092
```

Alternatively, we can use the `get_metadata_table_varval()` function to
get the variables and values present in a table.

``` r
# Using the function get_metadata_table_varval
values <- get_metadata_table_varval(idTable = 50902)
head(values, 4)
#>       Id Fk_Variable                             Nombre Codigo
#> 1 304092         762                     Índice general     00
#> 2 304093         762 Alimentos y bebidas no alcohólicas     01
#> 3 304094         762       Bebidas alcohólicas y tabaco     02
#> 4 304095         762                  Vestido y calzado     03
```

### Obtaining series

The data is only associated with the series object. To obtain
information about a particular series it is necessary to pass the
`codSeries` argument, which is the identification code of the series, to
the function `get_metadata_series()`.

``` r
# We use the function get_metadata_series with argument codSeries
series <- get_metadata_series(codSeries = "IPC251856")
as.data.frame(series)
#>       Id       COD FK_Operacion
#> 1 251856 IPC251856           25
#>                                              Nombre Decimales FK_Periodicidad
#> 1 Total Nacional. Índice general. Variación anual.          1               1
#>   FK_Publicacion FK_Clasificacion FK_Escala FK_Unidad
#> 1              8               90         1       135
```

To get the values and variables that define a series it is necessary to
pass the `codSeries` argument as well.

``` r
# We use the function get_metadata_series_values with argument codSeries
values <- get_metadata_series_values(codSeries = "IPC251856")
head(values)
#>       Id Fk_Variable          Nombre Codigo
#> 1  16473         349  Total Nacional     00
#> 2 304092         762  Índice general     00
#> 3     74           3 Variación anual
```

To get all the series that define a table it is necessary to pass the
`idTable` argument, which is the identification code of the table, to
the function `get_metadata_series_table()`.

``` r
# We use the function get_metadata_series_table with argument idTable
series <- get_metadata_series_table(idTable = 50902)
head(series[,c("COD", "Nombre")], 4)
#>         COD                                                          Nombre
#> 1 IPC251852                        Total Nacional. Índice general. Índice. 
#> 2 IPC251855             Total Nacional. Índice general. Variación mensual. 
#> 3 IPC251856               Total Nacional. Índice general. Variación anual. 
#> 4 IPC251858 Total Nacional. Índice general. Variación en lo que va de año.
```

#' Get data from a specific series
#'
#' @description Retrieve data from series published by INE calling the API
#'
#' @param codSeries (string): Code of the series. For further information about
#' codes click this [link](https://es-ine.github.io/ineapir/articles/identify_codes.html).
#' @param nlast (int): number of periods to retrieve. By default is set to 1 period.
#' @param dateStart (string): the initial date of the requested data. The required
#' format is yyyy/mm/dd. Additionally, dateStart can be a vector of dates, where
#' each date represents the start date of individual ranges where the end date should be found
#' at the same position in the dateEnd vector. If dateStart and dateEnd are equal,
#' the specified dates are retrieved. If no end date is entered,
#' all dates will be queried, from the corresponding start date to the last available period.
#' @param dateEnd (string): the end date of the requested data. The required
#' format is yyyy/mm/dd. Additionally, dateEnd can be a vector of dates, where
#' each date represents the end date of individual ranges where the initial date should be found
#' at the same position in the dateStart vector. The length of the dateEnd vector
#' must be less than or equal to the length of the dateStart vector.
#' @param det (int): level of detail. Valid values: 0, 1 or 2.
#' @param tip (string): set to 'A' for friendly output (e.g. readable dates),
#' set to 'M' to include metadata or set to 'AM' for both.
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param validate (logical): validate input parameters.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#' @param unnest (logical): set to TRUE to obtain a single data frame of data
#'
#' @return Data frame with data of a series according to the code specified in the function
#'
#' @examplesIf interactive()
#  # Get the last five periods of time series with code "IPC290750"
#' df <- get_data_series(codSeries = "IPC290750", nlast = 5)
#' head(df)
#'
#' # Get data for an open range date
#' df <- get_data_series(codSeries = "IPC290750", dateStart = "2024/01/01")
#' head(df)
#'
#' # Get data for a single range data
#' df <- get_data_series(codSeries = "IPC290750",
#' dateStart = "2023/01/01", dateEnd = "2023/05/01")
#' head(df)
#'
#' # Get data for specific dates
#' df <- get_data_series(codSeries = "IPC290750",
#' dateStart = c("2023/01/01","2024/01/01"),
#' dateEnd = c("2023/01/01","2024/01/01"))
#' head(df)
#'
#' # Get data for multiple date ranges
#' df <- get_data_series(codSeries = "IPC290750",
#' dateStart = c("2023/01/01","2024/01/01"),
#' dateEnd = c("2023/03/01","2024/03/01"))
#' head(df)
#'
#' @export
get_data_series <- function(codSeries = NULL, nlast = 1, dateStart = NULL, dateEnd = NULL, det = 0, tip = NULL, lang = "ES", validate = TRUE, verbose = FALSE, unnest = FALSE){

  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, list(fun = "DATOS_SERIE"))
  definition <- append(definition, list(input = codSeries))
  definition <- append(definition, list(tag = "codSeries"))

  # List of parameters to call the API
  parameters <- list()
  parameters <- append(parameters, if(is.null(dateStart) & is.null(dateEnd)) list(date = dateStart) else list(date = list(dateStart = dateStart, dateEnd = dateEnd)))
  parameters <- append(parameters, if(is.null(dateStart) & is.null(dateEnd)) list(nult = nlast) else list(nult = NULL))
  parameters <- append(parameters, list(det = det))
  parameters <- append(parameters, list(tip = tip))

  # List of addons
  addons <- list(validate = validate, verbose = verbose, unnest = unnest)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Build the complete URL to call the API
  url <- get_url(request)

  # Obtain the retrieved data calling the API
  data <- get_api_data(url, request)

  return(data)
}

#' Get data from series for a specific operation given a filter
#'
#' @param operation (string): Code of the operation. To obtain a list of
#' available operations see [get_metadata_operations()].
#' @param filter (list): list of variables and values.
#' ### Filtering data from series
#' When we request data from series there is the possibility of filtering data
#' on the fly using metadata information about the variables and their values
#' that define the series. To get variables for a given operation see
#' [get_metadata_variables()] and to get values for a specific variable see
#' [get_metadata_values()]. See also [get_metadata_series_varval()] to get all the values at once.
#'
#' #### Filter format
#' The format is `list(id_variable1 = id_value1, id_variable2 = id_value2)`.
#' Besides:
#' - A variable can take more than one value: `list(id_variable1 = c(id_value11, id_value12), id_variable2 = id_value2)`.
#' - A variable can take a empty character "" to get all its possible values: `list(id_variable1 = id_value1, id_variable2 = "")`.
#'
#' #### Using shortcuts
#' Additionally, shortcuts can be used to filter. They simplify the filtering
#' approach by using standardized names for variable IDs and therefore simplify
#' their use. The format is:
#' `list(shortcut_variable1 = name1, shortcut_variable2 = name2)`. Besides,
#' the *values* wrapper can also be used: `list(values = c(name1, name2)`.
#' To see a list of all available shortcuts, see [get_filter_shortcuts()] function.
#' Let’s also remark that for better performance is recommended to use numeric codes.
#' @param periodicity (int): id of the periodicity of the series. Common periodicities:
#' 1 (monthly), 3 (quarterly), 6 (bi-annual), 12 (annual). To obtain a list
#' of periodicities see [get_metadata_periodicity()].
#' @param nlast (int): number of periods to retrieve. By default is set to 1 period.
#' @param dateStart (string): the initial date of the requested data. The required
#' format is yyyy/mm/dd. Additionally, dateStart can be a vector of dates, where
#' each date represents the start date of individual ranges where the end date should be found
#' at the same position in the dateEnd vector. If dateStart and dateEnd are equal,
#' the specified dates are retrieved. If no end date is entered,
#' all dates will be queried, from the corresponding start date to the last available period.
#' @param dateEnd (string): the end date of the requested data. The required
#' format is yyyy/mm/dd. Additionally, dateEnd can be a vector of dates, where
#' each date represents the end date of individual ranges where the initial date should be found
#' at the same position in the dateStart vector. The length of the dateEnd vector
#' must be less than or equal to the length of the dateStart vector.
#' @param det (int): level of detail. Valid values: 0, 1 or 2.
#' @param tip (string): set to 'A' for friendly output (e.g. readable dates),
#' set to 'M' to include metadata or set to 'AM' for both.
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param page (int): page number. The retrieved result of the query is paginated. Default value is set to 1.
#' @param validate (logical): validate the input parameters. A FALSE value means fewer API calls.
#' Therefore, it is recommended to set it to FALSE when there is no doubt about the validity of the input parameters, including the filter.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#' @param unnest (logical): obtain a single data frame of data
#'
#' @return Data frame with data of series according to the operation and filter specified in the function
#'
#' @examplesIf interactive()
#' # Get last five periods and filter data of time series from "IPC" operation
#' filter <- list("115"= "28", "3" = "84", "762" = "304092")
#' df <- get_data_series_filter(operation = "IPC", periodicity = 1,
#' nlast = 5, unnest = TRUE, validate = FALSE, filter = filter)
#' head(df)
#'
#' @export
get_data_series_filter <- function(operation = NULL, filter = NULL, periodicity = NULL, nlast = 1, dateStart = NULL, dateEnd = NULL, det = 0, tip = NULL, lang = "ES", page = 1, validate = TRUE, verbose = FALSE, unnest = FALSE){

  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, list(fun = "DATOS_METADATAOPERACION"))
  definition <- append(definition, list(input = operation))
  definition <- append(definition, list(tag = "operation"))

  # List of parameters to call the API
  parameters <- list()
  parameters <- append(parameters, list(filter = list(operation = operation, filter = filter)))
  parameters <- append(parameters, list(p = list(operation = operation, p = periodicity)))
  parameters <- append(parameters, if(is.null(dateStart) & is.null(dateEnd)) list(date = dateStart) else list(date = list(dateStart = dateStart, dateEnd = dateEnd)))
  parameters <- append(parameters, if(is.null(dateStart) & is.null(dateEnd)) list(nult = nlast) else list(nult = NULL))
  #parameters <- append(parameters, list(nult = nlast))
  parameters <- append(parameters, list(det = det))
  parameters <- append(parameters, list(tip = tip))
  parameters <- append(parameters, if(page == 0) list(page = 1) else list(page = page))

  # List of addons
  addons <- list(validate = validate, verbose = verbose, unnest = unnest)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Build the complete URL to call the API
  url <- get_url(request)

  # Obtain the data retrieved calling the API
  data <- get_api_data(url, request)

  return(data)
}

test_that("test unnest", {
  skip_on_cran()
  expect_s3_class(get_data_series(codSeries = "IPC290750", unnest = TRUE, nlast = 5), "data.frame")
})

test_that("test det", {
  skip_on_cran()
  expect_s3_class(get_data_series(codSeries = "IPC290750", unnest = TRUE, nlast = 5, det = 2), "data.frame")
})

test_that("test date", {
  skip_on_cran()
  expect_s3_class(get_data_series(codSeries = "IPC290750", unnest = TRUE, nlast = 5,
                                  dateStart = c("2023/01/01","2024/01/01"),
                                  dateEnd = c("2023/01/01","2024/01/01")), "data.frame")
})

test_that("test filter", {
  skip_on_cran()
  expect_s3_class(get_data_series_filter(operation = "IPC", periodicity = 1, nlast = 5, unnest = TRUE, validate = FALSE,
                                         filter = list("115"= "28", "3" = "84", "762" = "304092")), "data.frame")

})

# URL root to call the API
API_URL = "https://servicios.ine.es/wstempus/js"

# API version
API_version <- "3"

# Number of rows per page
page_lenght = 500

# column names for different API version
column_names <- list("3" = list("variable.code" = list("es" = "Variable.Codigo"),
                                "variable.id" = list("es" = "Variable.Id"),
                                "variable.fk" = list("es" = "Fk_Variable"),
                                "variable.name" = list("es" = "Variable.Nombre"),
                                "value.code" = list("es" = "Codigo"),
                                "value.id" = list("es" = "Id"),
                                "value.name" = list("es" = "Nombre"),
                                "id" = list("es" = "Id"),
                                "ioe" = list("es" = "Cod_IOE"),
                                "codigo" = list("es" = "Codigo"),
                                "jerarquia.fk" = list("es" = "FK_JerarquiaPadres"),
                                "jerarquia" = list("es" = "JerarquiaPadres")),
                     "4" = list("variable.code" = list("es" = "Variable.Codigo"),
                                "variable.id" = list("es" = "Variable.Id"),
                                "variable.fk" = list("es" = "FK_Variable"),
                                "variable.name" = list("es" = "Variable.Nombre"),
                                "value.code" = list("es" = "Codigo"),
                                "value.id" = list("es" = "Id"),
                                "value.name" = list("es" = "Nombre"),
                                "id" = list("es" = "Id"),
                                "ioe" = list("es" = "Cod_IOE"),
                                "codigo" = list("es" = "Codigo"),
                                "jerarquia.fk" = list("es" = "FK_JerarquiaPadres"),
                                "jerarquia" = list("es" = "JerarquiaPadres")))

# Shortcuts used in filters
shortcuts_filter <- list(nac = "349",                 # national
                         prov = "115" ,                      # provinces
                         ccaa = "70",                        # ccaa
                         mun = "19",                         # municipalities
                         isla = "20", island = "20",         # islas
                         grupo = "762", group = "762",       # cpi groups
                         subgrupo = "763", subgroup = "763", # cpi subgroups
                         clase = "764", class = "764",       # cpi class
                         subclase = "765", subclass = "765", # cpi subclass
                         rubrica = "270",                    # cpi headings
                         heading = "270",                    # cpi headings
                         grupoespecial = "269",              # cpi special groups
                         specialgroup = "269",               # cpi special groups
                         tipodato = "3", datatype = "3",     # type of data
                         sexo = "18", sex = "18",            # sex
                         edad1 = "355", age1 = "355",        # simple age
                         edadt = "356", aget = "356",        # total age
                         edadg = "360", ageg = "360",        # age groups
                         edads = "357", ages = "357",        # semi-age intervals
                         edad = c("355", "356", "360", "357"),
                         age = c("355", "356", "360", "357"),
                         nacionalidad = "141", nationality = "141",
                         generacion = "612", generation = "612",  # generation ages
                         paisnacimiento = c("431", "432"),   # country of birth
                         birthcountry = c("431", "432"),     # country of birth
                         lugarnacimiento = c("97"),          # place of birth
                         birthplace = c("97"),               # place of birth
                         efectoscorr = "544",                # correction of effects
                         effectscorr = "544"                 # correction of effects
)

shortcuts_filter_comments <- list(nac = list(comment ="National", lang = "ALL"),
                                  prov = list(comment ="Provinces", lang = "ALL") ,
                                  ccaa = list(comment ="Autonomous Communities", lang = "ALL"),
                                  mun = list(comment ="Municipalities", lang = "ALL"),
                                  isla = list(comment ="Islands", lang = "ES"),
                                  island = list(comment ="Islands", lang = "EN"),
                                  grupo = list(comment ="CPI groups", lang = "ES"),
                                  group = list(comment ="CPI groups", lang = "EN"),
                                  subgrupo = list(comment ="CPI subgroups", lang = "ES"),
                                  subgroup = list(comment ="CPI subgroups", lang = "EN"),
                                  clase = list(comment ="764", lang = "ES"),
                                  class = list(comment ="764", lang = "EN"),
                                  subclase = list(comment ="CPI class", lang = "ES"),
                                  subclass = list(comment ="CPI class", lang = "EN"),
                                  rubrica = list(comment ="CPI headings", lang = "ES"),
                                  heading = list(comment ="CPI headings", lang = "EN"),
                                  grupoespecial = list(comment ="CPI special groups", lang = "ES"),
                                  specialgroup = list(comment ="CPI special groups", lang = "EN"),
                                  tipodato = list(comment ="Type of data", lang = "ES"),
                                  datatype = list(comment ="Type of data", lang = "EN"),
                                  sexo = list(comment ="Sex", lang = "ES"),
                                  sex = list(comment ="Sex", lang = "EN"),
                                  edad1 = list(comment ="Simple age", lang = "ES"),
                                  age1 = list(comment ="Simple age", lang = "EN"),
                                  edadt = list(comment ="Age totals", lang = "ES"),
                                  aget = list(comment ="Age totals", lang = "EN"),
                                  edadg = list(comment ="Age groups", lang = "ES"),
                                  ageg = list(comment ="Age groups", lang = "EN"),
                                  edads = list(comment ="Age semi-intervals", lang = "ES"),
                                  ages = list(comment ="Age semi-intervals", lang = "EN"),
                                  edad = list(comment ="Age wrapper", lang = "ES"),
                                  age = list(comment ="Age wrapper", lang = "EN"),
                                  nacionalidad = list(comment ="Nationality", lang = "ES"),
                                  nationality = list(comment ="Nationality", lang = "EN"),
                                  generacion = list(comment ="Generation/ages", lang = "ES"),
                                  generation = list(comment ="Generation/ages", lang = "EN"),
                                  paisnacimiento = list(comment ="Country of birth", lang = "ES"),
                                  birthcountry = list(comment ="Country of birth", lang = "EN"),
                                  lugarnacimiento = list(comment ="Place of birth", lang = "ES"),
                                  birthplace = list(comment ="Place of birth", lang = "EN"),
                                  efectoscorr = list(comment ="Correction of effects", lang = "ES"),
                                  effectscorr = list(comment ="Correction of effects", lang = "EN")
)

shortcut_wrapper <- c("values")

# Function to retrieve data from the aPI
get_api_data <- function(url, request){

  result <- NULL

  # Initiate a call to the aPI
  tryCatch(
    {
      # if the url is too large we used the method POST
      if(nchar(url$complete) > 2000){

        if(request$addons$verbose){
          cat(sprintf("- API URL: %s\n", url$endpointpar))
        }

        response <- httr::VERB("POST",
                               url = url$endpoint,
                               query = url$parameters,
                               body = url$filter,
                               encode = "form",
                               httr::content_type("application/x-www-form-urlencoded"),
                               httr::user_agent("ineapir")
        )

        # we use the GET method
      }else{
        if(request$addons$verbose){
          cat(sprintf("- API URL: %s\n", url$complete))
        }

        response <- httr::VERB("GET",
                               url = url$endpoint,
                               query = url$totalpar,
                               httr::user_agent("ineapir")
        )

      }
      # Get the content of the response
      content <- httr::content(response, "text", encoding = "UTF-8")

      if(jsonlite::validate(content)){
        result <- jsonlite::fromJSON(content , flatten = TRUE)
      }
    },
    error=function(e) {
      message('An error occurred calling the API')
      print(e)
    },
    warning=function(w) {
      message('A warning occurred calling the API')
      print(w)
    }
  )

  # Check the result retrieved for the API
  if(check_result(result, response)){

    # extract metadata to columns
    if((!is.null(request$addons$metanames) && request$addons$metanames) ||
       (!is.null(request$addons$metacodes) && request$addons$metacodes)){
      result <- extract_metadata(result, request)
    }

    # Unnest the Data column in one single dataframe
    if(!is.null(request$addons$unnest) && request$addons$unnest){
      result <- unnest_data(result)
    }

    # get the hierarchy tree of values
    if(!is.null(request$addons$hierarchy)){
      result <- get_values_hierarchy(result, request)
    }

  }

  return(result)
}

# Function to retrieve data from the aPI when the result is paginated
get_api_data_all_pages <- function(url, request){

  result <- NULL

  # Request all pages
  if(request$parameters$page == 0){
    # Page counter
    numpage <- 1

    # Update page
    request$parameters[["page"]] <- numpage

    # Build the URL to call the API
    url <- get_url(request)

    # Call de API
    result <- get_api_data(url, request)

    # Number of rows
    numrows <- if(!is.null(nrow(result))) nrow(result) else -1

    # if the number of rows is equal to the length of a page, we query the next page
    while (numrows > 0){
      numpage <- numpage + 1

      # Update page
      request$parameters[["page"]] <- numpage

      # Update the URL to call the API
      url <- get_url(request)

      # Call the API
      resultpage <- get_api_data(url, request)

      # Number of rows
      numrows <- if(!is.null(nrow(resultpage))) nrow(resultpage) else -1

      # Accumulated result
      result <- rbind(result, resultpage)
    }
    # Request a specific page
  }else{
    result <- get_api_data(url, request)
  }

  return(result)

}


# Get the url endpoint
get_definition_path <- function(request){

  path <- ""

  # Build the definition part. We remove tag (last one) from definition
  for(x in unlist(request$definition[-length(request$definition)])){
    if(!is.null(x)){
      path <- paste0(path,"/", x)
    }
  }

  return(path)
}

# Get url parameters minus filter
get_parameters_query <- function(request){

  parameters <- list()

  for(x in names(request$parameters)){
    val <- request$parameters[[x]]

    # Discard parameters with null value
    if(x != "filter" && !is.null(val)){

      # We have to format the input date
      if(x == "date"){
        val <- build_date(val)

        # manage array of dates
        for(i in 1:length(val)){
          parameters <- append(parameters, list(date = val[i]))
        }

        # Since the list also contains the operation
      }else if(x %in% c("p", "clasif")){
        val <- val[[x]]
        parameters[[x]] <- val
      }else{
        parameters[[x]] <- val
      }
    }
  }

  # include version API
  parameters[["ver"]] <- API_version

  return(parameters)

}

# Get url filter
get_parameters_filter <- function(request){

  val <- request$parameters[["filter"]]

  lval <- NULL
  if(!is.null(val)){
    lval <- build_filter(val, request$definition, request$addons, request$check$parameters$filter, det = request$parameters$det)
  }

  return(lval)
}

# Get url and its components
get_url <- function(request){
  # API url
  url <- httr::parse_url(API_URL)

  # Get the endpoint
  definition <- get_definition_path(request)

  # Get the parameters of the query
  parameters <- get_parameters_query(request)

  # Get the filter of the query
  parfilter <- get_parameters_filter(request)
  parfilter <- parfilter$filter

  # Update the path with the endpoint
  url$path <- paste0(url$path, definition)

  # Build the url endpoint
  endpoint <- httr::build_url(url)

  # Update the query of the url
  url$query <- parameters

  # Build the url with parameters
  endpointpar <- httr::build_url(url)

  # Update the query of the url adding the filter
  url$query <- append(parameters, parfilter)

  # Build the complete url
  complete <- httr::build_url(url)

  result <- list(complete = complete,
                 endpoint = endpoint,
                 endpointpar = endpointpar,
                 parameters = parameters,
                 filter = parfilter,
                 totalpar = append(parameters, parfilter)
  )

  return(result)
}

# Return the dates in the format used by the API
build_date <- function(date){
  # Date errors in some OS
  #dateStart <- format.Date(date$dateStart,'%Y%m%d')
  #dateEnd <- append(format.Date(date$dateEnd,'%Y%m%d'), rep("", length(date$dateStart)- length(date$dateEnd)))

  dateStart <- gsub("[/-]", "", date$dateStart)
  dateEnd <- append(gsub("[/-]", "", date$dateEnd), rep("", length(date$dateStart)- length(date$dateEnd)))

  return(paste(dateStart, dateEnd, sep = ":"))
}

# Return the cross of variables and values in the format used by the API
build_filter <- function(parameter, definition, addons, checkfilter, det = 0){
  # Values to return
  val <- character()
  lval <- list()

  # id to identify a table or a operation
  id <- parameter[[1]]

  # List of variables and values
  filter <- parameter[[2]]

  # Names in the list of the parameter
  parnames <- tolower(names(parameter))

  if(addons$verbose){
    cat("- Processing filter: 0% \r")
  }

  # If validate = TRUE there exists a dataframe with values
  if(addons$validate){
    # Dataframe with the values
    dfval <- checkfilter$values

    # If there are shortcuts present in the filter
    shortcut <- checkfilter$shortcut

    # Type of object: pxtable, tempus table or series
    origin <- checkfilter$origin

    # If validate = FALSE we get the values
  }else{
    # If there are shortcuts present in the filter
    shortcut <- check_shortcut(filter, definition)

    # Take into account negative values
    shortcut <- if(is_negative_filter_values(filter)) TRUE else shortcut

    # Dataframe with the values
    dfval <- get_filter_values(parameter, definition$lang, shortcut, verbose = FALSE, progress = addons$verbose, det = det, addons)

    if(!is.null(dfval)){
      origin <- dfval$origin
      dfval <- dfval$values
    }
  }

  # We go through all the variables
  i <- 1
  j <- 1
  for(n in names(filter)){
    # check if in the filter there are shortcuts
    short <- is.element(n, c(names(shortcuts_filter), shortcut_wrapper))

    # It is necessary to include shortcut in the case of a px table with a code equal to a shortcut (eg sexo)
    if(shortcut && short){
      if(!is.null(dfval)){
        # filter with ids
        filterout <- list()

        if(origin == "tablepx"){
          # column name depending on det parameter
          colname <- column_names[[API_version]][["variable.code"]][["es"]]

          # Select codes
          varid <- unique(dfval[[colname]])

          # We select only the values of variables present in the filter
          dfvalfilter <- subset(dfval, dfval[[colname]] %in% varid)

        }else if(origin == "tablepxid"){
          # column name depending on det parameter
          colname <- column_names[[API_version]][["variable.id"]][["es"]]

          # Select codes
          varid <- c(unique(dfval[[column_names[[API_version]][["variable.code"]][["es"]]]]), unique(dfval[[colname]]))

          # We select only the values of variables present in the filter
          dfvalfilter <- subset(dfval, dfval[[column_names[[API_version]][["variable.code"]][["es"]]]] %in% varid | dfval[[colname]] %in% varid)

        }else{
           # column name depending on det parameter
           colname <- if(det > 0 ) column_names[[API_version]][["variable.id"]][["es"]] else column_names[[API_version]][["variable.fk"]][["es"]]

          if(tolower(n) %in% shortcut_wrapper){
            # Select ids
            varid <- unique(dfval[[colname]])

          }else{
            # id of variables
            varid <- shortcuts_filter[[tolower(n)]]
          }

          # We select only the values of variables present in the filter
          dfvalfilter <- subset(dfval, dfval[[colname]] %in% varid)
        }

        # Reset the values found
        dfvalgrep <- NULL

        # Find a match between the filter inputs and the possible values
        for(f in filter[[n]]){

          # check if there is any negative value
          neg <- grepl("^-.*", as.character(f))

          # remove minus sign
          f <- if(sum(neg) > 0) gsub("^-", "", as.character(f)) else f

          ### Way one:  find a value for the largest word
          # Split the phrase
          valshort1 <- if(nchar(f) > 0 ) unlist(strsplit(as.character(f), "\\s+")) else f

          # Find the largest word
          valshort1 <- valshort1[which.max(nchar(valshort1))]

          # Find a match for the largest word and the possible values
          ind1 <- grepl(valshort1, dfvalfilter[[column_names[[API_version]][["value.name"]][["es"]]]], ignore.case = TRUE)

          # Dataframe with the matches
          dfvalgrep1 <- subset(dfvalfilter, ind1)

          ### Way two: find a value for the entire string
          # Find a match for the entire phrase and the possible values
          ind2 <- grepl(f, dfvalfilter[[column_names[[API_version]][["value.name"]][["es"]]]], ignore.case = TRUE)

          # Dataframe with the matches
          dfvalgrep2 <- subset(dfvalfilter, ind2)

          # Intersect the values from these two different ways
          if(nrow(dfvalgrep1) > 0 && nrow(dfvalgrep2) == 0){
            dfvalgreptmp <- dfvalgrep1

          }else if(nrow(dfvalgrep1) == 0 && nrow(dfvalgrep2) > 0){
            dfvalgreptmp <- dfvalgrep2

          }else if(nrow(dfvalgrep1) > 0 && nrow(dfvalgrep2) > 0){
            if(origin == "tablepx"){
              dfvalgrep2 <- subset(dfvalgrep2, select = c(column_names[[API_version]][["value.code"]][["es"]], colname))
              dfvalgreptmp <- merge(dfvalgrep1, dfvalgrep2, by = c(column_names[[API_version]][["value.code"]][["es"]], colname))

            }else if(origin == "tablepxid"){
              dfvalgrep2 <- subset(dfvalgrep2, select = c(column_names[[API_version]][["value.id"]][["es"]], colname))
              dfvalgreptmp <- merge(dfvalgrep1, dfvalgrep2, by = c(column_names[[API_version]][["value.id"]][["es"]], colname))

            }else{
              dfvalgrep2 <- subset(dfvalgrep2, select = c(column_names[[API_version]][["value.id"]][["es"]], colname))
              dfvalgreptmp <- merge(dfvalgrep1, dfvalgrep2, by = c(column_names[[API_version]][["value.id"]][["es"]], colname))
            }
          }else{
            dfvalgreptmp <- dfvalgrep1
          }

          # If there is no match result look in the id
          if(nrow(dfvalgreptmp) == 0){
            if(origin == "tablepx"){
              dfvalgreptmp <- subset(dfvalfilter, grepl(paste0("^",f,"$"), dfvalfilter[[column_names[[API_version]][["value.code"]][["es"]]]]))

            }else if(origin == "tablepxid"){
              dfvalgreptmp <- subset(dfvalfilter, grepl(paste0("^",f,"$"), c(dfvalfilter[[column_names[[API_version]][["value.code"]][["es"]]]], dfvalfilter[[column_names[[API_version]][["value.id"]][["es"]]]])))

            }else{
              dfvalgreptmp <- subset(dfvalfilter, grepl(paste0("^",f,"$"), dfvalfilter[[column_names[[API_version]][["value.id"]][["es"]]]]))
            }
          }

          # We add a column with the counter
          dfvalgreptmp$i <- rep(i,nrow(dfvalgreptmp))

          # Transform the filter in a the format used by the API
          if(nchar(f) > 0){

            # When grep found something
            if(nrow(dfvalgreptmp) > 0){

              # We go through all the matches
              for(r in 1:nrow(dfvalgreptmp)){
                if(origin == "tablepx"){
                  # Variable code
                  var <- dfvalgreptmp[[colname]][r]

                  # Value code
                  filterout[[var]] <- if(sum(neg) > 0) paste0("-", dfvalgreptmp[[column_names[[API_version]][["value.code"]][["es"]]]][r]) else dfvalgreptmp[[column_names[[API_version]][["value.code"]][["es"]]]][r]

                }else if(origin == "tablepxid"){
                  # Variable id
                  var <- dfvalgreptmp[[colname]][r]

                  # Value id
                  filterout[[var]] <- if(sum(neg) > 0) paste0("-", dfvalgreptmp[[column_names[[API_version]][["value.id"]][["es"]]]][r]) else dfvalgreptmp[[column_names[[API_version]][["value.id"]][["es"]]]][r]

                }else{
                  # Variable id
                  var <- dfvalgreptmp[[colname]][r]

                  # Value id
                  filterout[[var]] <- if(sum(neg) > 0) paste0("-", dfvalgreptmp[[column_names[[API_version]][["value.id"]][["es"]]]][r]) else dfvalgreptmp[[column_names[[API_version]][["value.id"]][["es"]]]][r]

                  if(exists("dfvalgrep") && is.data.frame(get("dfvalgrep")) ){

                    # If the variable id has been used in the filter, set the same counter
                    if(is.element(var, dfvalgrep[[colname]])){
                      i <- dfvalgrep[dfvalgrep[[colname]] == var,]$i[1]
                      dfvalgreptmp$i[r] <- i
                    }else{
                      if(nrow(dfvalgrep) > 0){
                        i <- max(dfvalgrep$i) + 1
                      }
                    }
                  }
                }

                # Check the filter comes from a table or a series
                parurl <- if(is.element("idtable",parnames)) "tv" else paste0("g", i)

                # check if there are negative values and remove
                filterout_neg <- check_negative_values(dfval, var, filterout[[var]], origin)

                # Build the filter with the format of the API
                #tmp <- paste0(parurl, "=", var, ":", filterout[[var]])
                tmp <- paste0(parurl, "=", filterout_neg)

                # Vector with all the values in the format of the API
                val <- append(val, tmp)

                if(length(lval) > 0){
                  # in case we have to remove repeated values included in the negative check
                  if(sum(neg) > 0){
                    # elements in the filter with variable equal to var
                    fvar <- lapply(strsplit(unlist(lval), ":"), function(x) x[1] == var)

                    # elements in the filter with variable not equal to var
                    fnovar <- lapply(strsplit(unlist(lval), ":"), function(x) x[1] != var)

                    lval1 <- list()
                    if(sum(unlist(fvar)) > 0){
                      # select from filter only the values for variable var
                      ulval <- lval[unlist(fvar)]

                      # format of the API
                      ulval <- paste0(names(ulval),"=",unlist(ulval))

                      # remove values with negative sign from filter
                      ulval <- intersect(ulval, tmp)

                      # format of the filter to return
                      lval1 <- lapply(strsplit(ulval, "="), `[[`, 2)
                      names(lval1) <- unlist(lapply(strsplit(ulval, "="), `[[`, 1))

                    }else{
                      lval1 <- lapply(strsplit(tmp, "="), `[[`, 2)
                      names(lval1) <- unlist(lapply(strsplit(tmp, "="), `[[`, 1))
                    }

                    # union
                    lval <- append(lval[unlist(fnovar)], lval1)

                  }else{
                    # List with all the values
                    #for(fo in filterout[[var]]){
                    for(fo in filterout_neg){
                      #lval <- append(lval, list(paste0(var, ":", fo)))
                      lval <- append(lval, list(fo))
                      names(lval)[length(lval)] <- parurl
                    }
                  }
                }else{
                  # List with all the values
                  for(fo in filterout_neg){
                    #lval <- append(lval, list(paste0(var, ":", fo)))
                    lval <- append(lval, list(fo))
                    names(lval)[length(lval)] <- parurl
                  }
                }
              }
            }
          }else{
            # Case when the value introduced is and empty character ""
            if(length(varid) == 1){
              # value set to ""
              filterout[[varid]] <- f

              # Check the filter comes from a table or a series
              parurl <- if(is.element("idtable",parnames)) "tv" else paste0("g", i)

              # Build the filter with the format of the API
              tmp <- paste0(parurl, "=", varid, ":", filterout[[varid]])

              # Vector with all the values in the format of the API
              val <- append(val, tmp)

              # List with all the values
              lval <- append(lval, list(paste0(varid, ":", filterout[[varid]])))
              names(lval)[length(lval)] <- parurl
            }
          }

          if (exists("dfvalgrep") && is.data.frame(get("dfvalgrep"))){
            dfvalgrep <- rbind(dfvalgrep,dfvalgreptmp)
          }else{
            dfvalgrep <- dfvalgreptmp
          }

          if(addons$verbose){
            cat(sprintf("- Processing filter: %s%%        \r", round(50 + j/sum(lengths(filter))*50,0)))
          }

          i <- i + 1
          j <- j + 1
        }
      }
      # When there are no shortcuts in the filter
    }else{
      # Check the filter comes from a table or a series
      parurl <- if(is.element("idtable",parnames)) "tv" else paste0("g", i)

      # check if there are negative values and remove
      filter_neg <- check_negative_values(dfval, n, filter[[n]], origin)

      # Build the filter with the format of the API
      #tmp <- paste0(parurl, "=", n, ":", filter[[n]])
      tmp <- paste0(parurl, "=", filter_neg)

      # Vector with all the values in the format of the API
      val <- append(val, tmp)

      #for(f in filter[[n]]){
        for(f in filter_neg){
        # List with all the values
        #lval <- append(lval, list(paste0(n, ":", f)))
        lval <- append(lval, list(f))
        names(lval)[length(lval)] <- parurl

        if(addons$verbose){
          cat(sprintf("- Processing filter: %s%%        \r", round(50 + j/sum(lengths(filter))*50,0)))
        }

        j <- j + 1
      }

      i <- i + 1
    }
  }

  if(addons$verbose){
    cat("- Processing filter: 100%         \n")
  }

  return(list(filter = lval, df = dfval))
}

# Get the all values used in a table or operation
get_filter_values <- function(parameter, lang, shortcut, verbose, progress = TRUE, det = 0, addons = NULL){

  # id to identify a table or a operation
  id <- parameter[[1]]

  # List of variables and values
  filter <- parameter[[2]]

  # Names in the list of the parameter
  parnames <- tolower(names(parameter))

  # Dataframe to return the values
  dfval <- NULL

  # The filter includes shortcuts in the names of variables and values
  if(shortcut){
    # The filter comes from a table
    if(is.element("idtable",parnames)){

      # Get the metadata information of the table
      dfval <- get_metadata_variable_values_table(idTable = id, verbose = verbose, validate = FALSE, lang = lang, progress = progress, det = det)

      # The filter comes from a series
    }else if (is.element("operation",parnames)){
      # We obtain the variables and values from the operation of the series
      dfval <- get_metadata_variable_values_operation(operation = id, verbose = verbose, validate = FALSE, lang, progress = progress, det = det)
    }else{
      # We obtain the variables and values from hierarchy tree
      dfval <- get_metadata_variable_values_hierarchy(variable = id, verbose = verbose, validate = FALSE, lang, progress = progress, det = det, hierarchy = addons$hierarchy)
    }
  }

  return(dfval)
}

# Check the request
check_request <- function(request){

  # Check addons
  cadd <- check_addons(request$parameters, request$addons, request$definition)

  # Check definition
  cdef <- check_definition(request$definition, request$addons)

  # Check parameters
  cpar <- check_parameters(request$parameters, request$addons, request$definition)

  # Check results to return
  check <- list()
  check <- append(check, list(definition = cdef))
  check <- append(check, list(addons = cadd))
  check <- append(check, list(parameters = cpar))

  request <- append(request, list(check = check))

  return(request)
}

# Check the definition of the request
check_definition <- function(definition, addons){
  result <- list()

  # Validate or not the definition
  check <- addons$validate

  if(check){
    for(x in names(definition)){
      val <- definition[[x]]

      if(!is.null(val)){
        r <- switch (x,
                     "lang" = check_lang(val, addons$verbose),
                     "input" = check_input(definition$tag, val, addons$verbose)
        )
        # Check results to return
        if(!is.null(r)){
          result <- append(result, r)
          names(result)[length(result)] <- x
        }
      }
    }
  }

  return(result)
}

# Check the parameters of the request
check_parameters <- function(parameters, addons, definition){
  result <- list()

  # Validate or not the parameters
  check <- addons$validate

  if(check){
    for(x in names(parameters)){
      val <- parameters[[x]]

      if(!is.null(val)){
        r <- switch (x,
                     "date" = check_dates(val, addons$verbose),
                     "p" = check_periodicity(val[[1]], val[[2]], addons$verbose),
                     "nult" = check_nlast(val, addons$verbose),
                     "det" = check_det(val, addons$verbose),
                     "tip" = check_tip(val, addons$verbose),
                     "geo" = check_geo(val, addons$verbose),
                     "page" = check_page(val, addons$verbose),
                     "filter" = check_filter(val, addons, definition, parameters$det),
                     "clasif" = check_classification(val[[1]], val[[2]], addons$verbose)
        )
        # Check results to return
        result <- append(result, r)
        names(result)[length(result)] <- x
      }
    }
  }

  return(result)
}

# Check the addons of the request
check_addons <- function(parameters, addons, definition){
  result <- list()

  # Validate or not the addons
  check <- addons$validate

  if(check){
    for(x in names(addons)){
      val <- addons[[x]]

      if(!is.null(val)){
        r <- switch (x,
                     "validate" = check_islogical(x, val),
                     "verbose" = check_islogical(x, val),
                     "unnest" = check_islogical(x, val),
                     "metanames"= check_extractmetadata(x, val, parameters$tip),
                     "metacodes"= check_extractmetadata(x, val, parameters$tip),
                     "hierarchy" = check_hierarchy(val, addons$verbose),
                     "filter" = check_filter(val, addons, definition, parameters$det)
        )
        # Check results to return
        result <- append(result, r)
        names(result)[length(result)] <- x
      }
    }
  }

  return(result)
}

#Check the result retrieved for the API
check_result <- function(result, response = NULL){
  check <- FALSE

  if(!is.null(result)){
    if(!check_result_status(result)){
      if(is.data.frame(result) && nrow(result) > 0){
        check <- TRUE
      }
      if(is.list(result) && !is.data.frame(result) && length(result) > 0){
        check <- TRUE
      }
    }
  }else{
    if(!is.null(response)){
      if(response$status_code != 200){
        # GRUPOS_TABLA used to check px tables
        if(!grepl("/GRUPOS_TABLA/", response$url, ignore.case = TRUE)){
          message(sprintf("An error occurred calling the API (status %s).\n%s", response$status_code, response$url))
        }
      }
    }
  }

  return(check)
}

#Check the result retrieved for the API
check_result_status <- function(result){
  check <- FALSE

  if(is.element("status", names(result))){
    check <- TRUE
    cat(sprintf("- %s\n", result$status))
  }

  return(check)
}

# check if lang argument in the definition is valid
check_lang <- function(lang, verbose){
  result <- TRUE

  if(!is.character(lang)){
    result <- FALSE
    stop("lang must be a string equal to 'ES' for Spanish or equal to 'EN' for English")
  }else{
    if(lang != "ES" && lang != "EN"){
      result <- FALSE
      stop("lang must be a string equal to 'ES' for Spanish or equal to 'EN' for English")
    }
  }

  if(verbose){
    cat(sprintf("- Check lang: OK\n"))
  }

  return(result)
}

# Check the input part of the definition
check_input <- function(tag, input, verbose){
  result <- list()

  r <- switch(
    tag,
    "operation" = check_operation(input, verbose = verbose),
    "operation_active_null" = check_operation(input, active_null = TRUE, verbose = verbose),
    "codSeries" = check_isnull(tag, input, verbose),
    "variable_operation" = check_variables_operation(input, verbose),
    "publication" = check_publication(input, verbose),
    "idTable" = check_isnull(tag, input, verbose),
    "idTable_idGroup" = check_idtable_idgroup(input, verbose)
  )
  # Check results to return
  result <- append(result, r)
  names(result)[length(result)] <- tag

  return(result)
}

# Check operation argument in API call
check_operation <- function(operation, active_null = FALSE, verbose){
  result <- TRUE

  if(!is.null(operation)){
    # Get all operations
    opes <- get_metadata_operations(validate = FALSE, verbose = verbose, page = 0)

    # Logical controls
    id <- FALSE
    ioe <- FALSE
    cod <- FALSE

    # Check id
    tmp <- opes$Id[trimws(opes[[column_names[[API_version]][["id"]][["es"]]]]) != ""]

    if(!is.element(operation,tmp)){
      id <- TRUE
    }

    # Check cod_IOE
    tmp <- paste0("IOE", opes[[column_names[[API_version]][["ioe"]][["es"]]]][trimws(opes[[column_names[[API_version]][["ioe"]][["es"]]]]) != ""])

    if(!is.element(operation,tmp)){
      ioe <- TRUE
    }

    # Check code
    tmp <- opes[[column_names[[API_version]][["codigo"]][["es"]]]][trimws(opes[[column_names[[API_version]][["codigo"]][["es"]]]]) != ""]

    if(!is.element(operation,tmp)){
      cod <- TRUE
    }

    result <- !(id & ioe & cod)

    if(!result){
      stop("The operation not exists")
    }
  }else{
    if(!active_null){
      result <- FALSE
      stop("The operation must be specified")
    }
  }

  if(verbose){
    cat(sprintf("- Check operation: OK\n"))
  }

  return(result)
}

# Check variables
check_variables_operation <- function(input, verbose){
  result <- TRUE

  # Variable id
  variable <- input$variable

  # Operation id
  operation <- if("operation" %in% names(input)) input$operation else NULL

  # Value id
  value <- if("value" %in% names(input)) input$value else NULL

  if(!is.null(operation)){
    # First we check if the operation is valid
    check_operation(operation, verbose = verbose)

    # Second we check if the variable is valid for the operation
    result <- check_variablesoperation(operation, variable, verbose)

  }else{
    # Check if the variable is valid
    result <- check_variable(variable, verbose)

    if(!is.null(value)){
      # Check if the value is valid
      result <- check_value(variable, value, verbose)
    }
  }

  return(result)
}

# Check if a variable is valid for an operation
check_variablesoperation <- function(operation, variable, verbose){
  result <- TRUE

  if(!is.null(variable)){
    vars <- get_metadata_variables(operation = operation, validate = FALSE, verbose = verbose, page = 0)

    if(!is.element(variable, vars[[column_names[[API_version]][["id"]][["es"]]]])){
      result <- FALSE
      stop(sprintf("%s is not a valid variable for operation %s. Valid ids: %s", variable, operation, paste0(vars$Id, collapse = ", ")))
    }
  }else{
    result <- FALSE
    stop("variable argument must be specified")
  }

  if(verbose){
    cat(sprintf("- Check variable: OK\n"))
  }

  return(result)
}

# Check if the variable is valid
check_variable <- function(variable, verbose){
  result <- TRUE

  if(!is.null(variable)){
    vars <- get_metadata_variables(validate = FALSE, verbose = verbose, page = 0)

    if(!is.element(variable, vars[[column_names[[API_version]][["id"]][["es"]]]])){
      result <- FALSE
      stop(sprintf("%s variable not exists", variable))
    }
  }else{
    result <- FALSE
    stop("variable argument must be specified")
  }

  if(verbose){
    cat(sprintf("- Check variable: OK\n"))
  }

  return(result)
}

# Check if a variable is valid for an operation
check_value <- function(variable, value, verbose){
  result <- TRUE

  if(!is.null(value)){
    vars <- get_metadata_values(variable = variable, validate = FALSE, verbose = verbose, page = 0)

    if(!is.element(value, vars[[column_names[[API_version]][["id"]][["es"]]]])){
      result <- FALSE
      stop(sprintf("%s is not a valid value for variable %s. Valid ids: %s", value, variable, paste0(vars$Id, collapse = ", ")))
    }
  }

  if(verbose){
    cat(sprintf("- Check value: OK\n"))
  }

  return(result)
}
# check if a publication is valid
check_publication <- function(publication, verbose){
  result <- TRUE

  if(!is.null(publication)){
    # Get all the publications
    pubs <- get_metadata_publications(validate = FALSE, verbose = verbose, page = 0)

    if(!is.element(publication, pubs[[column_names[[API_version]][["id"]][["es"]]]])){
      result <- FALSE
      stop(sprintf("%s publication not exists", publication))
    }
  }else{
    result <- FALSE
    stop("publication argument must be specified")
  }

  if(verbose){
    cat(sprintf("- Check publication: OK\n"))
  }
  return(result)
}

# Check if the argument is NULL
check_isnull <- function(name, id, verbose){
  result <- TRUE

  if(is.null(id)){
    result <- FALSE
    stop(sprintf("%s argument must be specified", name))
  }

  if(verbose){
    cat(sprintf("- Check %s: OK\n", name))
  }

  return(result)
}

# Check if both, table and group, are NULL
check_idtable_idgroup <- function(input, verbose){
  result <- TRUE

  idTable <- input$idTable
  idGroup <- input$idGroup

  nameid <- names(input)

  check_isnull(nameid[1], idTable, verbose)
  check_isnull(nameid[2], idGroup, verbose = FALSE)

  if(!is.null(idTable) && !is.null(idGroup)){
    # Get all the groups of the table
    groups <- get_metadata_table_groups(idTable = idTable, validate = FALSE, verbose = verbose)

    if(!is.element(idGroup, groups[[column_names[[API_version]][["id"]][["es"]]]])){
      result <- FALSE
      stop(sprintf("%s is not a valid group for table %s. Valid ids: %s", idGroup, idTable, paste0(groups$Id, collapse = ", ")))
    }
  }

  if(verbose){
    cat(sprintf("- Check idGroup: OK\n"))
  }

  return(result)
}

# Check date argument in API CALL
check_dates <- function(date, verbose){
  result <- TRUE

  dateStart <- date$dateStart
  dateEnd <- date$dateEnd

  namesdate = names(date)

  check_date_format(namesdate[1], dateStart)
  check_date_format(namesdate[2], dateEnd)

  if(!is.null(dateEnd)){
    if(!is.null(dateStart)){
      if(length(dateEnd) > length(dateStart)){
        result <- FALSE
        stop("the length of dateEnd must be less than or equal to the length of dateStart.")
      }else{
        for(i in 1:length(dateEnd)){
          if(dateStart[i] > dateEnd[i]){
            result <- FALSE
            stop(sprintf("dateStart (%s) must be previous to dateEnd (%s) or the same.", dateStart[i], dateEnd[i]))
          }
        }
      }
    }else{
      result <- FALSE
      stop("dateStart must be specified.")
    }
  }

  if(verbose){
    cat(sprintf("- Check date: OK\n"))
  }

  return(result)
}

# Check the input format of the date
check_date_format <- function(name, date){
  for(f in date){
    # Remove white spaces
    #f <- if(!is.null(f)) gsub("\\s+", "", f) else f

    # Input format must be yyyy/mm/dd
    format <- if(!is.null(f)) grepl("[0-9]{4}/[0-9]{2}/[0-9]{2}", f) else FALSE

    if(format){
      y <- substr(f, 1, 4)
      m <- substr(f, 6, 7)
      d <- substr(f, 9, 10)

      if(m > 12){
        stop(sprintf("%s month can not be greater than 12", name))
      }

      if(d > 31){
        stop(sprintf("%s day can not be greater than 31", name))
      }
    }else{
      if(!is.null(f)){
        stop(sprintf("%s format is not correct. Date format must be as follow: yyyy/mm/dd", name))
      }
    }
  }
}

# check if the periodicity argument is valid
check_periodicity <- function(operation, p, verbose){

  result <- TRUE

  if(!is.null(p)){
    # Get periodicities of an operation
    periodicity <- get_metadata_periodicity(operation = operation, validate = FALSE, verbose = verbose)

    if(!is.element(p, periodicity[[column_names[[API_version]][["id"]][["es"]]]])){
      result <- FALSE
      stop(sprintf("%s is not a valid periodicity for operation %s. Valid ids: %s", p, operation, paste0(periodicity[[column_names[[API_version]][["id"]][["es"]]]], collapse = ", ")))
    }
  }else{
    result <- FALSE
    stop("periodicity must be specified")
  }

  if(verbose){
    cat(sprintf("- Check periodicity: OK\n"))
  }
  return(result)
}

# Check if the nlast argument is valid
check_nlast <- function(nlast, verbose){
  result <- TRUE

  if(!is.numeric(nlast)){
    result <- FALSE
    stop("nlast must be a number greater or equal to 1")

  }else{
    if(nlast < 1){
      result <- FALSE
      stop("nlast must be a number greater or equal to 1")
    }
  }

  if(verbose){
    cat(sprintf("- Check nlast: OK\n"))
  }

  return(result)
}

# Check if the det argument is valid
check_det <- function(det, verbose){
  result <- TRUE

  if(!is.numeric(det)){
    result <- FALSE
    stop("det must be a number between 0 and 2")
  }else{
    if(det < 0 || det > 2){
      result <- FALSE
      stop("det value must be between 0 and 2")
    }
  }

  if(verbose){
    cat(sprintf("- Check det: OK\n"))
  }

  return(result)
}

# Check if the tip argument is valid
check_tip <- function(tip, verbose){
  result <- TRUE

  if(!is.null(tip)){
    tip <- toupper(tip)

    if(tip != "A" && tip != "M" && tip != "AM" && tip != "MA"){
      result <- FALSE
      stop("tip must be equal to 'A', 'M' or 'AM'")
    }
  }

  if(verbose){
    cat(sprintf("- Check tip: OK\n"))
  }

  return(result)
}

# Check if the geo argument is valid
check_geo <- function(geo, verbose){
  result <- TRUE

  if(!is.null(geo)){
    if(!is.numeric(geo)){
      result <- FALSE
      stop("geo must be a number equal to 0 or 1")
    }else{
      if(geo < 0 || geo > 1){
        result <- FALSE
        stop("geo must be a number equal to 0 or 1")
      }
    }
  }

  if(verbose){
    cat(sprintf("- Check geo: OK\n"))
  }
  return(result)
}

# check if the page argument is valid
check_page <- function(n, verbose){
  result <- TRUE

  if(!is.numeric(n)){
    result <- FALSE
    stop("page must be a number greater or equal to 0")
  }else{
    if(n < 0){
      result <- FALSE
      stop("page must be a number greater or equal to 0")
    }
  }

  if(verbose){
    cat(sprintf("- Check page: OK\n"))
  }
  return(result)
}

# Check if the filter argument is valid
check_filter <- function(parameter, addons, definition, det = 0){
  result <- TRUE

  # If there are shortcuts in the filter
  shortcut <- FALSE

  # check if there are negative values in the filter
  if(is_negative_filter_values(parameter[[2]])){
    # remove the negative signs
    parameter[[2]] <- remove_filter_negative_values(parameter[[2]])
  }

  # id to identify a table or a operation
  id <- parameter[[1]]

  # List of variables and values
  filter <- parameter[[2]]

  # Names in the list of the parameter
  parnames <- tolower(names(parameter))

  # Get the values from metadata of tables or operations
  df <- get_filter_values(parameter, definition$lang, shortcut = TRUE, verbose = addons$verbose, progress = FALSE, det = det, addons = addons)

  # Make sure the response is valid or null
  if(check_result(df$values)){

    # The filter comes from a px table
    if(df$origin == "tablepx"){
      check <- check_table_px_filter(id, filter, addons$verbose, df$values)

      result <- check$result
      shortcut <- check$shortcut

      # The filter comes from a px table with ids
    }else if(df$origin == "tablepxid"){
      check <- check_table_px_id_filter(id, filter, addons$verbose, df$values)

      result <- check$result
      shortcut <- check$shortcut

      # The filter comes from a tempus table
    }else if(df$origin == "tablet3"){
      check <- check_table_tempus_filter(parameter, addons$verbose, df$values, det = det)

      result <- check$result
      shortcut <- check$shortcut
    }

    # The filter comes from a series
    else if(df$origin == "series") {
      check <- check_series_filter(parameter, addons$verbose, df$values, det = det)

      result <- check$result
      shortcut <- check$shortcut
    }
    # The filter comes from variables
    else if(df$origin == "variables") {
      check <- check_variables_filter(parameter, addons$verbose, df$values, det = det)

      result <- check$result
      shortcut <- check$shortcut
    }
  }

  # If there are shortcuts in the filter
  df$shortcut <- shortcut

  return(list(df))
}

# check if the classification argument is valid
check_classification <- function(operation, clasif, verbose){

  result <- TRUE

  if(!is.null(clasif)){
    # Get periodicities of an operation
    classification <- get_metadata_classifications(operation = operation, validate = FALSE, verbose = verbose)

    if(!is.element(clasif, classification[[column_names[[API_version]][["id"]][["es"]]]])){
      result <- FALSE

      if(is.null(operation)){
        stop(sprintf("%s is not a valid classification. Valid ids: %s", clasif, paste0(classification[[column_names[[API_version]][["id"]][["es"]]]], collapse = ", ")))
      }else{
        stop(sprintf("%s is not a valid classification for operation %s. Valid ids: %s", clasif, operation, paste0(classification[[column_names[[API_version]][["id"]][["es"]]]], collapse = ", ")))
      }
    }
  }

  if(verbose){
    cat(sprintf("- Check classification: OK\n"))
  }
  return(result)
}

# Check if the det argument is valid
check_hierarchy <- function(depth, verbose){
  result <- TRUE

  if(!is.numeric(depth)){
    result <- FALSE
    stop("hierarchy must be a number between 0 and 10")
  }else{
    if(depth < 0 || depth > 10){
      result <- FALSE
      stop("hierarchy value must be between 0 and 10")
    }
  }

  if(verbose){
    cat(sprintf("- Check hierarchy: OK\n"))
  }

  return(result)
}

# Confirm if the metadata of the table contains information about the values id
exists_values_id <- function(metadata){
  result <- FALSE

  # Column names of the metadata of the table
  metacols <- tolower(unique(unlist(lapply(metadata, names))))

  # If there is a id column then
  if(is.element("id", metacols)){
    result <- TRUE
  }

  return(result)
}

# Confirm if the metadata of the table contains information about the values id
exists_variables_id <- function(metadata){
  result <- FALSE
  colname <- ""

  # Column names of the metadata of the table
  metacols <- unique(unlist(lapply(metadata, names)))

  # If there is a id column
  if(is.element("variable.id", tolower(metacols))){
    result <- TRUE
    colname <- metacols[grep("variable.id", metacols, ignore.case = TRUE)]
  }

  if(is.element("fk_variable", tolower(metacols))){
    result <- TRUE
    colname <- metacols[grep("fk_variable", metacols, ignore.case = TRUE)]
  }

  if(is.element("t3_variable", tolower(metacols))){
    result <- TRUE
    colname <- metacols[grep("t3_variable", metacols, ignore.case = TRUE)]
  }

  return(list(result = result, name = colname))
}

# Check if the filter argument is valid for a px table
check_table_px_filter <- function(idTable, pxfilter, verbose, df){
  result <- TRUE

  # If there are shortcuts in the filter
  shortcut <- FALSE

  # The filter must be a list
  if(is.list(pxfilter)){
    # Variables of the filter
    var <- names(pxfilter)

    # Go through all the variables
    for(v in var){
      # If the variable in the filter is not in the metadata is not valid
      if(!is.element(v, c(df[[column_names[[API_version]][["variable.code"]][["es"]]]], shortcut_wrapper))){
        result <- FALSE
        msg <- sprintf("%s is not a valid variable for %s idTable. Valid variable codes: %s",v,idTable, paste0(unique(df[[column_names[[API_version]][["variable.code"]][["es"]]]]), collapse = ", "))
        msg <- if(is.element(v, names(shortcuts_filter))) paste0(msg,"\nThe only shortcut valid for this table is the wrapper 'values'") else msg
        stop(msg)
      }

      # Has been used a shortcut name for the variable or not
      short <- is.element(tolower(v), shortcut_wrapper)

      # Identify a filter with shortcuts
      shortcut <- shortcut | short

      # subset of the metadata for an specific variable
      metavar <- if(v %in% shortcut_wrapper) df else df[df[[column_names[[API_version]][["variable.code"]][["es"]]]] == v,]

      # Go through all the values in the filter for the specific variable
      for(val in pxfilter[[v]]){

        # Split the value
        valshort <- if(nchar(val) > 0 ) unlist(strsplit(as.character(val), "\\s+")) else val

        validnames <- TRUE
        for(vs in valshort){
          validnames <- validnames & sum(grepl(vs, metavar[[column_names[[API_version]][["value.name"]][["es"]]]], ignore.case = TRUE)) > 0
        }

        # If the value in the filter is not in the metadata is not valid
        if(val != "" && !(is.element(val, metavar[[column_names[[API_version]][["value.code"]][["es"]]]]) || validnames )){
          result <- FALSE
          stop(sprintf("%s is not a valid value for variable %s", val, v))
        }
      }
    }
  }else{
    result <- FALSE
    stop("filter must be a list")
  }

  if(verbose){
    cat(sprintf("- Check filter: OK\n"))
  }

  return(list(result = result, shortcut = shortcut))
}

# Check if the filter argument is valid for a px table
check_table_px_id_filter <- function(idTable, pxfilter, verbose, df){
  result <- TRUE

  # If there are shortcuts in the filter
  shortcut <- FALSE

  # The filter must be a list
  if(is.list(pxfilter)){

    # check for alias ~ in the filter
    pxfilter <- check_alias_filter(pxfilter)

    # Variables of the filter
    var <- names(pxfilter)

    # Go through all the variables
    for(v in var){
      # If the variable in the filter is not in the metadata is not valid
      if(!is.element(v, c(df[[column_names[[API_version]][["variable.code"]][["es"]]]], df[[column_names[[API_version]][["variable.id"]][["es"]]]], shortcut_wrapper))){
        result <- FALSE
        msg <- sprintf("%s is not a valid variable for %s idTable. Valid variable codes: %s. Valid variable ids: %s",
                       v,
                       idTable,
                       paste0(unique(df[[column_names[[API_version]][["variable.code"]][["es"]]]][nchar(df[[column_names[[API_version]][["variable.code"]][["es"]]]]) > 0]), collapse = ", "),
                       paste0(unique(df[[column_names[[API_version]][["variable.code"]][["es"]]]]), collapse = ", "))
        msg <- if(is.element(v, names(shortcuts_filter))) paste0(msg,"\nThe only shortcut valid for this table is the wrapper 'values'") else msg
        stop(msg)
      }

      # Has been used a shortcut name for the variable or not
      short <- is.element(tolower(v), shortcut_wrapper)

      # Identify a filter with shortcuts
      shortcut <- shortcut | short

      # subset of the metadata for an specific variable
      metavar <- if(v %in% shortcut_wrapper) df else df[df[[column_names[[API_version]][["variable.code"]][["es"]]]] == v | df[[column_names[[API_version]][["variable.code"]][["es"]]]] == v,]

      # Go through all the values in the filter for the specific variable
      for(val in pxfilter[[v]]){
        # Split the value
        valshort <- if(nchar(val) > 0 ) unlist(strsplit(as.character(val), "\\s+")) else val

        validnames <- TRUE
        for(vs in valshort){
          validnames <- validnames & sum(grepl(vs, metavar[[column_names[[API_version]][["value.name"]][["es"]]]], ignore.case = TRUE)) > 0
        }

        # If the value in the filter is not in the metadata is not valid
        if(val != "" && !(is.element(val, c(metavar[[column_names[[API_version]][["value.code"]][["es"]]]], metavar[[column_names[[API_version]][["value.id"]][["es"]]]])) || validnames )){
          result <- FALSE
          stop(sprintf("%s is not a valid value for variable %s", val, v))
        }
      }
    }
  }else{
    result <- FALSE
    stop("filter must be a list")
  }

  if(verbose){
    cat(sprintf("- Check filter: OK\n"))
  }

  return(list(result = result, shortcut = shortcut))
}

# Check if the filter argument is valid for a tempus table
check_table_tempus_filter <- function(parameter, verbose, df, det = 0){
  result <- TRUE

  # If there are shortcuts in the filter
  shortcut <- FALSE

  # id to identify a table or a operation
  id <- parameter[[1]]

  # List of variables and values
  filter <- parameter[[2]]

  # Names in the list of the parameter
  parnames <- tolower(names(parameter))

  # The filter must be a list
  if(is.list(filter)){
    check <- check_tempus_filter(id, filter, parnames, df, det = det)
    shortcut <- check$shortcut

  }else{
    result <- FALSE
    stop("filter must be a list")
  }

  if(verbose){
    cat(sprintf("- Check filter: OK\n"))
  }

  return(list(result = result, shortcut = shortcut))
}

# Check if the filter argument is valid for a series
check_series_filter <- function(parameter, verbose, df, det = 0){
  result <- TRUE

  # If there are shortcuts in the filter
  shortcut <- FALSE

  # id to identify a table or a operation
  id <- parameter[[1]]

  # List of variables and values
  filter <- parameter[[2]]

  # Names in the list of the parameter
  parnames <- tolower(names(parameter))

  # The filter must be a list
  if(is.list(filter)){
    # Variables of the filter
    var <- names(filter)

    # Values of the filter
    val <- unlist(filter, use.names = FALSE)

    # The list must contain at least two values in the filter
    if(length(var) > 1 || (length(var) < 2 && length(val) > 1)){
      check <- check_tempus_filter(id, filter, parnames, df, det = det)
      shortcut <- check$shortcut

    }else{
      result <- FALSE
      stop("The list must contain at least two values in the filter")
    }
  }else{
    result <- FALSE
    stop("filter must be a list")
  }


  if(verbose){
    cat(sprintf("- Check filter: OK\n"))
  }

  return(list(result = result, shortcut = shortcut))
}

# Check if the filter argument is valid for variables
check_variables_filter <- function(parameter, verbose, df, det = 0){
  result <- TRUE

  # If there are shortcuts in the filter
  shortcut <- FALSE

  # id to identify a table or a operation
  id <- parameter[[1]]

  # List of variables and values
  filter <- parameter[[2]]

  # Names in the list of the parameter
  parnames <- tolower(names(parameter))

  # The filter must be a list
  if(is.list(filter)){
    check <- check_tempus_filter(id, filter, parnames, df, det = det)
    shortcut <- check$shortcut

  }else{
    result <- FALSE
    stop("filter must be a list")
  }

  if(verbose){
    cat(sprintf("- Check filter: OK\n"))
  }

  return(list(result = result, shortcut = shortcut))
}

# Check if the filter argument is valid
check_tempus_filter <- function(id, filter, parnames, df, det = 0){
  result <- TRUE

  # If there are shortcuts in the filter
  shortcut <- FALSE

  # column name depending on det parameter
  colname <- if(det > 0 ) column_names[[API_version]][["variable.id"]][["es"]] else column_names[[API_version]][["variable.fk"]][["es"]]

  # Variables of the filter
  var <- names(filter)

  for(v in var){
    # Has been used a shortcut name for the variable or not
    short <- is.element(tolower(v), c(names(shortcuts_filter), shortcut_wrapper))

    # Identify a filter with shortcuts
    shortcut <- shortcut | short

    if(short){
      # The values wrapper is present
      if(tolower(v) %in% shortcut_wrapper){
        variable <- unique(df[[colname]])

        # A shortcut is present
      }else{
        variable <- shortcuts_filter[[tolower(v)]]
      }
    }else{
      variable <- v
    }

    # The variable id is in the metadata information
    validvar <- intersect(variable, df[[colname]])

    if(!(is.element(v, df[[colname]]) || length(validvar) > 0 )){
      result <- FALSE
      stop(sprintf("%s is not a valid variable for %s %s",v, parnames[1],id))
    }

    # If the shortcut name includes more than one variable
    # obtain the metadata information for all the variables
    metavar <- subset(df, df[[colname]] %in% validvar)

    # Go through all the values of an specific variable
    for(val in filter[[v]]){
      # Multiple values
      for(f in val){

        # Split the value
        valshort <- if(nchar(f) > 0 ) unlist(strsplit(as.character(f), "\\s+")) else f

        validnames <- TRUE
        # Check each part of the value
        for(vs in valshort){
          validnames <- validnames & sum(grepl(vs, metavar[[column_names[[API_version]][["value.name"]][["es"]]]], ignore.case = TRUE)) > 0
        }

        # The id or the shortcut name of the value must exist in the metadata information
        if(f != "" && !(is.element(f, metavar[[column_names[[API_version]][["value.id"]][["es"]]]]) || validnames)){
          result <- FALSE

          if(is.element("idtable",parnames)){
            stop(sprintf("%s is not a valid value for variable %s or is not present in the values of the groups of the table", f, v))
          }else{
            stop(sprintf("%s is not a valid value for variable %s", f, v))
          }
        }
      }
    }
  }

  return(list(result = result, shortcut = shortcut))
}

# Check if an argument is logical
check_islogical <- function(name, par){
  result <- TRUE

  if(!is.logical(par)){
    result <- FALSE
    stop(sprintf("%s must be logical", name))
  }

  return(result)
}

# Check if the argument shortcut is set correctly
check_shortcut <- function(filter, definition){

  result <- FALSE

  if(!is.null(filter)){

    # Tables
    if(grepl("IdTable", definition$tag, ignore.case = TRUE)){

      # check the type of the table
      checktable <- check_type_table(idTable = definition$input)

      # PX table
      if(checktable$ispxtable){
        if(sum(is.element(tolower(names(filter)), shortcut_wrapper)) > 0){
          result <- TRUE
        }

        # Tempus table
      }else{
        # There are shortcuts in the filter
        if(sum(is.element(tolower(names(filter)), c(names(shortcuts_filter), shortcut_wrapper))) > 0 ){
          result <- TRUE
        }
      }

      # Tempus series
    }else{
      # There are shortcuts in the filter
      if(sum(is.element(tolower(names(filter)), c(names(shortcuts_filter), shortcut_wrapper))) > 0 ){
        result <- TRUE
      }
    }
  }

  return(result)
}


# Check if the extractmetadata argument is valid
check_extractmetadata <- function(name, val, tip){

  result <-  check_islogical(name, val)

  if(!is.null(tip)){
    tip <- toupper(tip)

    if(val && tip != "M" && tip != "AM" && tip != "MA"){
      result <- FALSE
      stop(sprintf("when %s is set TRUE, tip must be equal to 'M' or 'AM'", name))
    }
  }else{
    if(val){
      result <- FALSE
      stop(sprintf("when %s is set TRUE, tip must be equal to 'M' or 'AM'", name))
    }
  }

  return(result)
}

# Obtain an unique dataframe from a list of dataframes
unnest_data <- function(datain){

  # We have a dataframe
  if(is.data.frame(datain)){
    # Discard data and notas columns
    sel <- tolower(names(datain)) != "data" & tolower(names(datain)) != "notas"

    # Dataframe header without the data and notas columns
    dataout <- as.data.frame(datain[c(),sel])
    names(dataout) <- names(datain)[sel]

    # Data Dataframes of the list not empty
    datasel <- lengths(datain$Data) > 0

    # Only dataframes with Data
    datacol <- datain$Data[datasel]

    # Dataframes without Data
    nodata <- datain[!datasel,sel]

    # Go through all the dataframes with data
    if(length(datacol) > 0){
      # index of the dataframe with lower number of data columns
      ind <- which.min(lengths(datacol))

      # Data columns
      col_data <-   as.data.frame(datacol[[ind]][c(),])
      names(col_data) <- names(datacol[[ind]])

      # Adding Data column to the header
      dataout <- cbind(dataout,col_data)

      # Repeat each row by the number of data values
      tmp <- datain[rep(seq_len(nrow(datain)), times = sapply(datain$Data, nrow)), sel]

      # Unique dataframe of data normalizing the data columns
      data <- do.call(rbind, lapply(datain$Data, function(x) if (length(x) > 0) subset(x, select = names(col_data))))

      # Adding data
      dataout <- cbind(tmp,data)

      # In case we have only one column
      if(sum(sel) == 1){
        names(dataout)[1] <- names(datain)[1]
      }
    }

    # In case of dataframes without Data
    if(!is.null(nrow(nodata)) && nrow(nodata) > 0){

      if(length(datacol) > 0){
        # index of the dataframe with more values of data
        ind <- which.max(lapply(datacol, nrow))

        # Dataframe with values
        data <- datacol[[ind]][names(col_data)]

        # Change value for NA
        data$Valor <- NA

        # Repeat each row by the number of data values
        tmp <- nodata[rep(seq_len(nrow(nodata)), each = nrow(data)),]

        # Repeat each row by the number of nodata rows
        nodata <- data[rep(seq_len(nrow(data)), times = nrow(nodata)),]

        # Adding columns
        nodata <- cbind(tmp, nodata)

        # Adding rows
        dataout <- rbind(dataout, nodata)

      }else{
        dataout <- datain
      }
    }
  }

  # We have a list (series case)
  if(is.list(datain) && !is.data.frame(datain)){
    # Selection of single columns
    sel <- lengths(datain) == 1

    # Selection of metadata
    selmeta <- lengths(datain) > 1 & tolower(names(datain)) != "data" & tolower(names(datain)) != "notas"

    # Dataframe without metadata, data and notas columns
    tmp <- as.data.frame(datain[sel])

    # Adding metadata
    if(sum(selmeta) > 0){
      for(n in names(selmeta)){
        if(selmeta[[n]]){
          tmp[[n]] <- datain[n]
        }
      }
    }

    # Repeat each row by the number of data values
    tmp <- tmp[rep(seq_len(nrow(tmp)), each = max(lengths(datain$Data))),]

    # Obtain data
    data <- datain$Data

    # Adding data
    dataout <- cbind(tmp,data)
  }

  return(dataout)
}


# Check if the table is a px file or not
check_type_table <- function(idTable, verbose = FALSE, validate = FALSE, lang = "ES"){

  # Get the groups of the table
  groups <- get_metadata_table_groups(idTable = idTable, verbose = verbose, validate = validate, lang = lang)

  # If the result of the query is null is a px table
  result <- if(is.null(groups)) TRUE else FALSE
  origin <- if(result) "tablepx" else "tablet3"

  return(list(groups = groups, ispxtable = result, origin = origin))
}

check_alias_filter <- function(f){

  val <- f

  # Get variables
  n <- names(f)

  # Check if there are variables with aliases
  login <- grepl("~", n)

  # Check if there are values with aliases
  logif <- grepl("~", f)

  if(sum(login) != sum(logif)){
    stop("Filter aliases missing")

  }else{
    if(sum(login) > 0){
      # Get variables without aliases
      var <- unlist(lapply(n, function(x) gsub("~id|~cod","",x)))

      # Check if the aliases are valid
      if(length(grep("~", var)) > 0){
        stop("The alias in the variables of the filter is not valid. Valid aliases are 'id' and 'cod'")
      }

      # Get values without aliases
      val <- lapply(f, function(x) gsub("~id|~cod","",tolower(x)))

      # Check if the aliases are valid
      if(length(grep("~", unlist(val))) > 0){
        stop("The alias in the values of the filter is not valid. Valid aliases are 'id' and 'cod'")
      }

      # Get aliases of  variables
      varalias <- rep(unlist(lapply(strsplit(names(f), "~"),
                                    function(x) {if(length(x) > 1) x[2] else ""})),
                      times = lengths(f))

      # Get aliases of values
      valalias <- unlist(lapply(strsplit(unlist(f, use.names = FALSE), "~"),
                                function(x) {if(length(x) > 1) x[2] else ""}))

      if(sum(varalias == valalias) != length(valalias)){
        stop("The alias of a variable and its values must be the same. Valid aliases are 'id' and 'cod'")
      }

      names(val) <- var
    }
  }

  return(val)
}

# Extract metadata information from tables into columns
extract_metadata <- function(datain, request){
  # Obtain metadata
  metadata <- datain$MetaData

  dataout <- datain

  # Metadata columns to extract
  metacols <- character()
  metacolsnames <- character()

  if(request$addons$metanames){
    metacols <- append(metacols, "Nombre")
    metacolsnames <- append(metacolsnames, "")
  }

  if(request$addons$metacodes){
    metacols <- append(metacols, "Codigo")
    metacolsnames <- append(metacolsnames, ".Codigo")
  }

  # Tables
  if(grepl("IdTable",request$definition$tag, ignore.case = TRUE)){

    # check the type of the table
    checktable <- check_type_table(idTable = request$definition$input, lang = request$definition$lang)

    # Check if exits and id for variables
    existsvarid <- exists_variables_id(metadata)

    # Case one: tpx or px table
    if(checktable$ispxtable){
      # Number of variables in metadata information
      nummeta <- min(unique(do.call(rbind,lapply(metadata,nrow))))

      # Column to extract metadata
      varmeta <- if(exists_values_id(metadata) && existsvarid$result) existsvarid$name else column_names[[API_version]][["variable.code"]][["es"]]

      if(exists_values_id(metadata)){
        metacols <- append(metacols, "Id")
        metacolsnames <- append(metacolsnames, ".Id")
      }

      # Obtain variable codes for each row in metadata information
      varcode <- list()

      for(i in 1:nummeta){
        varcode <- append(varcode,
                          as.data.frame(unique(do.call(rbind,
                                                       lapply(metadata, '[',c(i),c(varmeta))))))
      }

      # Loop through all variables
      for(var in varcode){

        # Select a variable code and build a Unique dataframe of variable names
        dfcodes <- do.call(rbind,
                           lapply(metadata,
                                  function(x) subset(x,
                                                     x[[varmeta]] %in% var,
                                                     select = metacols)))

        # Rename column with variable code
        newname <- paste0(gsub("\\s+",".", var), collapse = "_")
        names(dfcodes) <- paste0(newname, metacolsnames)

        # Adding column to dataframe
        if(nrow(dfcodes) == nrow(dataout)){
          dataout <- cbind(dataout, dfcodes)
        }
      }
      # Case two: tempus table
    }else{

      if(request$addons$metacodes){
        metacols <- append(metacols, "Id")
        metacolsnames <- append(metacolsnames, ".Id")
      }

      # Loop through all groups
      for (g in checktable$groups$Id){
        # Get th values of the group
        values <- get_metadata_table_values(idTable = request$definition$input, idGroup = g, validate = FALSE, lang = request$definition$lang, det = 2)

        # name of the variable column
        colname <- if(tolower(existsvarid$name) == "t3_variable") "variable.name" else "variable.id"

        # Select a variable id and build a Unique dataframe of variable names
        dfcodes <- do.call(rbind,
                           lapply(metadata,
                                  function(x) subset(x,
                                                     x[[existsvarid$name]] %in% unique(values[[column_names[[API_version]][[colname]][["es"]]]]),
                                                     select = metacols)))

        # New name of the column
        newname <- unlist(subset(checktable$groups, checktable$groups$Id == g, select = c("Nombre")))
        newname <- gsub("\\s+",".", newname)

        # Rename column
        names(dfcodes) <- paste0(newname, metacolsnames)

        # Adding column to dataframe
        dataout <- cbind(dataout, dfcodes)
      }
    }
    # Series
  }else{

    # Number of variables in metadata information
    nummeta <- min(unique(do.call(rbind,lapply(metadata,nrow))))

    # Obtain variable codes for each row in metadata information
    varcode <- list()
    varname <- list()

    for(i in 1:nummeta){
      varcode <- append(varcode,
                        as.data.frame(unique(do.call(rbind,
                                                     lapply(metadata, '[',c(i),c("Variable.Id"))))))
      varname <- append(varname,
                        as.data.frame(unique(do.call(rbind,
                                                     lapply(metadata, '[',c(i),c("Variable.Nombre"))))))
    }

    if(request$addons$metacodes){
      metacols <- append(metacols, "Id")
      metacolsnames <- append(metacolsnames, ".Id")
    }

    # Loop through all variables
    k <- 1
    for(var in varcode){

      # Select a variable code and build a Unique dataframe of variable names
      dfcodes <- do.call(rbind,
                         lapply(metadata,
                                function(x) subset(x,
                                                   x[[column_names[[API_version]][["variable.id"]][["es"]]]] %in% var,
                                                   select = metacols)))

      # Rename column with variable code
      newname <- gsub("\\s+",".", varname[[k]][1])
      names(dfcodes) <- paste0(newname, metacolsnames)

      # Adding column to dataframe
      if(nrow(dfcodes) == nrow(dataout)){
        dataout <- cbind(dataout, dfcodes)
      }
      k <- k + 1
    }
  }

  return(dataout)
}

# Get metadata information about the variables and values present in a table
get_metadata_variable_values_table <- function(idTable, filter = NULL, verbose, validate, lang, progress = FALSE, det = 0, request = NULL){

  # Check the type of the table
  checktable <- check_type_table(idTable = idTable, validate = validate, verbose = verbose, lang = lang)

  dfvalues <- NULL
  origin <- ""
  # Make sure the response is valid or null
  if(!check_result_status(checktable$groups)){

    # The table is in px or tpx format
    if(checktable$ispxtable){
      origin <- checktable$origin

      # internal call of the funtion
      #if(is.null(request)){

        if(progress){
          cat(sprintf("- Processing filter: %s%%        \r", 50))
        }

        # Obtain metadata information
        df <- get_metadata_series_table(idTable = idTable, filter = filter, tip = "M", validate = FALSE, verbose = verbose, lang = lang, det = det)

        # Check if exits and id for variables
        existsvarid <- exists_variables_id(df$MetaData)

        # If exists variable's id and value's id add new origin and include ids in the selection
        if(exists_values_id(df$MetaData) && existsvarid$result) {
          #selcol <- c("Nombre", "Codigo", "Id", "Variable.Nombre","Variable.Codigo", existsvarid$name)
          selcol <- names(df$MetaData[[1]])[grepl("id|nombre|codigo|variable", names(df$MetaData[[1]]), ignore.case = TRUE)]
          origin <- "tablepxid"

        }else{
          #selcol <- c("Nombre", "Codigo", "Variable.Nombre","Variable.Codigo")
          selcol <- names(df$MetaData[[1]])[grepl("id|nombre|codigo|variable", names(df$MetaData[[1]]), ignore.case = TRUE)]
        }

        # Get the metadata with information of variables and values
        dfvalues <- lapply(df$MetaData,
                           function(x) subset(x, select = selcol))

        dfvalues <- unique(do.call(rbind, dfvalues))

      # call from get_metadata_table_varval
      # }else{
      #   # if(is.null(filter)){
      #     if(progress){
      #       cat(sprintf("- Processing filter: %s%%        \r", 50))
      #     }
      #
      #     # Obtain metadata information
      #     df <- get_metadata_series_table(idTable = idTable, filter = filter, tip = "M", validate = FALSE, verbose = verbose, lang = lang)
      #
      #     # Check if exits and id for variables
      #     existsvarid <- exists_variables_id(df$MetaData)
      #
      #     # If exists variable's id and value's id add new origin and include ids in the selection
      #     if(exists_values_id(df$MetaData) && existsvarid$result) {
      #       selcol <- names(df$MetaData[[1]])[grepl("id|nombre|codigo|variable", names(df$MetaData[[1]]), ignore.case = TRUE)]
      #       origin <- "tablepxid"
      #
      #     }else{
      #       selcol <- names(df$MetaData[[1]])[grepl("id|nombre|codigo|variable", names(df$MetaData[[1]]), ignore.case = TRUE)]
      #     }
      #
      #     # Get the metadata with information of variables and values
      #     dfvalues <- lapply(df$MetaData,
      #                        function(x) subset(x, select = selcol))
      #
      #     dfvalues <- unique(do.call(rbind, dfvalues))

        # }else{
        #   if(validate){
        #     # rerieve the filter values
        #     lvarval <- get_parameters_filter(request)
        #
        #     # filter values
        #     fvarval <- lvarval$filter
        #
        #     # dataframe of variables and values ids
        #     dfvalues <-lvarval$df
        #
        #     if(origin == "tablepxid"){
        #       # create column variable_id:value_id
        #       dfvalues$varval <- paste(as.character(dfvalues[, column_names[[API_version]][["variable.id"]][["es"]]]), as.character(dfvalues[, column_names[[API_version]][["value.id"]][["es"]]]), sep = ":")
        #
        #     }else{
        #       # create column variable_cod:value_cod
        #       dfvalues$varval <- paste(as.character(dfvalues[, column_names[[API_version]][["variable.code"]][["es"]]]), as.character(dfvalues[, column_names[[API_version]][["value.code"]][["es"]]]), sep = ":")
        #     }
        #
        #     # vector of var_id:val_id
        #     fvarval <- unique(unname(unlist(fvarval)))
        #
        #     # vector of var_id
        #     fvar <- unique(unlist(lapply(strsplit(fvarval, ":"), `[[`, 1)))
        #
        #     # retrieve varval in the filter
        #     df1 <- subset(dfvalues, dfvalues$varval %in% fvarval)
        #
        #     if(origin == "tablepxid"){
        #       # retrieve variables not in the filter
        #       df2 <- subset(dfvalues, !(dfvalues[[column_names[[API_version]][["variable.id"]][["es"]]]] %in% fvar))
        #     }else{
        #       # retrieve variables not in the filter
        #       df2 <- subset(dfvalues, !(dfvalues[[column_names[[API_version]][["variable.code"]][["es"]]]] %in% fvar))
        #     }
        #
        #     dfvalues <- rbind(df1, df2)
        #     dfvalues$varval <- NULL
        #     rm(df1)
        #     rm(df2)
        #
        #   }else{
        #     if(progress){
        #       cat(sprintf("- Processing filter: %s%%        \r", 50))
        #     }
        #
        #     # Obtain metadata information
        #     df <- get_metadata_series_table(idTable = idTable, filter = filter, tip = "M", validate = FALSE, verbose = verbose, lang = lang)
        #
        #     # Check if exits and id for variables
        #     existsvarid <- exists_variables_id(df$MetaData)
        #
        #     # If exists variable's id and value's id add new origin and include ids in the selection
        #     if(exists_values_id(df$MetaData) && existsvarid$result) {
        #       selcol <- names(df$MetaData[[1]])[grepl("id|nombre|codigo|variable", names(df$MetaData[[1]]), ignore.case = TRUE)]
        #       origin <- "tablepxid"
        #
        #     }else{
        #       selcol <- names(df$MetaData[[1]])[grepl("id|nombre|codigo|variable", names(df$MetaData[[1]]), ignore.case = TRUE)]
        #     }
        #
        #     # Get the metadata with information of variables and values
        #     dfvalues <- lapply(df$MetaData,
        #                        function(x) subset(x, select = selcol))
        #
        #     dfvalues <- unique(do.call(rbind, dfvalues))
        #   }
        # }
      #}
    # The table is stored in tempus
    }else{
      origin <- checktable$origin

      # internal call of the funtion
      if(is.null(request)){

        i <- 1
        for(g in checktable$groups$Id){
          if(progress){
            cat(sprintf("- Processing filter: %s%%        \r", round(i/nrow(checktable$groups)*50,0)))
            i <- i + 1
          }

          df <- get_metadata_table_values(idTable = idTable, idGroup = g, det = det, validate = FALSE, lang = lang, verbose = verbose)

          if(!is.null(df)){
            #df <- subset(df, select = c("Id", "Fk_Variable", "Nombre", "Codigo"))
            df <- subset(df, select = names(df)[grepl("id|nombre|codigo|variable", names(df), ignore.case = TRUE)])
            df$group <- g
          }

          if (exists("dfvalues") && is.data.frame(get("dfvalues"))){
            dfvalues <- rbind(dfvalues,df)
          }else{
            dfvalues <- df
          }
        }

        # create column fk_variable:id
        if(det > 0 ){
          dfvalues$varval <- paste(as.character(dfvalues[, column_names[[API_version]][["variable.id"]][["es"]]]), as.character(dfvalues[, column_names[[API_version]][["value.id"]][["es"]]]), sep = ":")

        }else{
          dfvalues$varval <- paste(as.character(dfvalues[, column_names[[API_version]][["variable.fk"]][["es"]]]), as.character(dfvalues[, column_names[[API_version]][["value.id"]][["es"]]]), sep = ":")
        }

      # call from get_metadata_table_varval
      }else{
        if(is.null(filter)){
          i <- 1
          for(g in checktable$groups$Id){
            if(progress){
              cat(sprintf("- Processing filter: %s%%        \r", round(i/nrow(checktable$groups)*50,0)))
              i <- i + 1
            }

            df <- get_metadata_table_values(idTable = idTable, idGroup = g, det = det, validate = FALSE, lang = lang, verbose = verbose)

            if(!is.null(df)){
              #df <- subset(df, select = c("Id", "Fk_Variable", "Nombre", "Codigo"))
              df <- subset(df, select = names(df)[grepl("id|nombre|codigo|variable", names(df), ignore.case = TRUE)])
            }

            if (exists("dfvalues") && is.data.frame(get("dfvalues"))){
              dfvalues <- rbind(dfvalues,df)
            }else{
              dfvalues <- df
            }
          }
        }else{

          # rerieve the filter values
          lvarval <- get_parameters_filter(request)

          # filter values
          fvarval <- lvarval$filter

          if(validate){
            # dataframe of variables and values ids
            dfvalues <-lvarval$df
          }else{
            i <- 1
            for(g in checktable$groups$Id){
              if(progress){
                cat(sprintf("- Processing filter: %s%%        \r", round(i/nrow(checktable$groups)*50,0)))
                i <- i + 1
              }

              df <- get_metadata_table_values(idTable = idTable, idGroup = g, det = det, validate = FALSE, lang = lang, verbose = verbose)

              if(!is.null(df)){
                #df <- subset(df, select = c("Id", "Fk_Variable", "Nombre", "Codigo"))
                df <- subset(df, select = names(df)[grepl("id|nombre|codigo|variable", names(df), ignore.case = TRUE)])
                df$group <- g
              }

              if (exists("dfvalues") && is.data.frame(get("dfvalues"))){
                dfvalues <- rbind(dfvalues,df)
              }else{
                dfvalues <- df
              }
            }
          }

          # vector of var_id:val_id
          fvarval <- unique(unname(unlist(fvarval)))

          # vector of var_id
          fvar <- unique(unlist(lapply(strsplit(fvarval, ":"), `[[`, 1)))

          # create column fk_variable:id
          if(det > 0 ){
            dfvalues$varval <- paste(as.character(dfvalues[, column_names[[API_version]][["variable.id"]][["es"]]]), as.character(dfvalues[, column_names[[API_version]][["value.id"]][["es"]]]), sep = ":")

            # group id of the variables in the filter
            fgroup <- subset(dfvalues, dfvalues[[column_names[[API_version]][["variable.id"]][["es"]]]] %in% fvar, select = "group")

          }else{
            dfvalues$varval <- paste(as.character(dfvalues[, column_names[[API_version]][["variable.fk"]][["es"]]]), as.character(dfvalues[, column_names[[API_version]][["value.id"]][["es"]]]), sep = ":")

            # group id of the variables in the filter
            fgroup <- subset(dfvalues, dfvalues[[column_names[[API_version]][["variable.fk"]][["es"]]]] %in% fvar, select = "group")
          }

          fgroup <- unique(unlist(fgroup))

          # retrieve varval in the filter
          df1 <- subset(dfvalues, dfvalues$varval %in% fvarval)

          # retrieve variables not in the filter
          if(det > 0 ){
            df2 <- subset(dfvalues, !(dfvalues[[column_names[[API_version]][["variable.id"]][["es"]]]] %in% fvar) & !(dfvalues$group %in% fgroup))
          }else{
            df2 <- subset(dfvalues, !(dfvalues[[column_names[[API_version]][["variable.fk"]][["es"]]]] %in% fvar) & !(dfvalues$group %in% fgroup))
          }

          dfvalues <- rbind(df1, df2)
          dfvalues$varval <- NULL
          dfvalues$group <- NULL
          rm(df1)
          rm(df2)
        }
      }
    }
  }

  return(list(origin = origin, values = dfvalues))
}

# Get metadata information about the variables and values present in an operation
get_metadata_variable_values_operation <- function(operation, verbose, validate, lang, progress = FALSE, det = 0, request = NULL){

  dfvalues <- NULL

  # We obtain the variables from the operation of the series
  opevar <- get_metadata_variables(operation = operation, validate = validate, verbose = verbose, lang = lang, page = 0)

  # We obtain the values of all the variables
  i <- 1
  for(var in opevar$Id){
    if(progress){
      cat(sprintf("- Processing filter: %s%%        \r", round(i/nrow(opevar)*50,0)))
      i <- i + 1
    }

    tmp <- get_metadata_values(operation = operation, variable = var, validate = FALSE, verbose = verbose, lang = lang, page = 0, det = det)

    selcol <- names(tmp)[grepl("id|nombre|codigo|variable", names(tmp), ignore.case = TRUE)]
    tmp <- subset(tmp, select = selcol)
    #tmp <- subset(tmp, select = c("Id","Fk_Variable","Nombre","Codigo"))

    # create column fk_variable:id
    if(det > 0 ){
      tmp$varval <- paste(as.character(tmp[, column_names[[API_version]][["variable.id"]][["es"]]]), as.character(tmp[, column_names[[API_version]][["value.id"]][["es"]]]), sep = ":")

    }else{
      tmp$varval <- paste(as.character(tmp[, column_names[[API_version]][["variable.fk"]][["es"]]]), as.character(tmp[, column_names[[API_version]][["value.id"]][["es"]]]), sep = ":")
    }

    if (exists("dfvalues") && is.data.frame(get("dfvalues"))){
      dfvalues <- rbind(dfvalues,tmp)
    }else{
      dfvalues <- tmp
    }
  }

  if(!is.null(request)){
    dfvalues$varval <- NULL
  }

  return(list(origin = "series", values = dfvalues))
}

# Get metadata information about the variables and values present in a hierarchy tree
get_metadata_variable_values_hierarchy <- function(variable, hierarchy, verbose, validate, lang, progress = FALSE, det = 0){

  dfvalues <- NULL

  values <- get_metadata_values(variable = variable, validate = FALSE, verbose = verbose, lang = lang, page = 0, det = det, hierarchy = hierarchy)


  # We obtain the values of all the variables
  for(i in 0:hierarchy){
    if(progress){
      cat(sprintf("- Processing filter: %s%%        \r", round(i/hierarchy*50,0)))
    }

    tmp <- subset(values, select = grepl(paste0("_", i, "$"), names(values)))
    names(tmp) <- gsub(paste0("_", i, "$"), "", names(tmp))
    tmp$level <- i

    selcol <- names(tmp)[grepl("id|nombre|codigo|variable|jerarquia|level", names(tmp), ignore.case = TRUE)]

    tmp <- unique(subset(tmp, select = selcol))


    # create column fk_variable:id
    if(det > 0 ){
      tmp$varval <- paste(as.character(tmp[, column_names[[API_version]][["variable.id"]][["es"]]]), as.character(tmp[, column_names[[API_version]][["value.id"]][["es"]]]), sep = ":")
      valfathers <- if(sum(grepl(column_names[[API_version]][["jerarquia"]][["es"]], names(tmp))) > 0) sapply(tmp[[column_names[[API_version]][["jerarquia"]][["es"]]]],`[[`, column_names[[API_version]][["id"]][["es"]]) else NA

    }else{
      tmp$varval <- paste(as.character(tmp[, column_names[[API_version]][["variable.fk"]][["es"]]]), as.character(tmp[, column_names[[API_version]][["value.id"]][["es"]]]), sep = ":")
      valfathers <- if(sum(grepl(column_names[[API_version]][["jerarquia.fk"]][["es"]], names(tmp))) > 0) tmp[[column_names[[API_version]][["jerarquia.fk"]][["es"]]]] else NA
    }

    tmp$fathers <- valfathers

    if (exists("dfvalues") && is.data.frame(get("dfvalues"))){
      dfvalues <- rbind(dfvalues,tmp)
    }else{
      dfvalues <- tmp
    }
  }

  return(list(origin = "variables", values = dfvalues))
}

# chek if there are negatives values in the filter and remove them
check_negative_values <- function(dfval, variable, values, origin){

  # check if there is any negative value
  neg <- grepl("^-.*", as.character(values))

  if(sum(neg) > 0){
    # remove minus sign
    neg_values <- gsub("^-", "", as.character(values[neg]))

    dfvalues <- NULL
    if(!is.null(dfval)){
      if(origin == "tablepx"){
        # We select only the values of variables present in the filter
        dfvalfilter <- subset(dfval,
                              dfval[[column_names[[API_version]][["variable.code"]][["es"]]]] %in% variable)

        # remove negative values
        dfvalues <- subset(dfvalfilter[[column_names[[API_version]][["value.code"]][["es"]]]],
                           !(dfvalfilter[[column_names[[API_version]][["value.code"]][["es"]]]] %in% neg_values))

        dfvalues <- paste0(variable, ":", dfvalues)

      }else if(origin == "tablepxid"){
        check_alias <- grepl("~id$|~cod$", variable)

        # variables and values without aliases
        variable <- gsub("~id$|~cod$","",as.character(variable))
        neg_values <- gsub("~id$|~cod$","",neg_values)

        # We select only the values of variables present in the filter
        dfvalfilter <- subset(dfval, dfval[[column_names[[API_version]][["variable.code"]][["es"]]]] %in% variable | dfval[[column_names[[API_version]][["variable.id"]][["es"]]]] %in% variable)

        if(check_alias){
          # remove negative values.
          dfvalues <- subset(dfvalfilter[[column_names[[API_version]][["value.id"]][["es"]]]],
                              !(dfvalfilter[[column_names[[API_version]][["value.id"]][["es"]]]] %in% neg_values))

          # include alias ~
          dfvalues <- if(length(dfvalues) > 0)  paste0(paste0(variable, "~id"), ":", paste0(dfvalues, "~id")) else NULL

        # Case with codes
        }else{
          # remove negative values
          dfvalues <- subset(dfvalfilter[[column_names[[API_version]][["value.code"]][["es"]]]],
                              !(dfvalfilter[[column_names[[API_version]][["value.code"]][["es"]]]] %in% neg_values) & grepl("\\S+", dfvalfilter[[column_names[[API_version]][["value.code"]][["es"]]]]))

          dfvalues <- if(length(dfvalues) > 0) paste0(variable, ":", dfvalues) else NULL
        }

      }else if(origin %in% c("series", "variables")){
        # column name depending on det parameter
        colname <- if(sum(grepl(column_names[[API_version]][["variable.fk"]][["es"]], names(dfval), ignore.case = TRUE)) > 0) column_names[[API_version]][["variable.fk"]][["es"]] else column_names[[API_version]][["variable.id"]][["es"]]

        # We select only the values of variables present in the filter
        dfvalfilter <- subset(dfval, dfval[[colname]] %in% variable)

        # remove negative values
        dfvalues <- subset(dfvalfilter,
                           !(dfvalfilter[[column_names[[API_version]][["value.id"]][["es"]]]] %in% neg_values), select = "varval")

        dfvalues <- unique(unlist(dfvalues))

      }else{
        # group id of the variables in the filter
        fgroup <- subset(dfval, dfval[[column_names[[API_version]][["value.id"]][["es"]]]] %in% neg_values, select = "group")

        # We select only the values of variable present in the filter
        dfvalfilter <- subset(dfval, dfval$group %in% unique(unlist(fgroup)))

        # remove negative values
        dfvalues <- subset(dfvalfilter,
                           !(dfvalfilter[[column_names[[API_version]][["value.id"]][["es"]]]] %in% neg_values), select = "varval")

        dfvalues <- unique(unlist(dfvalues))
      }
    }
  }else{
    dfvalues <- paste0(variable, ":", values)
  }

  return(dfvalues)
}

# remove negative sign from filter values
remove_filter_negative_values <- function(filter){

  if(is.list(filter)){
    # remove minus sign
    nfilter <- lapply(filter, function(x) gsub("^-", "", as.character(x)))
  }else{
    nfilter <- filter
  }

  return(nfilter)
}

# check if in the filter there are negative values
is_negative_filter_values <- function(filter){
  r <- FALSE
  if(is.list(filter)){
    s <- sum(unlist(lapply(filter, function(x) grepl("^-.*", as.character(x)))))
    r <- if(s > 0) TRUE else FALSE
  }

  return(r)
}

# get values hierarchy tree
get_values_hierarchy <- function(father, request){

  # depth of the hierarchy tree
  depth <- request$addons$hierarchy

  # get the filter
  filter <- NULL
  dfilter <- NULL

  if(!is.null(request$addons$filter)){
    filter <- get_addons_filter(request)

    # data frame
    dfilter <- filter$df

    # variables and values to filter
    filter <- filter$filter

    # dataframe use as father
    father <- if(is.null(dfilter)) father else subset(dfilter, dfilter$level == 0, !grepl("varval|level|fathers", names(dfilter), ignore.case = TRUE))
  }

  # First level column names
  names(father) <- paste(names(father), 0, sep = "_")

  # copy of input data
  data <- father

  # column names
  cols <- names(father)

  # for each level of the tree we get the children
  for (l in 1:depth){
    children <- get_children(l - 1, father, request$addons$verbose, filter, request$parameters$det, dfilter)

    # vector of column names
    cols <- append(cols, names(children))
    cols <- subset(cols, !grepl("padre", cols))

    # join the data by id of the father
    if(!is.null(children)){
      data <- merge(data, children, by.x = paste("Id", l - 1, sep = "_"), by.y = paste("padre", l, sep = "_"))
      data <- data[,cols]
      father <- children
    }
  }

  return(data)
}

get_children <- function(depth, df, verbose, filter, det, dfilter = NULL){
  data <- NULL

  # variables of the filter
  varfilter <- if(is.null(filter)) NULL else sapply(strsplit(unlist(filter), ":"), `[`, 1)

  # values of the filter
  valfilter <- if(is.null(filter)) NULL else sapply(strsplit(unlist(filter), ":"), `[`, 2)

  # column name of the variable
  varcol <- if(det > 0 ) paste(column_names[[API_version]][["variable.id"]][["es"]], depth, sep = "_") else paste(column_names[[API_version]][["variable.fk"]][["es"]], depth, sep = "_")

  # column name of the value
  valcol <- paste(column_names[[API_version]][["value.id"]][["es"]], depth, sep = "_")

  # filter variables
  v <- unique(df[[varcol]])
  v <- if(sum(is.element(varfilter, v)) > 0) intersect(varfilter, v) else v

  for( var in v){
    s <- subset(df, df[[varcol]] == var)

    # filter values
    va <- unique(s[[valcol]])
    va <- if(sum(is.element(valfilter, va)) > 0) intersect(valfilter, va) else va

    for(val in va){
      if(is.null(dfilter)){
        tmp <- get_metadata_values(variable = var, value = val, verbose = verbose, validate = FALSE, det = det)

      }else{
        # we use the dataframe from the filter
        tmp <- subset(dfilter, dfilter$level == depth + 1)

        # filter values
        rowsel <- sapply(tmp$fathers, function(x) is.element(val, x))
        tmp <-subset(tmp, rowsel, !grepl("varval|level|fathers", names(dfilter), ignore.case = TRUE))
      }

      if(!is.null(tmp)){
        # extract the father of the children
        if(det > 0){
          valfathers <- if(sum(grepl(column_names[[API_version]][["jerarquia"]][["es"]], names(tmp))) > 0) sapply(tmp[[column_names[[API_version]][["jerarquia"]][["es"]]]],`[[`, column_names[[API_version]][["id"]][["es"]]) else NA
        }else{
          valfathers <- if(sum(grepl(column_names[[API_version]][["jerarquia.fk"]][["es"]], names(tmp))) > 0) tmp[[column_names[[API_version]][["jerarquia.fk"]][["es"]]]] else NA
        }

        tmp$padre <- unlist(sapply(valfathers , function(x) intersect(val, x)))

        # filter variables in the last level
        v2 <- if(det > 0)  unique(tmp[[column_names[[API_version]][["variable.id"]][["es"]]]]) else unique(tmp[[column_names[[API_version]][["variable.fk"]][["es"]]]])
        v2 <- if(sum(is.element(varfilter, v2)) > 0) intersect(varfilter, v2) else v2

        if(det > 0){
          tmp <-subset(tmp, tmp[[column_names[[API_version]][["variable.id"]][["es"]]]] %in% v2)
        }else{
          tmp <-subset(tmp, tmp[[column_names[[API_version]][["variable.fk"]][["es"]]]] %in% v2)
        }
        names(tmp) <- paste(names(tmp), depth + 1, sep = "_")
        data <- rbind(data, tmp)

      }
    }
  }

  # filter variables in the last level
  varcol_new <- if(det > 0) paste(column_names[[API_version]][["variable.id"]][["es"]], depth + 1, sep = "_") else paste(column_names[[API_version]][["variable.fk"]][["es"]], depth + 1, sep = "_")

  v3 <- unique(data[[varcol_new]])
  v3 <- if(sum(is.element(varfilter, v3)) > 0) intersect(varfilter, v3) else v3

  data <-subset(data, data[[varcol_new]] %in% v3)

  return(data)
}


# Get url filter
get_addons_filter <- function(request){

  val <- request$addons[["filter"]]

  lval <- NULL
  if(!is.null(val)){
    lval <- build_filter(val, request$definition, request$addons, request$check$addons$filter, det = request$parameters$det)
  }

  return(lval)
}

#' Get all tables for a given operation
#'
#' @param operation (string): code of the operation. To obtain a list of
#' available operations see [get_metadata_operations()].
#' @param det (int): level of detail. Valid values: 0, 1 or 2.
#' @param tip (string): set to 'A' for friendly output (e.g. readable dates),
#' set to 'M' to include metadata or set to 'AM' for both.
#' @param geo (int): set to 0 for national tables or set to 1 for tables with
#' a greater level of disaggregation.
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English
#' @param page (int): page number. The retrieved result of the query is paginated (page=0 retrieves all pages).
#' @param validate (logical): validate input parameters. A FALSE value means fewer API calls.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#'
#' @return Data frame with information of the available tables according to the operation specified in the function
#'
#' @examplesIf interactive()
#' # Get all the tables of the "IPC" operation
#' df <- get_metadata_tables_operation(operation = "IPC", validate = FALSE)
#' head(df)
#'
#' @export
get_metadata_tables_operation <- function(operation = NULL, det = 0, tip = NULL, geo = NULL, lang = "ES", page = 0, validate = TRUE, verbose = FALSE){

  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, list(fun = "TABLAS_OPERACION"))
  definition <- append(definition, list(input = operation))
  definition <- append(definition, list(tag = "operation"))

  # List of parameters to call the API
  parameters <- list()
  parameters <- append(parameters, list(det = det))
  parameters <- append(parameters, list(tip = tip))
  parameters <- append(parameters, list(geo = geo))
  parameters <- append(parameters, list(page = page))

  # List of addons
  addons <- list(validate = validate, verbose = verbose)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Build the complete URL to call the API
  url <- get_url(request)

  # Obtain the retrieved data calling the API
  data <- get_api_data_all_pages(url, request)

  return(data)
}

#' Get all groups for a specific a table
#'
#' @param idTable (int): id of the table. For further information about
#' ids click this [link](https://es-ine.github.io/ineapir/articles/identify_codes.html).
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param validate (logical): validate input parameters.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#'
#' @return Data frame with information of the groups according to the table specified in the function
#'
#' @examplesIf interactive()
#' # Get the groups of the table with identification code "76125"
#' df <- get_metadata_table_groups(idTable = 76125)
#' head(df)
#'
#' @export
get_metadata_table_groups <- function(idTable = NULL, lang = "ES", validate = TRUE, verbose = FALSE){

  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, list(fun = "GRUPOS_TABLA"))
  definition <- append(definition, list(input = idTable))
  definition <- append(definition, list(tag = "idTable"))

  # List of parameters to call the API
  parameters <- list()

  # List of addons
  addons <- list(validate = validate, verbose = verbose)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Build the complete URL to call the API
  url <- get_url(request)

  # Obtain the retrieved data calling the API
  data <- get_api_data(url, request)

  return(data)
}

#' Get all values for a specific table group
#'
#' @param idTable (int): id of the table. For further information about
#' ids click this [link](https://es-ine.github.io/ineapir/articles/identify_codes.html).
#' @param idGroup (int): id of the group of variables. To get all groups
#' for a specific table see [get_metadata_table_groups()].
#' @param det (int): level of detail. Valid values: 0, 1 or 2.
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param validate (logical): validate input parameters. A FALSE value means fewer API calls.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#'
#' @return Data frame with information of the values of a table group according to the table and group specified in the function
#'
#' @examplesIf interactive()
#' # Get the values of the group "155577" of the table with identification
#' # code "76125"
#' df <- get_metadata_table_values(idTable = 76125, idGroup = 155577, validate = FALSE)
#' head(df)
#'
#' @export
get_metadata_table_values <- function(idTable = NULL, idGroup = NULL, det = 0, lang = "ES", validate = TRUE, verbose = FALSE){

  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, list(fun = "VALORES_GRUPOSTABLA"))
  definition <- append(definition, list(input = list(idTable = idTable, idGroup = idGroup)))
  definition <- append(definition, list(tag = "idTable_idGroup"))

  # List of parameters to call the API
  parameters <- list()
  parameters <- append(parameters, list(det = det))

  # List of addons
  addons <- list(validate = validate, verbose = verbose)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Build the complete URL to call the API
  url <- get_url(request)

  # Obtain the retrieved data calling the API
  data <- get_api_data(url, request)

  return(data)
}

#' Get the operation for a given table
#'
#' @param idTable (int): id of the table. For further information about
#' ids click this [link](https://es-ine.github.io/ineapir/articles/identify_codes.html).
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param validate (logical): validate input parameters.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#'
#' @return Data frame with information of the operation according to the table specified in the function
#'
#' @examplesIf interactive()
#' # Get the operation of the table with identification code "76125"
#' df <- get_metadata_operation_table(idTable = 76125)
#' head(df)
#'
#' @export
get_metadata_operation_table <- function(idTable = NULL, lang = "ES", validate = TRUE, verbose = FALSE){

  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, list(fun = "OPERACIONES_TABLA"))
  definition <- append(definition, list(input = idTable))
  definition <- append(definition, list(tag = "idTable"))

  # List of parameters to call the API
  parameters <- list()

  # List of addons
  addons <- list(validate = validate, verbose = verbose)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Build the complete URL to call the API
  url <- get_url(request)

  # Obtain the retrieved data calling the API
  data <- get_api_data(url, request)

  return(data)
}

#' Get metadata information about the variables and values for a given table
#'
#' @param idTable (int): id of the table. For further information about
#' ids click this [link](https://es-ine.github.io/ineapir/articles/identify_codes.html).
#' @param det (int): level of detail. Valid values: 0, 1 or 2.
#' @param filter (list): list of variables and values.
#' ### Filtering data from tables
#' When we request data from tables there is the possibility of filtering data
#' on the fly using metadata information about the variables and their values
#' that define the series. There are different approaches to build the filter depending on the table type.
#'
#' #### Case one: tempus table
#' [URL example](https://www.ine.es/jaxiT3/Tabla.htm?t=76125).
#' For a tempus table the filter is based on ids. The format is `list(id_variable1 = id_value1, id_variable2 = id_value2)`.
#' Besides:
#' - A variable can take more than one value: `list(id_variable1 = c(id_value11, id_value12), id_variable2 = id_value2)`.
#' - A variable can take a empty character "" to get all its possible values: `list(id_variable1 = id_value1, id_variable2 = "")`.
#'
#' #### Case two: px tables
#' [URL example](https://www.ine.es/jaxi/Tabla.htm?path=/t20/e245/p08/l0/&file=01001.px).
#' For a px table the filter is based on codes. The format is `list(cod_variable1 = cod_value1, cod_variable2 = cod_value2)`.
#' Besides:
#' - A variable can take more than one value: `list(cod_variable1 = c(cod_value11, cod_value12), id_variable2 = cod_value2)`.
#' - A variable can take a empty character "" to get all its possible values: `list(cod_variable1 = cod_value1, cod_variable2 = "")`.

#' #### Case three: tpx table
#' [URL example](https://www.ine.es/jaxi/Tabla.htm?tpx=33387&L=0).
#' For a tpx table the filter is based on codes. The format is `list(cod_variable1 = cod_value1, cod_variable2 = cod_value2)`.
#' Besides:
#' - A variable can take more than one value: `list(cod_variable1 = c(cod_value11, cod_value12), id_variable2 = cod_value2)`.
#' - A variable can take a empty character "" to get all its possible values: `list(cod_variable1 = cod_value1, cod_variable2 = "")`.
#'
#' [URL example](https://www.ine.es/jaxi/Tabla.htm?tpx=52056&L=0).
#' There are tpx tables that contain variable ids and value ids. In this case,
#' we can use the ids instead of the codes to build the filter. To do this we add
#' the alias *~id* at the end of each id: `list(id_variable1~id = id_value1~id, id_variable2~id = id_value2~id)`.
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param validate (logical): validate input parameters.A FALSE value means fewer API calls.
#' Therefore, it is recommended to set it to FALSE when there is no doubt about the validity of the input parameters, including the filter.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#'
#' @return Data frame with information about the variables and values that define a table according to the table specified in the function
#'
#' @examplesIf interactive()
#' # Get all the variable and values of the table with identification code "76125"
#' df <- get_metadata_table_varval(idTable = 76125)
#' head(df)
#'
#' # Filter variables and values
#' df <- get_metadata_table_varval(idTable = 52056,
#' filter = list(NAC = "00"), validate = FALSE)
#' head(df)
#'
#' @export
get_metadata_table_varval <- function(idTable = NULL, det = 0, filter = NULL, lang = "ES", validate = TRUE, verbose = FALSE){
  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, list(input = idTable))
  definition <- append(definition, list(tag = "idTable"))

  # List of parameters to call the API
  parameters <- list()
  parameters <- append(parameters, if(is.null(filter)) list(filter = filter) else list(filter = list(idTable = idTable, filter = filter)))
  parameters <- append(parameters, list(det = det))

  # List of addons
  addons <- list(validate = validate, verbose = verbose)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Get the metadata information of the table
  df <- get_metadata_variable_values_table(idTable, filter, verbose, validate, lang, det = det, request = request)

  return(df$values)
}

test_that("test get_metadata_tables_operation", {
  skip_on_cran()
  expect_s3_class(get_metadata_tables_operation(operation = "IPC", validate = FALSE), "data.frame")
})

test_that("test get_metadata_table_groups", {
  skip_on_cran()
  expect_s3_class(get_metadata_table_groups(idTable = 76125), "data.frame")
})

test_that("test get_metadata_table_values", {
  skip_on_cran()
  expect_s3_class(get_metadata_table_values(idTable = 76125, idGroup = 155577, validate = FALSE), "data.frame")
  expect_s3_class(get_metadata_table_values(idTable = 76125, idGroup = 155577, det = 2, validate = FALSE), "data.frame")
})

test_that("test get_metadata_operation_table", {
  skip_on_cran()
  expect_s3_class(get_metadata_operation_table(idTable = 76125), "data.frame")
})

test_that("test get_metadata_table_varval", {
  skip_on_cran()
  expect_s3_class(get_metadata_table_varval(idTable = 76125), "data.frame")
  expect_s3_class(get_metadata_table_varval(idTable = 76125, det = 2), "data.frame")
  expect_s3_class(get_metadata_table_varval(idTable = 76125, validate = FALSE,
                                            filter = list("3" = "74")), "data.frame")
})

#' Get information for a specific series
#'
#' @param codSeries (string): code of the series. For further information about
#' codes click this [link](https://es-ine.github.io/ineapir/articles/identify_codes.html).
#' @param det (int): level of detail. Valid values: 0, 1 or 2.
#' @param tip (string): set to 'A' for friendly output (e.g. readable dates),
#' set to 'M' to include metadata or set to 'AM' for both.
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param validate (logical): validate input parameters.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#'
#' @return Data frame with information of a series according to the code specified in the function
#'
#' @examplesIf interactive()
#' # Get information of time series with code "IPC290750"
#' df <- get_metadata_series(codSeries = "IPC290750")
#' head(df)
#'
#' @export
get_metadata_series <- function(codSeries = NULL, det = 0, tip = NULL, lang = "ES", validate = TRUE, verbose = FALSE){

  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, list(fun = "SERIE"))
  definition <- append(definition, list(input = codSeries))
  definition <- append(definition, list(tag = "codSeries"))

  # List of parameters to call the API
  parameters <- list()
  parameters <- append(parameters, list(det = det))
  parameters <- append(parameters, list(tip = tip))

  # List of addons
  addons <- list(validate = validate, verbose = verbose)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Build the complete URL to call the API
  url <- get_url(request)

  # Obtain the retrieved data calling the API
  data <- get_api_data(url, request)

  return(data)
}

#' Get all the series for a specific operation
#'
#' @param operation (string): code of the operation. To obtain a list of
#' available operations see [get_metadata_operations()].
#' @param det (int): level of detail. Valid values: 0, 1 or 2.
#' @param tip (string): set to 'A' for friendly output (e.g. readable dates),
#' set to 'M' to include metadata or set to 'AM' for both.
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param page (int): page number. The retrieved result of the query is paginated. Default value is set to 1.
#' @param validate (logical): validate input parameters. A FALSE value means fewer API calls.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#'
#' @return Data frame with information of the series belonging to an operation.
#'
#' @examplesIf interactive()
#' # Get metadata of time series from "IPC" operation
#' # Retrieve page 1
#' df <- get_metadata_series_operation(operation = "IPC", validate = FALSE)
#' nrow(df)
#'
#' # Retrieve page 2
#' df <- get_metadata_series_operation(operation = "IPC", validate = FALSE,
#' page = 2)
#' nrow(df)
#'
#' @export
get_metadata_series_operation <- function(operation = NULL, det = 0, tip = NULL, lang = "ES", page = 1 ,validate = TRUE, verbose = FALSE){

  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, list(fun = "SERIES_OPERACION"))
  definition <- append(definition, list(input = operation))
  definition <- append(definition, list(tag = "operation"))

  # List of parameters to call the API
  parameters <- list()
  parameters <- append(parameters, list(det = det))
  parameters <- append(parameters, list(tip = tip))
  parameters <- append(parameters, if(page == 0) list(page = 1) else list(page = page))

  # List of addons
  addons <- list(validate = validate, verbose = verbose)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Build the complete URL to call the API
  url <- get_url(request)

  # Obtain the retrieved data calling the API
  data <- get_api_data(url, request)

  return(data)
}

#' Get all the values for a given series
#'
#' @param codSeries (string): code of the series. For further information about
#' codes click this [link](https://es-ine.github.io/ineapir/articles/identify_codes.html).
#' @param det (int): level of detail. Valid values: 0, 1 or 2.
#' @param tip (string): set to 'A' for friendly output (e.g. readable dates),
#' set to 'M' to include metadata or set to 'AM' for both.
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param validate (logical): validate input parameters.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#'
#' @return Data frame with information of the values of a series according to the code specified in the function
#'
#' @examplesIf interactive()
#' # Get metadata of time series with code "IPC290750"
#' df <- get_metadata_series_values(codSeries = "IPC290750")
#' head(df)
#'
#' @export
get_metadata_series_values <- function(codSeries = NULL, det = 0, tip = NULL, lang = "ES", validate = TRUE, verbose = FALSE){

  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, list(fun = "VALORES_SERIE"))
  definition <- append(definition, list(input = codSeries))
  definition <- append(definition, list(tag = "codSeries"))

  # List of parameters to call the API
  parameters <- list()
  parameters <- append(parameters, list(det = det))
  parameters <- append(parameters, list(tip = tip))

  # List of addons
  addons <- list(validate = validate, verbose = verbose)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Build the complete URL to call the API
  url <- get_url(request)

  # Obtain the retrieved data calling the API
  data <- get_api_data(url, request)

  return(data)
}

#' Get all the series for a given table
#'
#' @param idTable (int): id of the table. For further information about
#' ids click this [link](https://es-ine.github.io/ineapir/articles/identify_codes.html).
#' @param filter (list): list of variables and values.
#' ### Filtering data from tables
#' When we request data from tables there is the possibility of filtering data
#' on the fly using metadata information about the variables and their values
#' that define the series. See [get_metadata_table_varval()] to get all the values at once.
#' There are different approaches to build the filter depending on the table type.
#'
#' #### Case one: tempus table
#' [URL example](https://www.ine.es/jaxiT3/Tabla.htm?t=76125).
#' For a tempus table the filter is based on ids. The format is `list(id_variable1 = id_value1, id_variable2 = id_value2)`.
#' Besides:
#' - A variable can take more than one value: `list(id_variable1 = c(id_value11, id_value12), id_variable2 = id_value2)`.
#' - A variable can take a empty character "" to get all its possible values: `list(id_variable1 = id_value1, id_variable2 = "")`.
#'
#' #### Case two: px tables
#' [URL example](https://www.ine.es/jaxi/Tabla.htm?path=/t20/e245/p08/l0/&file=01001.px).
#' For a px table the filter is based on codes. The format is `list(cod_variable1 = cod_value1, cod_variable2 = cod_value2)`.
#' Besides:
#' - A variable can take more than one value: `list(cod_variable1 = c(cod_value11, cod_value12), id_variable2 = cod_value2)`.
#' - A variable can take a empty character "" to get all its possible values: `list(cod_variable1 = cod_value1, cod_variable2 = "")`.

#' #### Case three: tpx table
#' [URL example](https://www.ine.es/jaxi/Tabla.htm?tpx=33387&L=0).
#' For a tpx table the filter is based on codes. The format is `list(cod_variable1 = cod_value1, cod_variable2 = cod_value2)`.
#' Besides:
#' - A variable can take more than one value: `list(cod_variable1 = c(cod_value11, cod_value12), id_variable2 = cod_value2)`.
#' - A variable can take a empty character "" to get all its possible values: `list(cod_variable1 = cod_value1, cod_variable2 = "")`.
#'
#' [URL example](https://www.ine.es/jaxi/Tabla.htm?tpx=52056&L=0).
#' There are tpx tables that contain variable ids and value ids. In this case,
#' we can use the ids instead of the codes to build the filter. To do this we add
#' the alias *~id* at the end of each id: `list(id_variable1~id = id_value1~id, id_variable2~id = id_value2~id)`.
#'
#' #### Using shortcuts
#' Additionally, shortcuts can be used to filter. They simplify the filtering
#' approach by using standardized names for variable IDs and therefore simplify
#' their use. The format for a tempus table is:
#' `list(shortcut_variable1 = name1, shortcut_variable2 = name2)`. However,
#' for px and tpx tables the format is: `list(values = c(name1, name2)`. The
#' *values* wrapper can also be used with tempus tables.
#' To see a list of all available shortcuts, see [get_filter_shortcuts()] function.
#' Let’s also remark that for better performance is recommended to use numeric ids
#' for tempus tables and alphanumeric codes for px and tpx tables.
#' @param det (int): level of detail. Valid values: 0, 1 or 2.
#' @param tip (string): set to 'A' for friendly output (e.g. readable dates),
#' set to 'M' to include metadata or set to 'AM' for both.
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param validate (logical): validate input parameters. A FALSE value means fewer API calls.
#' Therefore, it is recommended to set it to FALSE when there is no doubt about the validity of the input parameters, including the filter.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#' @param metanames (logical): set to TRUE to extract the name of the values that defined the table.
#' The names are extracted from the metadata information (it is mandatory to include 'M' in the tip parameter).
#' Several columns are created corresponding to the values of the different variables.
#' @param metacodes (logical): set to TRUE to extract the codes and ids of the values that defined the table.
#' The codes and ids are extracted from the metadata information (it is mandatory to include 'M' in the tip parameter).
#' Several columns are created corresponding to the values of the different variables.
#'
#' @return Data frame with information of the series for a given table.
#'
#' @examplesIf interactive()
#' # Get time series without data from table with identification code "76125"
#' filter <- list("3" = "83")
#' df <- get_metadata_series_table(idTable = 76125, validate = FALSE,
#' filter = filter)
#' head(df)
#'
#' # Get metadata as well
#' df <- get_metadata_series_table(idTable = 76125, validate = FALSE,
#' filter = filter, metanames = TRUE, metacodes = TRUE, tip = "M")
#' head(df)
#'
#' @export
get_metadata_series_table <- function(idTable = NULL, filter = NULL, det = 0, tip = NULL, lang = "ES", validate = TRUE, verbose = FALSE, metanames = FALSE, metacodes = FALSE){

  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, list(fun = "SERIES_TABLA"))
  definition <- append(definition, list(input = idTable))
  definition <- append(definition, list(tag = "idTable"))

  # List of parameters to call the API
  parameters <- list()
  parameters <- append(parameters, if(is.null(filter)) list(filter = filter) else list(filter = list(idTable = idTable, filter = filter)))
  parameters <- append(parameters, list(det = det))
  parameters <- append(parameters, list(tip = tip))

  # List of addons
  addons <- list(validate = validate, verbose = verbose, metanames = metanames, metacodes = metacodes)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Build the complete URL to call the API
  url <- get_url(request)

  # Obtain the retrieved data calling the API
  data <- get_api_data(url, request)

  return(data)
}

#' Get all the series for a specific operation given a filter
#'
#' @param operation (string): code of the operation. To obtain a list of
#' available operations see [get_metadata_operations()].
#' @param filter (list): list of variables and values.
#' ### Filtering data from series
#' When we request data from series there is the possibility of filtering data
#' on the fly using metadata information about the variables and their values
#' that define the series. To get variables for a given operation see
#' [get_metadata_variables()] and to get values for a specific variable see
#' [get_metadata_values()]. See also [get_metadata_series_varval()] to get all the values at once.
#'
#' #### Filter format
#' The format is `list(id_variable1 = id_value1, id_variable2 = id_value2)`.
#' Besides:
#' - A variable can take more than one value: `list(id_variable1 = c(id_value11, id_value12), id_variable2 = id_value2)`.
#' - A variable can take a empty character "" to get all its possible values: `list(id_variable1 = id_value1, id_variable2 = "")`.
#'
#' #### Using shortcuts
#' Additionally, shortcuts can be used to filter. They simplify the filtering
#' approach by using standardized names for variable IDs and therefore simplify
#' their use. The format is:
#' `list(shortcut_variable1 = name1, shortcut_variable2 = name2)`. Besides,
#' the *values* wrapper can also be used: `list(values = c(name1, name2)`.
#' To see a list of all available shortcuts, see [get_filter_shortcuts()] function.
#' Let’s also remark that for better performance is recommended to use numeric codes.
#' @param periodicity (int): id of the periodicity of the series. Common periodicities:
#' 1 (monthly), 3 (quarterly), 6 (bi-annual), 12 (annual). To obtain a list
#' of periodicities see [get_metadata_periodicity()].
#' @param det (int): level of detail. Valid values: 0, 1 or 2.
#' @param tip (string): set to 'A' for friendly output (e.g. readable dates),
#' set to 'M' to include metadata or set to 'AM' for both.
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param page (int): page number. The retrieved result of the query is paginated. Default value is set to 1.
#' @param validate (logical): validate input parameters. A FALSE value means fewer API calls.
#' Therefore, it is recommended to set it to FALSE when there is no doubt about the validity of the input parameters, including the filter.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#'
#' @return Data frame with information of the series according to the operation and filter specified in the function
#'
#' @examplesIf interactive()
#' # Get time series from "IPC" operation that include the filter variables
#' # and values
#' df <- get_metadata_series_filter(operation = "IPC", periodicity = 1,
#' filter = list("115"= "29", "3" = "84", "762" = ""), validate = FALSE)
#' head(df)
#'
#' @export
get_metadata_series_filter <- function(operation = NULL, filter = NULL, periodicity = NULL, det = 0, tip = NULL, lang = "ES", page = 1, validate = TRUE, verbose = FALSE){

  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, list(fun = "SERIE_METADATAOPERACION"))
  definition <- append(definition, list(input = operation))
  definition <- append(definition, list(tag = "operation"))

  # List of parameters to call the API
  parameters <- list()
  parameters <- append(parameters, list(filter = list(operation = operation, filter = filter)))
  parameters <- append(parameters, list(p = list(operation = operation, p = periodicity)))
  parameters <- append(parameters, list(det = det))
  parameters <- append(parameters, list(tip = tip))
  parameters <- append(parameters, if(page == 0) list(page = 1) else list(page = page))

  # List of addons
  addons <- list(validate = validate, verbose = verbose)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Build the complete URL to call the API
  url <- get_url(request)

  # Obtain the data retrieved calling the API
  data <- get_api_data(url, request)

  return(data)
}

#' Get metadata information about the variables and values of series for a given operation
#'
#' @param operation (string): code of the operation. To obtain a list of
#' available operations see [get_metadata_operations()].
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param det (int): level of detail. Valid values: 0, 1 or 2.
#' @param validate (logical): validate input parameters. A FALSE value means fewer API calls.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#'
#' @return Data frame with information about the variables and values that
#' define the series according to the operation specified in the function
#'
#' @examplesIf interactive()
#' # Get metadata information of time series from "IPC" operation
#' df <- get_metadata_series_varval(operation = "IPC", validate = FALSE)
#' head(df)
#'
#' @export
get_metadata_series_varval <- function(operation = NULL, lang = "ES", det = 0, validate = TRUE, verbose = FALSE){
  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, list(input = operation))
  definition <- append(definition, list(tag = "operation"))

  # List of parameters to call the API
  parameters <- list()
  parameters <- append(parameters, list(det = det))

  # List of addons
  addons <- list(validate = validate, verbose = verbose)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)
  # Get the metadata information of the table
  df <- get_metadata_variable_values_operation(operation, verbose, validate, lang, det = det, request = request)

  return(df$values)
}

test_that("get_metadata_series", {
  skip_on_cran()
  expect_type(get_metadata_series(codSeries = "IPC290750"), "list")
})

test_that("test get_metadata_series_operation", {
  skip_on_cran()
  expect_s3_class(get_metadata_series_operation(operation = "IPC", validate = FALSE), "data.frame")
})

test_that("test get_metadata_series_values", {
  skip_on_cran()
  expect_s3_class(get_metadata_series_values(codSeries = "IPC290750"), "data.frame")
  expect_s3_class(get_metadata_series_values(codSeries = "IPC290750", det = 2), "data.frame")
})

test_that("test get_metadata_series_table", {
  skip_on_cran()
  expect_s3_class(get_metadata_series_table(idTable = 76125, filter = list("3" = "74", "762" = "304092"),
                                                     metanames = TRUE, metacodes = TRUE, tip = "M"), "data.frame")
})

test_that("test get_metadata_series_filter", {
  skip_on_cran()
  expect_s3_class(get_metadata_series_filter(operation = "IPC", periodicity = 1, validate = FALSE,
                                                      filter = list("115"= "28", "3" = "84", "762" = "304092")), "data.frame")
})

test_that("test get_metadata_series_varval", {
  skip_on_cran()
  expect_s3_class(get_metadata_series_varval(operation = "IPC", validate = FALSE), "data.frame")
  expect_s3_class(get_metadata_series_varval(operation = "IPC", det = 2, validate = FALSE), "data.frame")
})

#' Get all available operations
#'
#' @param operation (string): code of the operation. To obtain a list of
#' available operations see [get_metadata_operations()].
#' If no operation is specified then all the operations will be shown
#' @param lang  (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param geo (int): set to 0 for operations with national data or set to 1 for operations with data with
#' a greater level of disaggregation.
#' @param page (int): page number. The retrieved result of the query is paginated (page=0 retrieves all pages).
#' @param validate (logical): validate input parameters. A FALSE value means fewer API calls.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#'
#' @return Data frame with information of the available operations
#'
#' @examplesIf interactive()
#' # Get all operations
#' df <- get_metadata_operations()
#' head(df)
#'
#' # Get a specific operation
#' df <- get_metadata_operations(operation = "IPC", validate = FALSE)
#' head(df)
#'
#' # Get operations with territorial disaggregation
#' df <- get_metadata_operations(geo = 1)
#' head(df)
#'
#' @export
get_metadata_operations <- function(operation = NULL, lang = "ES", geo = NULL, page = 0, validate = TRUE, verbose = FALSE){
  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, if(is.null(operation)) list(fun = "OPERACIONES_DISPONIBLES") else list(fun = "OPERACION"))
  definition <- append(definition, list(input = operation))
  definition <- append(definition, list(tag = "operation_active_null"))

  # List of parameters to call the API
  parameters <- list()
  parameters <- append(parameters, list(geo = geo))
  parameters <- append(parameters, list(page = page))

  # List of addons
  addons <- list(validate = validate, verbose = verbose)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Build the complete URL to call the API
  url <- get_url(request)

  # Obtain the retrieved data calling the API
  data <- get_api_data_all_pages(url, request)

  return(data)
}

#' Get all available variables
#'
#' @param operation (string): Code of the operation. Provide code to get all
#' the variables for the given operation. To obtain a list of
#' available operations see [get_metadata_operations()].
#' If no operation is specified then all the variables will be shown.
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param det (int): level of detail. Valid values: 0, 1 or 2.
#' @param page (int): page number. The retrieved result of the query is paginated (page=0 retrieves all pages).
#' @param validate (logical): validate input parameters. A FALSE value means fewer API calls.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#'
#' @return Data frame with information of the available variables
#'
#' @examplesIf interactive()
#' # Gel all variables
#' df <- get_metadata_variables()
#' head(df)
#'
#' # Get variables for a specific operation
#' df <- get_metadata_variables(operation = "IPC", validate = FALSE)
#' head(df)
#'
#' @export
get_metadata_variables <- function(operation = NULL, lang = "ES", det = 0, page = 0, validate = TRUE, verbose = FALSE){
  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, if(is.null(operation)) list(fun = "VARIABLES") else list(fun = "VARIABLES_OPERACION"))
  definition <- append(definition, list(input = operation))
  definition <- append(definition, list(tag = "operation_active_null"))

  # List of parameters to call the API
  parameters <- list()
  parameters <- append(parameters, list(det = det))
  parameters <- append(parameters, list(page = page))

  # List of addons
  addons <- list(validate = validate, verbose = verbose)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Build the complete URL to call the API
  url <- get_url(request)

  # Obtain the retrieved data calling the API
  data <- get_api_data_all_pages(url, request)

  return(data)
}

#' Get all values for a specific variable
#'
#' @param operation (string): code of the operation. Provide code to get all
#' the values for the given operation. To obtain a list of
#' available operations see [get_metadata_operations()].
#' @param variable (int): id of a variable. To obtain a list of
#' available variables see [get_metadata_variables()].
#' @param value (int): id of a value. If an id value is specified, the children
#' of the value are requested. To obtain a list of
#' available values for a variable use `get_metadata_values(variable = id_variable)`.
#' @param det (int): level of detail. Valid values: 0, 1 or 2.
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param page (int): page number. The retrieved result of the query is paginated (page=0 retrieves all pages).
#' @param classification (int): id of a classification. To obtain a list of available
#' classifications see [get_metadata_classifications()].
#' @param validate (logical): validate input parameters. A FALSE value means fewer API calls.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#' @param hierarchy (int): depth of the hierarchy tree.
#' @param filter (list): list of variables and values. When we request the hierarchy tree
#' there is the possibility of filtering using metadata information about the variables and their values
#' that define the series.
#' The format is `list(id_variable1 = id_value1, id_variable2 = id_value2)`. Besides:
#' - A variable can take more than one value: `list(id_variable1 = c(id_value11, id_value12), id_variable2 = id_value2)`.
#' - A variable can take a empty character "" to get all its possible values: `list(id_variable1 = id_value1, id_variable2 = "")`.
#'
#' @return Data frame with information of the available values for the variable specified in the function
#'
#' @examplesIf interactive()
#' # Get the values of the variable "115"
#' df <- get_metadata_values(variable = 115)
#' head(df)
#'
#' # Get the values of a variable for a specific operation
#' df <- get_metadata_values(operation = "IPC", variable = 115, validate = FALSE)
#' head(df)
#'
#' # Get the children of a value (provinces of Galicia)
#' # Variable: Autonomous communities (id=70)
#' # Value: Galicia (id=9008)
#' df <- get_metadata_values(variable = 70, value = 9008)
#' head(df)
#'
#' @export
get_metadata_values <- function(operation = NULL, variable =  NULL, value = NULL, det = 0, lang = "ES", page = 0, classification = NULL, validate = TRUE, verbose = FALSE, hierarchy = NULL, filter = NULL){
  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))

  if(is.null(operation)){
    if(is.null(value)){
      definition <- append(definition, list(fun = "VALORES_VARIABLE"))
    }else{
      definition <- append(definition, list(fun = "VALORES_HIJOS"))
    }
  }else{
    definition <- append(definition, list(fun = "VALORES_VARIABLEOPERACION"))
  }

  definition <- append(definition, if(is.null(operation)) list(input = list(variable = variable, value = value)) else list(input = list(variable = variable, operation = operation)))
  definition <- append(definition, list(tag = "variable_operation"))

  # List of parameters to call the API
  parameters <- list()
  parameters <- append(parameters, list(det = det))
  parameters <- append(parameters, list(page = page))
  parameters <- append(parameters, list(clasif = list(operation = operation, clasif = classification)))

  # List of addons
  addons <- list(validate = validate, verbose = verbose, hierarchy = hierarchy)
  addons <- append(addons, if(is.null(filter)) list(filter = filter) else list(filter = list(variable = variable, filter = filter)))

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Build the complete URL to call the API
  url <- get_url(request)

  # Obtain the retrieved data calling the API
  data <- NULL
  if(is.null(operation)){
    if(is.null(value)){
      data <- get_api_data_all_pages(url, request)
    }else{
      data <- get_api_data(url, request)
    }
  }else{
    data <- get_api_data_all_pages(url, request)
  }

  return(data)
}

#' Get all publications
#'
#' @param operation (string): code of the operation. Provide code to get all
#' the publications for the given operation. To obtain a list of
#' available operations see [get_metadata_operations()].
#' If no operation is specified then all the publications will be shown.
#' @param det (int): level of detail. Valid values: 0, 1 or 2.
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param page (int): page number. The retrieved result of the query is paginated (page=0 retrieves all pages).
#' @param validate (logical): validate input parameters. A FALSE value means fewer API calls.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#'
#' @return Data frame with information about publications
#'
#' @examplesIf interactive()
#' # Get all publications
#' df <- get_metadata_publications()
#' head(df)
#'
#' # Get publications for a specific operation
#' df <- get_metadata_publications(operation = "IPC", validate = FALSE)
#' head(df)
#'
#' @export
get_metadata_publications <- function(operation = NULL, det = 0, lang = "ES", page = 0, validate = TRUE, verbose = FALSE){
  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, if(is.null(operation)) list(fun = "PUBLICACIONES") else list(fun = "PUBLICACIONES_OPERACION"))
  definition <- append(definition, list(input = operation))
  definition <- append(definition, list(tag = "operation_active_null"))

  # List of parameters to call the API
  parameters <- list()
  parameters <- append(parameters, list(det = det))
  parameters <- append(parameters, list(page = page))

  # List of addons
  addons <- list(validate = validate, verbose = verbose)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Build the complete URL to call the API
  url <- get_url(request)

  # Obtain the retrieved data calling the API
  data <- get_api_data_all_pages(url, request)

  return(data)
}

#' Get the dates of a publication
#'
#' @param publication (int): id of the publication. To obtain a list of
#' available publications see [get_metadata_publications()].
#' @param det (int): level of detail. Valid values: 0, 1 or 2.
#' @param tip (string): set to 'A' for friendly output (e.g. readable dates),
#' set to 'M' to include metadata or set to 'AM' for both.
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param page (int): page number. The retrieved result of the query is paginated (page=0 retrieves all pages).
#' @param validate (logical): validate input parameters. A FALSE value means fewer API calls.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#'
#' @return Data frame with information of the dates of the publication specified in the function
#'
#' @examplesIf interactive()
#' # Get the dates of a publication
#' df <- get_metadata_publication_dates(publication = 8, validate = FALSE)
#' head(df)
#'
#' @export
get_metadata_publication_dates <- function(publication = NULL, det = 0, tip = NULL, lang = "ES", page = 0, validate = TRUE, verbose = FALSE){
  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, list(fun = "PUBLICACIONFECHA_PUBLICACION"))
  definition <- append(definition, list(input = publication))
  definition <- append(definition, list(tag = "publication"))

  # List of parameters to call the API
  parameters <- list()
  parameters <- append(parameters, list(det = det))
  parameters <- append(parameters, list(tip = tip))
  parameters <- append(parameters, list(page = page))

  # List of addons
  addons <- list(validate = validate, verbose = verbose)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Build the complete URL to call the API
  url <- get_url(request)

  # Obtain the retrieved data calling the API
  data <- get_api_data_all_pages(url, request)

  return(data)
}

#' Get all available periodicities
#'
#' @param operation (string): Code of the operation. Provide code to get all
#' the periodicities for the given operation. To obtain a list of
#' available operations see [get_metadata_operations()].
#' If no operation is specified then all the periodicities will be shown.
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param validate (logical): validate input parameters. A FALSE value means fewer API calls.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#'
#' @return Data frame with information of the available periodicities
#'
#' @examplesIf interactive()
#' # Get all periodicities
#' df <- get_metadata_periodicity()
#' head(df)
#'
#' # Get periodicities for a specific operation
#' df <- get_metadata_periodicity(operation = "IPC", validate = FALSE)
#' head(df)
#'
#' @export
get_metadata_periodicity <- function(operation = NULL, lang = "ES", validate = TRUE, verbose = FALSE){
  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, if(is.null(operation)) list(fun = "PERIODICIDADES") else list(fun = "PERIODICIDAD_OPERACION"))
  definition <- append(definition, list(input = operation))
  definition <- append(definition, list(tag = "operation_active_null"))

  # List of parameters to call the API
  parameters <- list()

  # List of addons
  addons <- list(validate = validate, verbose = verbose)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Build the complete URL to call the API
  url <- get_url(request)

  # Obtain the retrieved data calling the API
  data <- get_api_data(url, request)

  return(data)
}

#' Get all available filter shortcuts
#'
#' @param lang (string): language. Set to 'ES' for the Spanish version of the
#' shortcuts or set to 'EN' for the English version of the shortcuts.
#' @param validate (logical): validate input parameters.
#' @param verbose (logical): print additional information.
#'
#' @return Data frame with information of the available filter shortcuts
#'
#' @examples
#' # Shortcuts in spanish
#' df <- get_filter_shortcuts()
#' head(df)
#'
#' # Shortcuts in english
#' df <- get_filter_shortcuts(lang = "EN")
#' head(df)
#'
#' @export
get_filter_shortcuts <- function(lang = "ES", validate = TRUE, verbose = FALSE){

  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))

  # List of parameters to call the API
  parameters <- list()

  # List of addons
  addons <- list(validate = validate, verbose = verbose)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  short <- names(shortcuts_filter)
  varid <- character()
  comment <- character()
  language <- character()

  for (s in short){
    varid <- append(varid, paste0(shortcuts_filter[[s]], collapse = ", "))
    comment <- append(comment, shortcuts_filter_comments[[s]]$comment)
    language <- append(language, shortcuts_filter_comments[[s]]$lang)
  }

  # Add values wrapper
  short <- append(short, "values")
  varid <- append(varid, "-")
  comment <- append(comment, "Values wrapper")
  language <- append(language, "ALL")

  # Select language
  df <- data.frame(Shortcut = short, "Variable.ID" = varid, Comment = comment, language = language)

  # Select columns
  df <- subset(df, language %in% c(lang, "ALL"), select = c("Shortcut", "Variable.ID", "Comment"))

  return(df)
}

#' Get all available classifications
#'
#' @param operation (string): Code of the operation. Provide code to get all
#' the classifications for the given operation. To obtain a list of
#' available operations see [get_metadata_operations()].
#' If no operation is specified then all the classifications will be shown.
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param validate (logical): validate input parameters. A FALSE value means fewer API calls.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#'
#' @return Data frame with information of the available classifications
#'
#' @examplesIf interactive()
#' # Get all classifications
#' df <- get_metadata_classifications()
#' head(df)
#'
#' # Get classifications for a specific operation
#' df <- get_metadata_classifications(operation = "IPC", validate = FALSE)
#' head(df)
#'
#' @export
get_metadata_classifications <- function(operation = NULL, lang = "ES", validate = TRUE, verbose = FALSE){
  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, if(is.null(operation)) list(fun = "CLASIFICACIONES") else list(fun = "CLASIFICACIONES_OPERACION"))
  definition <- append(definition, list(input = operation))
  definition <- append(definition, list(tag = "operation_active_null"))

  # List of parameters to call the API
  parameters <- list()

  # List of addons
  addons <- list(validate = validate, verbose = verbose)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Build the complete URL to call the API
  url <- get_url(request)

  # Obtain the retrieved data calling the API
  data <- get_api_data(url, request)

  return(data)
}

test_that("test get_metadata_operations", {
  skip_on_cran()
  expect_s3_class(get_metadata_operations(), "data.frame")
  expect_type(get_metadata_operations(operation = "IPC", validate = FALSE), "list")
})

test_that("test get_metadata_variables", {
  skip_on_cran()
  expect_s3_class(get_metadata_variables(), "data.frame")
  expect_s3_class(get_metadata_variables(operation = "IPC", validate = FALSE), "data.frame")
})

test_that("test get_metadata_values", {
  skip_on_cran()
  expect_s3_class(get_metadata_values(variable = 70, validate = FALSE), "data.frame")
  expect_s3_class(get_metadata_values(variable = 70, operation = "IPC", validate = FALSE), "data.frame")
  expect_s3_class(get_metadata_values(variable = 70, value = 8997, validate = FALSE), "data.frame")
})

test_that("test get_metadata_publications", {
  skip_on_cran()
  expect_s3_class(get_metadata_publications(), "data.frame")
  expect_s3_class(get_metadata_publications(operation = "IPC", validate = FALSE), "data.frame")
})

test_that("test get_metadata_publication_dates", {
  skip_on_cran()
  expect_s3_class(get_metadata_publication_dates(publication = 8, validate = FALSE), "data.frame")
})

test_that("test get_metadata_periodicity", {
  skip_on_cran()
  expect_s3_class(get_metadata_periodicity(), "data.frame")
  expect_s3_class(get_metadata_periodicity(operation = "IPC", validate = FALSE), "data.frame")
})

test_that("test get_filter_shortcuts", {
  expect_s3_class(get_filter_shortcuts(), "data.frame")
  expect_s3_class(get_filter_shortcuts(lang = "EN"), "data.frame")
})

test_that("test get_metadata_classifications", {
  skip_on_cran()
  expect_s3_class(get_metadata_classifications(), "data.frame")
  expect_s3_class(get_metadata_classifications(operation = "IPC", validate = FALSE), "data.frame")
})

#' Get data from a specific table
#'
#' @param idTable (int): id of the table. For further information about
#' ids click this [link](https://es-ine.github.io/ineapir/articles/identify_codes.html).
#' @param filter (list): list of variables and values.
#' ### Filtering data from tables
#' When we request data from tables there is the possibility of filtering data
#' on the fly using metadata information about the variables and their values
#' that define the series. See [get_metadata_table_varval()] to get all the values at once.
#' There are different approaches to build the filter depending on the table type.
#'
#' #### Case one: tempus table
#' [URL example](https://www.ine.es/jaxiT3/Tabla.htm?t=76125).
#' For a tempus table the filter is based on ids. The format is `list(id_variable1 = id_value1, id_variable2 = id_value2)`.
#' Besides:
#' - A variable can take more than one value: `list(id_variable1 = c(id_value11, id_value12), id_variable2 = id_value2)`.
#' - A variable can take a empty character "" to get all its possible values: `list(id_variable1 = id_value1, id_variable2 = "")`.
#'
#' #### Case two: px tables
#' [URL example](https://www.ine.es/jaxi/Tabla.htm?path=/t20/e245/p08/l0/&file=01001.px).
#' For a px table the filter is based on codes. The format is `list(cod_variable1 = cod_value1, cod_variable2 = cod_value2)`.
#' Besides:
#' - A variable can take more than one value: `list(cod_variable1 = c(cod_value11, cod_value12), id_variable2 = cod_value2)`.
#' - A variable can take a empty character "" to get all its possible values: `list(cod_variable1 = cod_value1, cod_variable2 = "")`.

#' #### Case three: tpx table
#' [URL example](https://www.ine.es/jaxi/Tabla.htm?tpx=33387&L=0).
#' For a tpx table the filter is based on codes. The format is `list(cod_variable1 = cod_value1, cod_variable2 = cod_value2)`.
#' Besides:
#' - A variable can take more than one value: `list(cod_variable1 = c(cod_value11, cod_value12), id_variable2 = cod_value2)`.
#' - A variable can take a empty character "" to get all its possible values: `list(cod_variable1 = cod_value1, cod_variable2 = "")`.
#'
#' [URL example](https://www.ine.es/jaxi/Tabla.htm?tpx=52056&L=0).
#' There are tpx tables that contain variable ids and value ids. In this case,
#' we can use the ids instead of the codes to build the filter. To do this we add
#' the alias *~id* at the end of each id: `list(id_variable1~id = id_value1~id, id_variable2~id = id_value2~id)`.
#'
#' #### Using shortcuts
#' Additionally, shortcuts can be used to filter. They simplify the filtering
#' approach by using standardized names for variable IDs and therefore simplify
#' their use. The format for a tempus table is:
#' `list(shortcut_variable1 = name1, shortcut_variable2 = name2)`. However,
#' for px and tpx tables the format is: `list(values = c(name1, name2)`. The
#' *values* wrapper can also be used with tempus tables.
#' To see a list of all available shortcuts, see [get_filter_shortcuts()] function.
#' Let’s also remark that for better performance is recommended to use numeric ids
#' for tempus tables and alphanumeric codes for px and tpx tables.
#' @param nlast (int): number of periods to retrieve. By default it retrieves all available periods.
#' @param dateStart (string): the initial date of the requested data. The required
#' format is yyyy/mm/dd. Additionally, dateStart can be a vector of dates, where
#' each date represents the start date of individual ranges where the end date should be found
#' at the same position in the dateEnd vector. If dateStart and dateEnd are equal,
#' the specified dates are retrieved. If no end date is entered,
#' all dates will be queried, from the corresponding start date to the last available period.
#' @param dateEnd (string): the end date of the requested data. The required
#' format is yyyy/mm/dd. Additionally, dateEnd can be a vector of dates, where
#' each date represents the end date of individual ranges where the initial date should be found
#' at the same position in the dateStart vector. The length of the dateEnd vector
#' must be less than or equal to the length of the dateStart vector.
#' @param det (int): level of detail. Valid values: 0, 1 or 2.
#' @param tip (string): set to 'A' for friendly output (e.g. readable dates),
#'  set to 'M' to include metadata or set to 'AM' for both.
#' @param lang (string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.
#' @param validate (logical): validate the input parameters. A FALSE value means fewer API calls.
#' Therefore, it is recommended to set it to FALSE when there is no doubt about the validity of the input parameters, including the filter.
#' @param verbose (logical): print additional information, including the URL to call the API service.
#' @param unnest (logical): set to TRUE to obtain a single data frame of data.
#' @param metanames (logical): set to TRUE to extract the name of the values that defined the table.
#' The names are extracted from the metadata information (it is mandatory to include 'M' in the tip parameter).
#' Several columns are created corresponding to the values of the different variables.
#' @param metacodes (logical): set to TRUE to extract the codes and ids of the values that defined the table.
#' The codes and ids are extracted from the metadata information (it is mandatory to include 'M' in the tip parameter).
#' Several columns are created corresponding to the values of the different variables.
#'
#' @return Data frame with data of a table according to the id specified in the function
#'
#' @examplesIf interactive()
#' # Obtaining the last two periods and filter data
#' filter <- list("3" = "74", "762" = "304092")
#' df <-get_data_table(idTable = 76125, nlast = 2, unnest = TRUE,
#' filter = filter, validate = FALSE)
#' head(df)
#'
#' # Get data for an open range date
#' df <- get_data_table(idTable = 76125, unnest = TRUE, tip= "A",
#' filter = filter, validate = FALSE,
#' dateStart = "2025/01/01")
#' head(df)
#'
#' # Get data for a single range data
#' df <- get_data_table(idTable = 76125, unnest = TRUE, tip= "A",
#' filter = filter, validate = FALSE,
#' dateStart = "2023/01/01", dateEnd = "2023/05/01")
#' head(df)
#'
#' # Get data for specific dates
#' df <- get_data_table(idTable = 76125, unnest = TRUE, tip= "A",
#' filter = filter, validate = FALSE,
#' dateStart = c("2023/01/01","2024/01/01"),
#' dateEnd = c("2023/01/01","2024/01/01"))
#' head(df)
#'
#' # Get data for multiple date ranges
#' df <- get_data_table(idTable = 76125, unnest = TRUE, tip= "A",
#' filter = filter, validate = FALSE,
#' dateStart = c("2023/01/01","2024/01/01"),
#' dateEnd = c("2023/03/01","2024/03/01"))
#' head(df)
#'
#' # Get medatada as well
#' df <- get_data_table(idTable = 76125, nlast = 2, unnest = TRUE,
#' filter = filter, validate = FALSE,
#' metanames = TRUE, metacodes = TRUE, tip = "M")
#' head(df)
#'
#' @export
get_data_table <- function(idTable = NULL, filter = NULL, nlast = NULL, dateStart = NULL, dateEnd = NULL, det = 0, tip = NULL, lang = "ES", validate = TRUE, verbose = FALSE, unnest = FALSE, metanames = FALSE, metacodes = FALSE){

  # List of values to define the call to the API
  definition <- list()
  definition <- append(definition, list(lang = lang))
  definition <- append(definition, list(fun = "DATOS_TABLA"))
  definition <- append(definition, list(input = idTable))
  definition <- append(definition, list(tag = "idTable"))

  # List of parameters to call the API
  parameters <- list()
  parameters <- append(parameters, if(is.null(filter)) list(filter = filter) else list(filter = list(idTable = idTable, filter = filter)))
  parameters <- append(parameters, if(is.null(dateStart) & is.null(dateEnd)) list(date = dateStart) else list(date = list(dateStart = dateStart, dateEnd = dateEnd)))
  parameters <- append(parameters, if(is.null(dateStart) & is.null(dateEnd)) list(nult = nlast) else list(nult = NULL))
  #parameters <- append(parameters, list(nult = nlast))
  parameters <- append(parameters, list(det = det))
  parameters <- append(parameters, list(tip = tip))

  # List of addons
  addons <- list(validate = validate, verbose = verbose, unnest = unnest, metanames = metanames, metacodes = metacodes)

  # List of definitions and parameters
  request <- list(definition = definition, parameters = parameters, addons = addons)

  # Check request
  request <- check_request(request)

  # Build the complete URL to call the API
  url <- get_url(request)

  # Obtain the retrieved data calling the API
  data <- get_api_data(url, request)

  return(data)
}

test_that("test unnest", {
  skip_on_cran()
  expect_s3_class(get_data_table(idTable = 76125, unnest = TRUE, nlast = 2), "data.frame")
})


test_that("test metadata", {
  skip_on_cran()
  expect_s3_class(get_data_table(idTable = 76125, unnest = TRUE, nlast = 2, tip = "M",
                                          metanames = TRUE, metacodes = TRUE), "data.frame")

})

test_that("test filter", {
  skip_on_cran()
  expect_s3_class(get_data_table(idTable = 76125, unnest = TRUE, nlast = 2,
                                          filter <- list("3" = "74", "762" = "304092")), "data.frame")
  expect_s3_class(get_data_table(idTable = 76125, unnest = TRUE, nlast = 2, validate = FALSE,
                                          filter <- list("3" = "74", "762" = "304092")), "data.frame")
  expect_s3_class(get_data_table(idTable = 76125, unnest = TRUE, nlast = 2, validate = FALSE,
                                          filter <- list("3" = "-74", "762" = "304092")), "data.frame")
  expect_s3_class(get_data_table(idTable = 76125, unnest = TRUE, nlast = 2, validate = FALSE,
                                          filter <- list(values = c("variación anual" , "índice general"))), "data.frame")
})

test_that("test det", {
  skip_on_cran()
  expect_s3_class(get_data_table(idTable = 76125, unnest = TRUE, nlast = 2, det = 2), "data.frame")
  expect_s3_class(get_data_table(idTable = 76125, unnest = TRUE, nlast = 2, tip = "M", det = 2,
                                          metanames = TRUE, metacodes = TRUE), "data.frame")
  expect_s3_class(get_data_table(idTable = 76125, unnest = TRUE, nlast = 2,
                                          validate = FALSE, det = 2,
                                          filter <- list("3" = "-74", "762" = "304092")), "data.frame")
})

test_that("test date", {
  skip_on_cran()
  expect_s3_class(get_data_table(idTable = 76125, unnest = TRUE, nlast = 2, validate = FALSE,
                                          filter <- list("3" = "74", "762" = "304092"),
                                          dateStart = c("2023/01/01","2024/01/01"),
                                          dateEnd = c("2023/01/01","2024/01/01")), "data.frame")
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_series.R
\name{get_data_series}
\alias{get_data_series}
\title{Get data from a specific series}
\usage{
get_data_series(
  codSeries = NULL,
  nlast = 1,
  dateStart = NULL,
  dateEnd = NULL,
  det = 0,
  tip = NULL,
  lang = "ES",
  validate = TRUE,
  verbose = FALSE,
  unnest = FALSE
)
}
\arguments{
\item{codSeries}{(string): Code of the series. For further information about
codes click this \href{https://es-ine.github.io/ineapir/articles/identify_codes.html}{link}.}

\item{nlast}{(int): number of periods to retrieve. By default is set to 1 period.}

\item{dateStart}{(string): the initial date of the requested data. The required
format is yyyy/mm/dd. Additionally, dateStart can be a vector of dates, where
each date represents the start date of individual ranges where the end date should be found
at the same position in the dateEnd vector. If dateStart and dateEnd are equal,
the specified dates are retrieved. If no end date is entered,
all dates will be queried, from the corresponding start date to the last available period.}

\item{dateEnd}{(string): the end date of the requested data. The required
format is yyyy/mm/dd. Additionally, dateEnd can be a vector of dates, where
each date represents the end date of individual ranges where the initial date should be found
at the same position in the dateStart vector. The length of the dateEnd vector
must be less than or equal to the length of the dateStart vector.}

\item{det}{(int): level of detail. Valid values: 0, 1 or 2.}

\item{tip}{(string): set to 'A' for friendly output (e.g. readable dates),
set to 'M' to include metadata or set to 'AM' for both.}

\item{lang}{(string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.}

\item{validate}{(logical): validate input parameters.}

\item{verbose}{(logical): print additional information, including the URL to call the API service.}

\item{unnest}{(logical): set to TRUE to obtain a single data frame of data}
}
\value{
Data frame with data of a series according to the code specified in the function
}
\description{
Retrieve data from series published by INE calling the API
}
\examples{
\dontshow{if (interactive()) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
df <- get_data_series(codSeries = "IPC290750", nlast = 5)
head(df)

# Get data for an open range date
df <- get_data_series(codSeries = "IPC290750", dateStart = "2024/01/01")
head(df)

# Get data for a single range data
df <- get_data_series(codSeries = "IPC290750",
dateStart = "2023/01/01", dateEnd = "2023/05/01")
head(df)

# Get data for specific dates
df <- get_data_series(codSeries = "IPC290750",
dateStart = c("2023/01/01","2024/01/01"),
dateEnd = c("2023/01/01","2024/01/01"))
head(df)

# Get data for multiple date ranges
df <- get_data_series(codSeries = "IPC290750",
dateStart = c("2023/01/01","2024/01/01"),
dateEnd = c("2023/03/01","2024/03/01"))
head(df)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata.R
\name{get_metadata_classifications}
\alias{get_metadata_classifications}
\title{Get all available classifications}
\usage{
get_metadata_classifications(
  operation = NULL,
  lang = "ES",
  validate = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{operation}{(string): Code of the operation. Provide code to get all
the classifications for the given operation. To obtain a list of
available operations see \code{\link[=get_metadata_operations]{get_metadata_operations()}}.
If no operation is specified then all the classifications will be shown.}

\item{lang}{(string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.}

\item{validate}{(logical): validate input parameters. A FALSE value means fewer API calls.}

\item{verbose}{(logical): print additional information, including the URL to call the API service.}
}
\value{
Data frame with information of the available classifications
}
\description{
Get all available classifications
}
\examples{
\dontshow{if (interactive()) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
# Get all classifications
df <- get_metadata_classifications()
head(df)

# Get classifications for a specific operation
df <- get_metadata_classifications(operation = "IPC", validate = FALSE)
head(df)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata.R
\name{get_metadata_variables}
\alias{get_metadata_variables}
\title{Get all available variables}
\usage{
get_metadata_variables(
  operation = NULL,
  lang = "ES",
  det = 0,
  page = 0,
  validate = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{operation}{(string): Code of the operation. Provide code to get all
the variables for the given operation. To obtain a list of
available operations see \code{\link[=get_metadata_operations]{get_metadata_operations()}}.
If no operation is specified then all the variables will be shown.}

\item{lang}{(string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.}

\item{det}{(int): level of detail. Valid values: 0, 1 or 2.}

\item{page}{(int): page number. The retrieved result of the query is paginated (page=0 retrieves all pages).}

\item{validate}{(logical): validate input parameters. A FALSE value means fewer API calls.}

\item{verbose}{(logical): print additional information, including the URL to call the API service.}
}
\value{
Data frame with information of the available variables
}
\description{
Get all available variables
}
\examples{
\dontshow{if (interactive()) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
# Gel all variables
df <- get_metadata_variables()
head(df)

# Get variables for a specific operation
df <- get_metadata_variables(operation = "IPC", validate = FALSE)
head(df)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata_tables.R
\name{get_metadata_table_varval}
\alias{get_metadata_table_varval}
\title{Get metadata information about the variables and values for a given table}
\usage{
get_metadata_table_varval(
  idTable = NULL,
  det = 0,
  filter = NULL,
  lang = "ES",
  validate = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{idTable}{(int): id of the table. For further information about
ids click this \href{https://es-ine.github.io/ineapir/articles/identify_codes.html}{link}.}

\item{det}{(int): level of detail. Valid values: 0, 1 or 2.}

\item{filter}{(list): list of variables and values.
\subsection{Filtering data from tables}{

When we request data from tables there is the possibility of filtering data
on the fly using metadata information about the variables and their values
that define the series. There are different approaches to build the filter depending on the table type.
\subsection{Case one: tempus table}{

\href{https://www.ine.es/jaxiT3/Tabla.htm?t=76125}{URL example}.
For a tempus table the filter is based on ids. The format is \code{list(id_variable1 = id_value1, id_variable2 = id_value2)}.
Besides:
\itemize{
\item A variable can take more than one value: \code{list(id_variable1 = c(id_value11, id_value12), id_variable2 = id_value2)}.
\item A variable can take a empty character "" to get all its possible values: \code{list(id_variable1 = id_value1, id_variable2 = "")}.
}
}

\subsection{Case two: px tables}{

\href{https://www.ine.es/jaxi/Tabla.htm?path=/t20/e245/p08/l0/&file=01001.px}{URL example}.
For a px table the filter is based on codes. The format is \code{list(cod_variable1 = cod_value1, cod_variable2 = cod_value2)}.
Besides:
\itemize{
\item A variable can take more than one value: \code{list(cod_variable1 = c(cod_value11, cod_value12), id_variable2 = cod_value2)}.
\item A variable can take a empty character "" to get all its possible values: \code{list(cod_variable1 = cod_value1, cod_variable2 = "")}.
}
}

\subsection{Case three: tpx table}{

\href{https://www.ine.es/jaxi/Tabla.htm?tpx=33387&L=0}{URL example}.
For a tpx table the filter is based on codes. The format is \code{list(cod_variable1 = cod_value1, cod_variable2 = cod_value2)}.
Besides:
\itemize{
\item A variable can take more than one value: \code{list(cod_variable1 = c(cod_value11, cod_value12), id_variable2 = cod_value2)}.
\item A variable can take a empty character "" to get all its possible values: \code{list(cod_variable1 = cod_value1, cod_variable2 = "")}.
}

\href{https://www.ine.es/jaxi/Tabla.htm?tpx=52056&L=0}{URL example}.
There are tpx tables that contain variable ids and value ids. In this case,
we can use the ids instead of the codes to build the filter. To do this we add
the alias \emph{~id} at the end of each id: \verb{list(id_variable1~id = id_value1~id, id_variable2~id = id_value2~id)}.
}

}}

\item{lang}{(string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.}

\item{validate}{(logical): validate input parameters.A FALSE value means fewer API calls.
Therefore, it is recommended to set it to FALSE when there is no doubt about the validity of the input parameters, including the filter.}

\item{verbose}{(logical): print additional information, including the URL to call the API service.}
}
\value{
Data frame with information about the variables and values that define a table according to the table specified in the function
}
\description{
Get metadata information about the variables and values for a given table
}
\examples{
\dontshow{if (interactive()) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
# Get all the variable and values of the table with identification code "76125"
df <- get_metadata_table_varval(idTable = 76125)
head(df)

# Filter variables and values
df <- get_metadata_table_varval(idTable = 52056,
filter = list(NAC = "00"), validate = FALSE)
head(df)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata.R
\name{get_metadata_values}
\alias{get_metadata_values}
\title{Get all values for a specific variable}
\usage{
get_metadata_values(
  operation = NULL,
  variable = NULL,
  value = NULL,
  det = 0,
  lang = "ES",
  page = 0,
  classification = NULL,
  validate = TRUE,
  verbose = FALSE,
  hierarchy = NULL,
  filter = NULL
)
}
\arguments{
\item{operation}{(string): code of the operation. Provide code to get all
the values for the given operation. To obtain a list of
available operations see \code{\link[=get_metadata_operations]{get_metadata_operations()}}.}

\item{variable}{(int): id of a variable. To obtain a list of
available variables see \code{\link[=get_metadata_variables]{get_metadata_variables()}}.}

\item{value}{(int): id of a value. If an id value is specified, the children
of the value are requested. To obtain a list of
available values for a variable use \code{get_metadata_values(variable = id_variable)}.}

\item{det}{(int): level of detail. Valid values: 0, 1 or 2.}

\item{lang}{(string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.}

\item{page}{(int): page number. The retrieved result of the query is paginated (page=0 retrieves all pages).}

\item{classification}{(int): id of a classification. To obtain a list of available
classifications see \code{\link[=get_metadata_classifications]{get_metadata_classifications()}}.}

\item{validate}{(logical): validate input parameters. A FALSE value means fewer API calls.}

\item{verbose}{(logical): print additional information, including the URL to call the API service.}

\item{hierarchy}{(int): depth of the hierarchy tree.}

\item{filter}{(list): list of variables and values. When we request the hierarchy tree
there is the possibility of filtering using metadata information about the variables and their values
that define the series.
The format is \code{list(id_variable1 = id_value1, id_variable2 = id_value2)}. Besides:
\itemize{
\item A variable can take more than one value: \code{list(id_variable1 = c(id_value11, id_value12), id_variable2 = id_value2)}.
\item A variable can take a empty character "" to get all its possible values: \code{list(id_variable1 = id_value1, id_variable2 = "")}.
}}
}
\value{
Data frame with information of the available values for the variable specified in the function
}
\description{
Get all values for a specific variable
}
\examples{
\dontshow{if (interactive()) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
# Get the values of the variable "115"
df <- get_metadata_values(variable = 115)
head(df)

# Get the values of a variable for a specific operation
df <- get_metadata_values(operation = "IPC", variable = 115, validate = FALSE)
head(df)

# Get the children of a value (provinces of Galicia)
# Variable: Autonomous communities (id=70)
# Value: Galicia (id=9008)
df <- get_metadata_values(variable = 70, value = 9008)
head(df)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata_tables.R
\name{get_metadata_table_values}
\alias{get_metadata_table_values}
\title{Get all values for a specific table group}
\usage{
get_metadata_table_values(
  idTable = NULL,
  idGroup = NULL,
  det = 0,
  lang = "ES",
  validate = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{idTable}{(int): id of the table. For further information about
ids click this \href{https://es-ine.github.io/ineapir/articles/identify_codes.html}{link}.}

\item{idGroup}{(int): id of the group of variables. To get all groups
for a specific table see \code{\link[=get_metadata_table_groups]{get_metadata_table_groups()}}.}

\item{det}{(int): level of detail. Valid values: 0, 1 or 2.}

\item{lang}{(string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.}

\item{validate}{(logical): validate input parameters. A FALSE value means fewer API calls.}

\item{verbose}{(logical): print additional information, including the URL to call the API service.}
}
\value{
Data frame with information of the values of a table group according to the table and group specified in the function
}
\description{
Get all values for a specific table group
}
\examples{
\dontshow{if (interactive()) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
# Get the values of the group "155577" of the table with identification
# code "76125"
df <- get_metadata_table_values(idTable = 76125, idGroup = 155577, validate = FALSE)
head(df)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata_series.R
\name{get_metadata_series_table}
\alias{get_metadata_series_table}
\title{Get all the series for a given table}
\usage{
get_metadata_series_table(
  idTable = NULL,
  filter = NULL,
  det = 0,
  tip = NULL,
  lang = "ES",
  validate = TRUE,
  verbose = FALSE,
  metanames = FALSE,
  metacodes = FALSE
)
}
\arguments{
\item{idTable}{(int): id of the table. For further information about
ids click this \href{https://es-ine.github.io/ineapir/articles/identify_codes.html}{link}.}

\item{filter}{(list): list of variables and values.
\subsection{Filtering data from tables}{

When we request data from tables there is the possibility of filtering data
on the fly using metadata information about the variables and their values
that define the series. See \code{\link[=get_metadata_table_varval]{get_metadata_table_varval()}} to get all the values at once.
There are different approaches to build the filter depending on the table type.
\subsection{Case one: tempus table}{

\href{https://www.ine.es/jaxiT3/Tabla.htm?t=76125}{URL example}.
For a tempus table the filter is based on ids. The format is \code{list(id_variable1 = id_value1, id_variable2 = id_value2)}.
Besides:
\itemize{
\item A variable can take more than one value: \code{list(id_variable1 = c(id_value11, id_value12), id_variable2 = id_value2)}.
\item A variable can take a empty character "" to get all its possible values: \code{list(id_variable1 = id_value1, id_variable2 = "")}.
}
}

\subsection{Case two: px tables}{

\href{https://www.ine.es/jaxi/Tabla.htm?path=/t20/e245/p08/l0/&file=01001.px}{URL example}.
For a px table the filter is based on codes. The format is \code{list(cod_variable1 = cod_value1, cod_variable2 = cod_value2)}.
Besides:
\itemize{
\item A variable can take more than one value: \code{list(cod_variable1 = c(cod_value11, cod_value12), id_variable2 = cod_value2)}.
\item A variable can take a empty character "" to get all its possible values: \code{list(cod_variable1 = cod_value1, cod_variable2 = "")}.
}
}

\subsection{Case three: tpx table}{

\href{https://www.ine.es/jaxi/Tabla.htm?tpx=33387&L=0}{URL example}.
For a tpx table the filter is based on codes. The format is \code{list(cod_variable1 = cod_value1, cod_variable2 = cod_value2)}.
Besides:
\itemize{
\item A variable can take more than one value: \code{list(cod_variable1 = c(cod_value11, cod_value12), id_variable2 = cod_value2)}.
\item A variable can take a empty character "" to get all its possible values: \code{list(cod_variable1 = cod_value1, cod_variable2 = "")}.
}

\href{https://www.ine.es/jaxi/Tabla.htm?tpx=52056&L=0}{URL example}.
There are tpx tables that contain variable ids and value ids. In this case,
we can use the ids instead of the codes to build the filter. To do this we add
the alias \emph{~id} at the end of each id: \verb{list(id_variable1~id = id_value1~id, id_variable2~id = id_value2~id)}.
}

\subsection{Using shortcuts}{

Additionally, shortcuts can be used to filter. They simplify the filtering
approach by using standardized names for variable IDs and therefore simplify
their use. The format for a tempus table is:
\code{list(shortcut_variable1 = name1, shortcut_variable2 = name2)}. However,
for px and tpx tables the format is: \verb{list(values = c(name1, name2)}. The
\emph{values} wrapper can also be used with tempus tables.
To see a list of all available shortcuts, see \code{\link[=get_filter_shortcuts]{get_filter_shortcuts()}} function.
Let’s also remark that for better performance is recommended to use numeric ids
for tempus tables and alphanumeric codes for px and tpx tables.
}

}}

\item{det}{(int): level of detail. Valid values: 0, 1 or 2.}

\item{tip}{(string): set to 'A' for friendly output (e.g. readable dates),
set to 'M' to include metadata or set to 'AM' for both.}

\item{lang}{(string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.}

\item{validate}{(logical): validate input parameters. A FALSE value means fewer API calls.
Therefore, it is recommended to set it to FALSE when there is no doubt about the validity of the input parameters, including the filter.}

\item{verbose}{(logical): print additional information, including the URL to call the API service.}

\item{metanames}{(logical): set to TRUE to extract the name of the values that defined the table.
The names are extracted from the metadata information (it is mandatory to include 'M' in the tip parameter).
Several columns are created corresponding to the values of the different variables.}

\item{metacodes}{(logical): set to TRUE to extract the codes and ids of the values that defined the table.
The codes and ids are extracted from the metadata information (it is mandatory to include 'M' in the tip parameter).
Several columns are created corresponding to the values of the different variables.}
}
\value{
Data frame with information of the series for a given table.
}
\description{
Get all the series for a given table
}
\examples{
\dontshow{if (interactive()) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
# Get time series without data from table with identification code "76125"
filter <- list("3" = "83")
df <- get_metadata_series_table(idTable = 76125, validate = FALSE,
filter = filter)
head(df)

# Get metadata as well
df <- get_metadata_series_table(idTable = 76125, validate = FALSE,
filter = filter, metanames = TRUE, metacodes = TRUE, tip = "M")
head(df)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata_series.R
\name{get_metadata_series_values}
\alias{get_metadata_series_values}
\title{Get all the values for a given series}
\usage{
get_metadata_series_values(
  codSeries = NULL,
  det = 0,
  tip = NULL,
  lang = "ES",
  validate = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{codSeries}{(string): code of the series. For further information about
codes click this \href{https://es-ine.github.io/ineapir/articles/identify_codes.html}{link}.}

\item{det}{(int): level of detail. Valid values: 0, 1 or 2.}

\item{tip}{(string): set to 'A' for friendly output (e.g. readable dates),
set to 'M' to include metadata or set to 'AM' for both.}

\item{lang}{(string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.}

\item{validate}{(logical): validate input parameters.}

\item{verbose}{(logical): print additional information, including the URL to call the API service.}
}
\value{
Data frame with information of the values of a series according to the code specified in the function
}
\description{
Get all the values for a given series
}
\examples{
\dontshow{if (interactive()) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
# Get metadata of time series with code "IPC290750"
df <- get_metadata_series_values(codSeries = "IPC290750")
head(df)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata_series.R
\name{get_metadata_series_varval}
\alias{get_metadata_series_varval}
\title{Get metadata information about the variables and values of series for a given operation}
\usage{
get_metadata_series_varval(
  operation = NULL,
  lang = "ES",
  det = 0,
  validate = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{operation}{(string): code of the operation. To obtain a list of
available operations see \code{\link[=get_metadata_operations]{get_metadata_operations()}}.}

\item{lang}{(string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.}

\item{det}{(int): level of detail. Valid values: 0, 1 or 2.}

\item{validate}{(logical): validate input parameters. A FALSE value means fewer API calls.}

\item{verbose}{(logical): print additional information, including the URL to call the API service.}
}
\value{
Data frame with information about the variables and values that
define the series according to the operation specified in the function
}
\description{
Get metadata information about the variables and values of series for a given operation
}
\examples{
\dontshow{if (interactive()) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
# Get metadata information of time series from "IPC" operation
df <- get_metadata_series_varval(operation = "IPC", validate = FALSE)
head(df)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata.R
\name{get_metadata_publication_dates}
\alias{get_metadata_publication_dates}
\title{Get the dates of a publication}
\usage{
get_metadata_publication_dates(
  publication = NULL,
  det = 0,
  tip = NULL,
  lang = "ES",
  page = 0,
  validate = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{publication}{(int): id of the publication. To obtain a list of
available publications see \code{\link[=get_metadata_publications]{get_metadata_publications()}}.}

\item{det}{(int): level of detail. Valid values: 0, 1 or 2.}

\item{tip}{(string): set to 'A' for friendly output (e.g. readable dates),
set to 'M' to include metadata or set to 'AM' for both.}

\item{lang}{(string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.}

\item{page}{(int): page number. The retrieved result of the query is paginated (page=0 retrieves all pages).}

\item{validate}{(logical): validate input parameters. A FALSE value means fewer API calls.}

\item{verbose}{(logical): print additional information, including the URL to call the API service.}
}
\value{
Data frame with information of the dates of the publication specified in the function
}
\description{
Get the dates of a publication
}
\examples{
\dontshow{if (interactive()) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
# Get the dates of a publication
df <- get_metadata_publication_dates(publication = 8, validate = FALSE)
head(df)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata.R
\name{get_filter_shortcuts}
\alias{get_filter_shortcuts}
\title{Get all available filter shortcuts}
\usage{
get_filter_shortcuts(lang = "ES", validate = TRUE, verbose = FALSE)
}
\arguments{
\item{lang}{(string): language. Set to 'ES' for the Spanish version of the
shortcuts or set to 'EN' for the English version of the shortcuts.}

\item{validate}{(logical): validate input parameters.}

\item{verbose}{(logical): print additional information.}
}
\value{
Data frame with information of the available filter shortcuts
}
\description{
Get all available filter shortcuts
}
\examples{
# Shortcuts in spanish
df <- get_filter_shortcuts()
head(df)

# Shortcuts in english
df <- get_filter_shortcuts(lang = "EN")
head(df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata_series.R
\name{get_metadata_series_filter}
\alias{get_metadata_series_filter}
\title{Get all the series for a specific operation given a filter}
\usage{
get_metadata_series_filter(
  operation = NULL,
  filter = NULL,
  periodicity = NULL,
  det = 0,
  tip = NULL,
  lang = "ES",
  page = 1,
  validate = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{operation}{(string): code of the operation. To obtain a list of
available operations see \code{\link[=get_metadata_operations]{get_metadata_operations()}}.}

\item{filter}{(list): list of variables and values.
\subsection{Filtering data from series}{

When we request data from series there is the possibility of filtering data
on the fly using metadata information about the variables and their values
that define the series. To get variables for a given operation see
\code{\link[=get_metadata_variables]{get_metadata_variables()}} and to get values for a specific variable see
\code{\link[=get_metadata_values]{get_metadata_values()}}. See also \code{\link[=get_metadata_series_varval]{get_metadata_series_varval()}} to get all the values at once.
\subsection{Filter format}{

The format is \code{list(id_variable1 = id_value1, id_variable2 = id_value2)}.
Besides:
\itemize{
\item A variable can take more than one value: \code{list(id_variable1 = c(id_value11, id_value12), id_variable2 = id_value2)}.
\item A variable can take a empty character "" to get all its possible values: \code{list(id_variable1 = id_value1, id_variable2 = "")}.
}
}

\subsection{Using shortcuts}{

Additionally, shortcuts can be used to filter. They simplify the filtering
approach by using standardized names for variable IDs and therefore simplify
their use. The format is:
\code{list(shortcut_variable1 = name1, shortcut_variable2 = name2)}. Besides,
the \emph{values} wrapper can also be used: \verb{list(values = c(name1, name2)}.
To see a list of all available shortcuts, see \code{\link[=get_filter_shortcuts]{get_filter_shortcuts()}} function.
Let’s also remark that for better performance is recommended to use numeric codes.
}

}}

\item{periodicity}{(int): id of the periodicity of the series. Common periodicities:
1 (monthly), 3 (quarterly), 6 (bi-annual), 12 (annual). To obtain a list
of periodicities see \code{\link[=get_metadata_periodicity]{get_metadata_periodicity()}}.}

\item{det}{(int): level of detail. Valid values: 0, 1 or 2.}

\item{tip}{(string): set to 'A' for friendly output (e.g. readable dates),
set to 'M' to include metadata or set to 'AM' for both.}

\item{lang}{(string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.}

\item{page}{(int): page number. The retrieved result of the query is paginated. Default value is set to 1.}

\item{validate}{(logical): validate input parameters. A FALSE value means fewer API calls.
Therefore, it is recommended to set it to FALSE when there is no doubt about the validity of the input parameters, including the filter.}

\item{verbose}{(logical): print additional information, including the URL to call the API service.}
}
\value{
Data frame with information of the series according to the operation and filter specified in the function
}
\description{
Get all the series for a specific operation given a filter
}
\examples{
\dontshow{if (interactive()) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
# Get time series from "IPC" operation that include the filter variables
# and values
df <- get_metadata_series_filter(operation = "IPC", periodicity = 1,
filter = list("115"= "29", "3" = "84", "762" = ""), validate = FALSE)
head(df)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_series.R
\name{get_data_series_filter}
\alias{get_data_series_filter}
\title{Get data from series for a specific operation given a filter}
\usage{
get_data_series_filter(
  operation = NULL,
  filter = NULL,
  periodicity = NULL,
  nlast = 1,
  dateStart = NULL,
  dateEnd = NULL,
  det = 0,
  tip = NULL,
  lang = "ES",
  page = 1,
  validate = TRUE,
  verbose = FALSE,
  unnest = FALSE
)
}
\arguments{
\item{operation}{(string): Code of the operation. To obtain a list of
available operations see \code{\link[=get_metadata_operations]{get_metadata_operations()}}.}

\item{filter}{(list): list of variables and values.
\subsection{Filtering data from series}{

When we request data from series there is the possibility of filtering data
on the fly using metadata information about the variables and their values
that define the series. To get variables for a given operation see
\code{\link[=get_metadata_variables]{get_metadata_variables()}} and to get values for a specific variable see
\code{\link[=get_metadata_values]{get_metadata_values()}}. See also \code{\link[=get_metadata_series_varval]{get_metadata_series_varval()}} to get all the values at once.
\subsection{Filter format}{

The format is \code{list(id_variable1 = id_value1, id_variable2 = id_value2)}.
Besides:
\itemize{
\item A variable can take more than one value: \code{list(id_variable1 = c(id_value11, id_value12), id_variable2 = id_value2)}.
\item A variable can take a empty character "" to get all its possible values: \code{list(id_variable1 = id_value1, id_variable2 = "")}.
}
}

\subsection{Using shortcuts}{

Additionally, shortcuts can be used to filter. They simplify the filtering
approach by using standardized names for variable IDs and therefore simplify
their use. The format is:
\code{list(shortcut_variable1 = name1, shortcut_variable2 = name2)}. Besides,
the \emph{values} wrapper can also be used: \verb{list(values = c(name1, name2)}.
To see a list of all available shortcuts, see \code{\link[=get_filter_shortcuts]{get_filter_shortcuts()}} function.
Let’s also remark that for better performance is recommended to use numeric codes.
}

}}

\item{periodicity}{(int): id of the periodicity of the series. Common periodicities:
1 (monthly), 3 (quarterly), 6 (bi-annual), 12 (annual). To obtain a list
of periodicities see \code{\link[=get_metadata_periodicity]{get_metadata_periodicity()}}.}

\item{nlast}{(int): number of periods to retrieve. By default is set to 1 period.}

\item{dateStart}{(string): the initial date of the requested data. The required
format is yyyy/mm/dd. Additionally, dateStart can be a vector of dates, where
each date represents the start date of individual ranges where the end date should be found
at the same position in the dateEnd vector. If dateStart and dateEnd are equal,
the specified dates are retrieved. If no end date is entered,
all dates will be queried, from the corresponding start date to the last available period.}

\item{dateEnd}{(string): the end date of the requested data. The required
format is yyyy/mm/dd. Additionally, dateEnd can be a vector of dates, where
each date represents the end date of individual ranges where the initial date should be found
at the same position in the dateStart vector. The length of the dateEnd vector
must be less than or equal to the length of the dateStart vector.}

\item{det}{(int): level of detail. Valid values: 0, 1 or 2.}

\item{tip}{(string): set to 'A' for friendly output (e.g. readable dates),
set to 'M' to include metadata or set to 'AM' for both.}

\item{lang}{(string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.}

\item{page}{(int): page number. The retrieved result of the query is paginated. Default value is set to 1.}

\item{validate}{(logical): validate the input parameters. A FALSE value means fewer API calls.
Therefore, it is recommended to set it to FALSE when there is no doubt about the validity of the input parameters, including the filter.}

\item{verbose}{(logical): print additional information, including the URL to call the API service.}

\item{unnest}{(logical): obtain a single data frame of data}
}
\value{
Data frame with data of series according to the operation and filter specified in the function
}
\description{
Get data from series for a specific operation given a filter
}
\examples{
\dontshow{if (interactive()) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
# Get last five periods and filter data of time series from "IPC" operation
filter <- list("115"= "28", "3" = "84", "762" = "304092")
df <- get_data_series_filter(operation = "IPC", periodicity = 1,
nlast = 5, unnest = TRUE, validate = FALSE, filter = filter)
head(df)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata_tables.R
\name{get_metadata_table_groups}
\alias{get_metadata_table_groups}
\title{Get all groups for a specific a table}
\usage{
get_metadata_table_groups(
  idTable = NULL,
  lang = "ES",
  validate = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{idTable}{(int): id of the table. For further information about
ids click this \href{https://es-ine.github.io/ineapir/articles/identify_codes.html}{link}.}

\item{lang}{(string): language of the retrieved data. Set to 'ES' for Spanish or set to 'EN' for English.}

\item{validate}{(logical): validate input parameters.}

\item{verbose}{(logical): print additional information, including the URL to call the API service.}
}
\value{
Data frame with information of the groups according to the table specified in the function
}
\description{
Get all groups for a specific a table
}
\examples{
\dontshow{if (interactive()) (if (getRversion() >= "3.4") withAutoprint else force)(\{ # examplesIf}
# Get the groups of the table with identification code "76125"
df <- get_metadata_table_groups(idTable = 76125)
head(df)
\dontshow{\}) # examplesIf}
}
